package edu.uoc.pfc.business.interfaces;

import java.util.List;

import edu.uoc.pfc.business.exceptions.AccountLockedException;
import edu.uoc.pfc.business.exceptions.BusinessRulesViolationException;
import edu.uoc.pfc.business.exceptions.DataStoreException;
import edu.uoc.pfc.business.exceptions.DuplicatedFieldException;
import edu.uoc.pfc.business.exceptions.InvalidIdException;
import edu.uoc.pfc.business.exceptions.SendEmailException;
import edu.uoc.pfc.business.transfers.TOUser;

/**
 * <p>Interfaz que define todos los m&eacute;todos que la capa de negocio ofrece a la
 * capa de presentaci&oacute;n relacionados con la gesti&oacute;n de las cuentas
 * de los usuarios de la aplicaci&oacute;n.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0 
 */
public interface IBAccountManager 
{
	
	// -------- MTODOS PBLICOS Y ABSTRACTOS.
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado para crear una cuenta de usuario a 
	 * partir de la informaci&aocute;n recogida dentro del objeto <code>TOUser</code>
	 * que se le pasa como par&aacute;metro.</p>
	 * 
	 * @param user <code>TOUser</code> con la informaci&oacute;n para crear la cuenta.
	 * @return <code>TOUser</code> que contendr&aacute; la informaci&oacute;n completa
	 * de la cuenta del usuario una vez creada (incluir&aacute;, entre otras cosas, el
	 * identificador del usuario <i>id</i> dentro de la aplicaci&oacute;n).</p>
	 * @throws DataStoreException si se produce un error en el marco de persistencia.
	 * @throws DuplicatedFieldException si el login proporcionado est&aacute; repetido.
	 */
	public abstract TOUser createAccount(TOUser user) 
			throws DataStoreException, DuplicatedFieldException;
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado para cambiar el estado de la cuenta
	 * del usuario cuyo identificador se le pasa como parmetro. El sistema comprobar&aacute;
	 * que no la cuenta a cambiar no pertenece al &uacute;nico administrador del 
	 * sistema (siempre tiene que haber una cuenta de administrador activa).</p>
	 * 
	 * <p>Si el m&eeacute;todo se ejecuta correctamente el estado del usuario pasar&aacute;
	 * a ser el contrario del que ten&iacute;a antes de llamarlo. As&iacute; si el estado
	 * era "activo" pasar&aacute; a estar "inactivo" y viceversa.</p>
	 * 
	 * @param id_user <code>int</code> identificador del usuario.
	 * @return <code>TOUser</code> con la informaci&oacute;n del usuario una vez se le 
	 * ha cambiado el estado.
	 * @throws DataStoreException si se produce un error en el marco de persistencia.
	 * @throws InvalidIdException si el identificador de usuario que se ha recibido no
	 * pertenece a un usuario registrado en el sistema.
	 * @throws BusinessRulesViolationException si se intenta cambiar el estado del &uacute;nico
	 * administrador activo dentro de la aplicaci&oacute;n.
	 */
	public abstract TOUser changeState(int id_user) 
			throws DataStoreException, InvalidIdException, BusinessRulesViolationException;
	
	/**
	 * <p>Borra del sistema la cuenta de usuario cuyo identificador se pasa en el par&aacute;metro
	 * <i>id</i>. El sistema comprobar&aacute; que se trata de un identificador v&aacute;lido 
	 * (pertenece a una cuenta) y adem&aacute;s que no es el identificador del &uacute;nico 
	 * administrador registrado.</p>
	 * 
	 * @param id_user <code>int</code> identificador del usuario.
	 * @throws DataStoreException si se produce un error en el marco de persistencia.
	 * @throws InvalidIdException si el identificador de usuario que se ha recibido no
	 * pertenece a un usuario registrado en el sistema.
	 * @throws BusinessRulesViolationException si se intenta borrar la cuenta del &uacute;nico 
	 * administrador registrador en el sistema.
	 */
	public abstract void deleteAccount(int id_user) 
			throws DataStoreException, InvalidIdException, BusinessRulesViolationException;
	
	/**
	 * <p>Env&iacute;a a la cuenta de correo electr&oacute;nico que se le ha pasado 
	 * como par&aacute;metro un email inform&aacute;ndole del nombre de usuario que 
	 * tiene en el sistema. En el caso de que la direcci&oacute;n de correo no corresponda
	 * con una cuenta del sistema este m&eeacute;todo no realizar&aacute; nada.</p>
	 * 
	 * @param email <code>String</code> direcci&oacute;n de correo electr&oacute;nico del 
	 * usuario para el cual se quiere recuperar su cuenta en el sistema.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de acceder
	 * a la capa de persistencia para obtener la informaci&oacute;n.
	 * @throws BusinessRulesViolationException si se intenta recuperar el nombre de usuario
	 * pero est&aacute; deshabilitado en env&iacute;o de correos electr&oacute;nicos dentro
	 * de la aplicaci&oacute;n.
	 * @throws SendEmailException si se produce un error al enviar el correo electr&oacute;nico
	 * con la informaci&oacute;n del nombre de usuario.
	 */
	public abstract void forgotLogin(String email) 
			throws DataStoreException, BusinessRulesViolationException, SendEmailException;
	
	/**
	 * <p>Env&iacute;a a la cuenta de correo electr&oacute;nico que se le ha pasado 
	 * como par&aacute;metro un email inform&aacute;ndole de la contrase&ntilde;a que 
	 * tiene en el sistema. En el caso de que la direcci&oacute;n de correo no corresponda
	 * con una cuenta del sistema este m&eeacute;todo no realizar&aacute; nada.</p>
	 * 
	 * @param email <code>String</code> direcci&oacute;n de correo electr&oacute;nico del 
	 * usuario para el cual se quiere recuperar su contrase&ntilde;a en el sistema.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de acceder
	 * a la capa de persistencia para obtener la informaci&oacute;n.
	 * @throws BusinessRulesViolationException si se intenta recuperar la contrase&ntilde;a
	 * pero est&aacute; deshabilitado en env&iacute;o de correos electr&oacute;nicos dentro
	 * de la aplicaci&oacute;n.
	 * @throws SendEmailException si se produce un error al enviar el correo electr&oacute;nico
	 * con la informaci&oacute;n de la contrase&ntilde;a.
	 */
	public abstract void forgotPassword(String email)
			throws DataStoreException, BusinessRulesViolationException, SendEmailException;
	
	/**
	 * <p>Retorna un objeto <code>TOUser</code> que contendr&aacute; la informaci&aocute;n de
	 * la cuenta de un usuario cuyo identificador se le ha pasado como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> el identificador del usuario.
	 * @return <code>TOUser</code> con la inforamci&oacute;n de la cuenta del usuario cuyo
	 * <i>id</i> es igual al que se ha recibido como par&aacute;metro.
	 * @throws DataStoreException si se produce una excepci&oacute;n al obtener la informaci&oacute;n
	 * de la capa de persistencia.
	 * @throws InvalidIdException si el identificador que se ha pasado no corresponde con el
	 * de ninguna de las cuentas de usuarios registradas en el sistema.
	 */
	public abstract TOUser getAccount(int id) throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado por un administrador del sistema cuando desee
	 * modificar la informa&oacute;n de la cuenta de usuario cuyo identificador se le pasa 
	 * en el par&aacute;metro <i>id</i>.</p>
	 * 
	 * @param id <code>int</code> identificador de la cuenta de usuario.
	 * @param password <code>String</code> nueva contrase&ntilde;a para la cuenta.
	 * @param isAdmin <code>boolean</code> indica si es un administrador o no.
	 * @param isTeacher <code>boolean</code> indica si es un profesor o no.
	 * @param isStudent <code>boolean</code> indica si es un alumno o no.
	 * @param active <code>boolean</code> indica si la cuenta est&aacute; activa o no.
	 * @param name <code>String</code> nombre real del usuario.
	 * @param surname <code>String</code> apellidos del usuario.
	 * @return <code>TOUser</code> con toda la informaci&aocute;n de la cuenta del usuario
	 * ya actualizada.
	 * @throws DataStoreException si se produce un error a la hora de realizar la operaci&oacute;n
	 * dentro del marco de persistencia.
	 * @throws InvalidIdException si el identificador que se ha pasado no corresponde con el
	 * de ninguna de las cuentas de usuarios registradas en el sistema.
	 * @throws BusinessRulesViolationException si se viola alguna regla de negocio de la
	 * aplicaci&oacute; si se permitiese realizar la operaci&oacute;n.
	 */
	public abstract TOUser modifyAccount(int id, String password, boolean isAdmin, boolean isTeacher,
			boolean isStudent, boolean active, String name, String surname) 
			throws DataStoreException, InvalidIdException, BusinessRulesViolationException; 
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado cuando se quiera modificar al informaci&aocute;n
	 * de la cuenta correspondiente al usuario <i>user</i> con los datos que se pasan como
	 * par&aacute;metros.</p>
	 * 
	 * @param user <code>TOUser</code> el usurio a modificar.
	 * @param password <code>String</code> la nueva contrase&ntilde;a de la cuenta.
	 * @param name <code>String</code> el nombre real del usuario.
	 * @param surname <code>String</code> los apellidos del usuario.
	 * @return <code>TOUser</code> con la informaci&oacute;n modificada del usuario.
	 * @throws DataStoreException si se produce un error a la hora de realizar la operaci&oacute;n
	 * dentro del marco de persistencia.
	 * @throws InvalidIdException si el objeto <code>TOUser</code> que se ha pasado como parmetro
	 * no corresponde con nunguna de las cuentas registradas dentro del sistema.
	 */
	public abstract TOUser modifyAccount(TOUser user, String password, 
			String name, String surname)
			throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>Retorna un objeto <code>List</code> formada por objetos de tipo <code>TOUser</code>
	 * donde cada uno de los elementos contendr&aacute; la informaci&oacute;n de una cuenta
	 * de usuario que est&aacute; registrada en el sistema (independientemente del estado
	 * en el que se encuentre).</p>
	 * 
	 * @return <code>List&lt;TOUser&gt;</code>
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de acceder
	 * a la capa de persistencia para obtener la informac&oacute;n.
	 */
	public abstract List<TOUser> listAccount() throws DataStoreException;
	
	/**
	 * <p>M&eeacute;to que ser&aacute; llamado desde la capa de presentaci&oacute;n en el
	 * momento en que un usuario quiera acceder a su cuenta dentro del sistema.</p>
	 * 
	 * @param login <code>String</code> nombre del usuario (login).
	 * @param password <code>String</code> contrase&ntilde;a de acceso.
	 * @return <code>TOUser</code> con la informaci&oacute;n del usuario dentro del sistema.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de acceder
	 * a la capa de persistencia para obtener la informac&oacute;n.
	 * @throws InvalidIdException si los datos proporcionados no corresponden con ninguna
	 * de las cuentas del sistema.
	 * @throws AccountLockedException si la cuenta del usuario no est&aacute; activa.
	 */
	public abstract TOUser signin(String login, String password) 
			throws DataStoreException, InvalidIdException, AccountLockedException;
	
	/**
	 * <p>M&eeacute;todo que ser&acute; llamado desde la capa de presentaci&oacute;n 
	 * en el momento en que un usuario desee salir de su cuenta.</p>
	 * 
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora
	 * de acceder a la capa de persistencia para obtener la informacin.
	 */
	public abstract void signout() throws DataStoreException;
	
	/**
	 * <p>Retorna un objeto <code>List</code> formada por objetos de tipo <code>TOUser</code>
	 * donde cada uno de los elementos contendr&aacute; la informaci&oacute;n de una cuenta
	 * de usuario que est&aacute; registrada en el sistema (independientemente del estado
	 * en el que se encuentre).</p>
	 * 
	 * @return <code>List&lt;TOUser&gt;</code>
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de acceder
	 * a la capa de persistencia para obtener la informac&oacute;n.
	 */
	public abstract List<TOUser> getAccounts() throws DataStoreException;
	
	/**
	 * <p>Retonar una lista de objetos <code>TOUser</code> donde cada uno de ellos
	 * representa a un usuario registrado como un Profesor dentro de la aplicaci&oacute;n.</p>
	 * 
	 * @return <code>List&lt;TOUser&gt;</code> con todos los Profesores.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de acceder
	 * a la capa de persistencia para obtener la informac&oacute;n.
	 */
	public abstract List<TOUser> getTeachers() throws DataStoreException;
	
	/**
	 * <p>Retonar una lista de objetos <code>TOUser</code> donde cada uno de ellos
	 * representa a un usuario registrado como un Alumno dentro de la aplicaci&oacute;n.</p>
	 * 
	 * @return <code>List&lt;TOUser&gt;</code> con todos los Alumnos.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de acceder
	 * a la capa de persistencia para obtener la informac&oacute;n.
	 */
	public abstract List<TOUser> getStudents() throws DataStoreException;
	
}
