package edu.uoc.pfc.business.interfaces;

import java.util.Date;
import java.util.List;

import edu.uoc.pfc.business.exceptions.BusinessRulesViolationException;
import edu.uoc.pfc.business.exceptions.DataStoreException;
import edu.uoc.pfc.business.exceptions.DuplicatedFieldException;
import edu.uoc.pfc.business.exceptions.InvalidIdException;
import edu.uoc.pfc.business.transfers.TOClassroom;
import edu.uoc.pfc.business.transfers.TOUser;

/**
 * <p>Interfaz que define las operaciones que la capa de negocio proporciona
 * a la capa de presentaci&oacute;n relacionadas con la gesti&oacute;n de las
 * aulas dentro de la aplicaci&oacute;n.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0 
 */
public interface IBClassroomManager 
{
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cada vez que quiera asignar
	 * uno de sus Cuestionarios a un Aula. Si el Cuestionarios ya hab&iacute;a sido previamente
	 * asignado al Aula este m&eacute;todo no har&aacute; nada y finaliza su ejecuci&oacute;n 
	 * sin ning&uacute;n tipo de error.</p>
	 * 
	 * @param idClassroom <code>int</code> identificador del Aula.
	 * @param idTest <code>int</code> identificador del Cuestionario.
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param openDate <code>Date</code> fecha de publicaci&oacute;n.
	 * @param closeDate <code>Date</code> fecha de cierre.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de acceder
	 * a la capa de persistencia.
	 * @throws InvalidIdException si el identificador del profesor no corresponde con el de
	 * ninguno de los profesores registrados en el sistema, si el identificador del aula no 
	 * corresponde con el de ninguna de las aulas asignadas al profesor o si el identificador
	 * del cuestionario no se corresponde con el de ninguno de los cuestionarios del profesor.
	 */
	public abstract void addTest(int idClassroom, int idTest, int idTeacher, Date openDate, 
			Date closeDate) throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado cuando se quiera a&ntilde;adir un Alumno
	 * a un Aula dentro de la aplicaci&oacute;n.</p>
	 * 
	 * @param idClassroom <code>int</code> identificador del Aula.
	 * @param idStudent <code>int</code> identificador del Estudiante.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de acceder
	 * a la capa de persistencia.
	 * @throws InvalidIdException si el identificador del Aula que se pasa como par&aacute;metro
	 * no corresponde con el de ninguna de las Aulas que est&aacute;n registradas en la aplicaci&oacute;n
	 * o si el identificador del Estudante que se pasa como par&aacute;metro tampoco corresponde a 
	 * ninguno de los Estudiantes que est&aacute;n registrados en la aplicaci&oacute;n.
	 * @throws BusinessRulesViolationException si el Estudiante ya ha sido asignado previamente al
	 * Aula como Profesor. 
	 */
	public abstract void assignStudent(int idClassroom, int idStudent)
			throws DataStoreException, InvalidIdException, BusinessRulesViolationException;
	
	/**
	 * <p>M&eacute;todo que ser&aacute; ejecutado cuando se quiera a&ntilde;adir un Profesor a un 
	 * Aula dentro de la aplicaci&oacute;n.</p>
	 * 
	 * @param idClassroom <code>int</code> identificador del Aula.
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de acceder
	 * a la capa de persistencia.
	 * @throws InvalidIdException si el identificador del Aula que se pasa como par&aacute;metro
	 * no corresponde con el de ninguna de las Aulas que est&aacute;n registradas en la aplicaci&oacute;n
	 * o si el identificador del Profesor que se pasa como par&aacute;metro tampoco corresponde a 
	 * ninguno de los Profesores que est&aacute;n registrados en la aplicaci&oacute;n.
	 * @throws BusinessRulesViolationException si el Profesor ya ha sido asignado previamente al
	 * Aula como Alumno. 
	 */
	public abstract void assignTeacher(int idClassroom, int idTeacher) 
			throws DataStoreException, InvalidIdException, BusinessRulesViolationException;
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado siempre que se quiera crear un nuevo Aula dentro
	 * del sistema.</p>
	 * 
	 * @param idCurriculum <code>int</code> identificador del Plan de Estudios del Aula.
	 * @param idSubject <code>int</code> identificador de la Asignatura que se imparte en el Aula.
	 * @param course <code>int</code> a&ntilde;o acad&eacute;mico.
	 * @param letter <code>String</code> letra que le corresponde al Aula.
	 * @return <code>TOClassroom</code> con la informaci&oacute;n correspondiente al nuevo Aula creado.
	 * @throws DataStoreException si se produce un error a la hora de acceder a la capa de persistencia.
	 * @throws InvalidIdException si el identificador del Plan de Estudios o de la Asignatura no se 
	 * corresponde con el de ning&uacute;n Plan de Estudios y Asignatura registrada.
	 * @throws DuplicatedFieldException si el Aula que se quiere registrar ya est&aacute; registrada 
	 * dentro del sistema.
	 */
	public abstract TOClassroom createClassroom(int idCurriculum, int idSubject, int course, String letter)
			throws DataStoreException, InvalidIdException, DuplicatedFieldException;
	
	/**
	 * <p>M&eacute;todo para eliminar del sistema la informaci&oacute;n del Aula cuyo
	 * identificador se le pasa como par&aacute;metro.</p>
	 * 
	 * @param idClassroom <code>int</code> identificador del Aula.
	 * @throws DataStoreException si se produce un error a la hora de trabajar con la capa de persistencia.
	 * @throws InvalidIdException si el identificador de Aula pasado no corresponde con el de ninguna
	 * de las Aulas que est&aacute;n registradas dentro de la aplicaci&oacute;n.
	 */
	public abstract void deleteClassroom(int idClassroom) throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>M&eacute;todo para ser&aacute; llamado para obtener la informaci&oacute;n de un
	 * Aula del sistema (aquella cuyo identificador se le pasa como par&aacute;metro).</p>
	 * 
	 * @param id <code>int</code> identificador del Aula.
	 * @return <code>TOClassroom</code> con la informaci&oacute;n del Aula.
	 * @throws DataStoreException si se produce un error a la hora de acceder a la capa de persistencia.
	 * @throws InvalidIdException si el identificador que se ha pasado no se corresponde 
	 * con el de ninguna de las Aulas registradas en el sistema.
	 */
	public abstract TOClassroom getClassroom(int id) throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>M&eacute;todo que retornar&aacute; un objeto <code>TOClassroom</code> con la informaci&oacute;n
	 * del Aula cuyo identificador se le pasa como par&aacute;metro siempre y cuando ese Aula corresponda
	 * al Estudiante cuyo identificador se le pasa como par&aacute;metro (es decir, que el Estudiante est&eacute;
	 * asignado como tal al Aula).</p>
	 * 
	 * @param idStudent <code>int</code> identificador del Estudiante.
	 * @param idClassroom <code>int</code> identificador del Aula.
	 * @return <code>TOClassroom</code> con toda la informaci&oacute;n del Aula.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con la
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador de Estudiante que se pasa no corresponde con el de un
	 * Estudiante registrado en el sistema o si el identificador del Aula no corresponde con el de ninguna 
	 * de las Aulas que el Estudiante tiene asignadas.
	 */
	public abstract TOClassroom getClassroomStudent(int idStudent, int idClassroom)
			throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>M&eacute;todo que retornar&aacute; un objeto <code>TOClassroom</code> con la informaci&oacute;n
	 * del Aula cuyo identificador se le pasa como par&aacute;metro siempre y cuando ese Aula corresponda
	 * al Profesor cuyo identificador se le pasa como par&aacute;metro (es decir, que el Profesor est&eacute;
	 * asignado como tal al Aula).</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idClassroom <code>int</code> identificador del Aula.
	 * @return <code>TOClassroom</code> con toda la informaci&oacute;n del Aula.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con la
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador de Profesor que se pasa no corresponde con el de un
	 * Profesor registrado en el sistema o si el identificador del Aula no corresponde con el de ninguna 
	 * de las Aulas que el Profesor tiene asignadas.
	 */
	public abstract TOClassroom getClassroomTeacher(int idTeacher, int idClassroom) 
			throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>Retorna una lista de objectos <code>TOUser</code> donde cada uno de ellos recoge la informaci&oacute;n
	 * de un Alumno que no ha sido asignado al Aula cuyo identificador se le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador del Aula.
	 * @return <code>List&lt;TOUser&gt;</code> con la informaci&oacute;n de todos los Alumnos que todav&iacute;a
	 * no han sido asignados al Aula.
	 * @throws DataStoreException si se produce un error a la hora de acceder a la capa de persistencia.
	 * @throws InvalidIdException si el identificador que se ha pasado no se corresponde 
	 * con el de ninguna de las Aulas registradas en el sistema.
	 */
	public abstract List<TOUser> getUnassignedStudents(int id) throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>Retorna una lista de objectos <code>TOClassroom</code> donde cada uno 
	 * de los elementos representa a un Aula registrada en el sistema.</p>
	 * 
	 * @return <code>List&lt;TOClassroom&gt;</code> donde cada uno de los elementos
	 * representa a un Aula registrada en el sistema.
	 * @throws DataStoreException si se produce un error a la hora de trabajar con
	 * la capa de persistencia.
	 */
	public abstract List<TOClassroom> listClassroom() throws DataStoreException; 
	
	/**
	 * <p>M&eacute;todo que permite obtener una lista de objetos <code>TOClassroom</code> donde cada uno
	 * de ellos representa un Aula donde el Estudiante cuyo identificador se le pasa como par&aacute;metro
	 * est&aacute; asignado.</p>
	 * 
	 * @param idStudent <code>int</code> identificador del Estudiante.
	 * @return <code>List&lt;TOClassroom&gt;</code> con todas las Aulas donde el usuario est&aacute;
	 * asignado como Estudiante.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con la
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador de usuario que ser recibe no corresponde con el 
	 * de ninguno de los Estudiantes que est&aacute;n registrados en el sistema.
	 */
	public abstract List<TOClassroom> listClassroomStudent(int idStudent) 
			throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>M&eacute;todo que proporciona una lista de objetos <code>TOClassromm</code> donde 
	 * cada uno de ellos representa un Aula donde est&aacute; asignado como profesor el usuario
	 * cuyo identificador se le pasa como parmetro.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @return <code>List&lt;TOClassroom&gt;</code> con todas las Aulas donde el usuario est&aacute;
	 * asignado como Profesor.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con la
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador de usuario que ser recibe no corresponde con el 
	 * de ninguno de los Profesores que est&aacute;n registrados en el sistema.
	 */
	public abstract List<TOClassroom> listClassroomTeacher(int idTeacher) 
			throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado para desasignar un Cuestionario de un Aula.</p> 
	 * 
	 * @param idTest <code>int</code> identificador del Cuestionario.
	 * @param idClassroom <code>int</code> identificador del Aula.
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con la
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador del profesor no corresponde con el de
	 * ninguno de los profesores registrados en el sistema, si el identificador del aula no 
	 * corresponde con el de ninguna de las aulas asignadas al profesor o si el identificador
	 * del cuestionario no se corresponde con el de ninguno de los cuestionarios del profesor.
	 */
	public abstract void removeTest(int idTest, int idClassroom, int idTeacher)
			throws DataStoreException, InvalidIdException;

	/**
	 * <p>Desasigna el Alumno cuyo identificador se le pasa en el par&aacute;metro <i>idStudent</i>
	 * del Aula cuyo identificador se le pasa en el par&aacute;metro <i>idClassroom</i>.</p>
	 * 
	 * @param idClassroom <code>int</code> identificador del Aula.
	 * @param idStudent <code>int</code> identificador del Alumno.
	 * @throws DataStoreException si se produce un error a la hora de acceder a la capa de persistencia.
	 * @throws InvalidIdException si <i>idStudent</i> no se corresponde con ninguno el ninguno
	 * de los estudiantes asignados al Aula o si <i>idClassroom</i> no se corresponde con el
	 * identificador de ninguna de las Aulas.
	 */
	public abstract void unassignStudent(int idClassroom, int idStudent) 
			throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>Desasigna el Profesor que est&aacute; asignado al Aula cuyo identificador se 
	 * le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador del Aula.
	 * @throws DataStoreException si se produce un error a la hora de acceder a la capa de persistencia.
	 * @throws InvalidIdException si el identificador que se ha pasado no se corresponde 
	 * con el de ninguna de las Aulas registradas en el sistema.
	 */
	public abstract void unassignTeacher(int id) throws DataStoreException, InvalidIdException;
	
}
