package edu.uoc.pfc.business.interfaces;

import edu.uoc.pfc.business.exceptions.BusinessRulesViolationException;
import edu.uoc.pfc.business.exceptions.DataStoreException;
import edu.uoc.pfc.business.transfers.TOConfig;

/** <p>Interfaz que define todos los m&eacute;todos que la capa de negocio ofrece a la
 * capa de presentaci&oacute;n relacionados con la gesti&oacute;n de la configuraci&oacute;n
 * de la aplicaci&oacute;n.</p>
 * 
 * <p>En concreto la interfaz definir&aacute; los m&eacute;todos para:</p>
 * <ul>
 * <li>Obtener la configuraci&oacute;n que est&aacute; activa.</li>
 * </ul>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0 
 */
public interface IBConfigManager
{
	
	/**
	 * <p>Retorna un objeto <code>TOConfig</code> con la informaci&oacute;n de configuraci&oacute;n
	 * que est&aacute; activa dentro de la aplicaci&oacute;n.</p>
	 * 
	 * @return <code>TOConfig</code> con la informaci&oacute;n de configuraci&oacute;n.
	 * @throws DataStoreException no se ha podido acceder a la capa de persistencia.
	 * @throws BusinessRulesViolationException si existe m&aacute;s de una configuraci&oacute;n
	 * activa dentro de la aplicaci&oacute;n.
	 */
	public abstract TOConfig getConfiguration() throws DataStoreException, 
			BusinessRulesViolationException;
	
	/**
	 * <p>Modifica la informaci&oacute;n de configuraci&oacute;n de la aplicaci&oacute;n con los
	 * valores que se le pasan como par&aacute;metros.</p>
	 * 
	 * @param loginAttempts <code>int</code> n&uacute;meros de intentos de acceso a la cuenta.
	 * @param activeCourse <code>int</code> curso actual (activo).
	 * @param adminEmail <code>String</code> email del administrador.
	 * @return <code>TOConfig</code> con la informaci&oacute;n de configuraci&oacute;n actualizada.
	 * @throws DataStoreException si se produce un error a la hora de trabajar con la 
	 * capa de persistencia.
	 * @throws BusinessRulesViolationException si existe m&aacute;s de una configuraci&oacute;n
	 * activa dentro de la aplicaci&oacute;n.
	 */
	public abstract TOConfig modifyConfiguration(int loginAttempts, int activeCourse, String adminEmail) 
			throws DataStoreException, BusinessRulesViolationException;
	
	/**
	 * <p>Modifica la informaci&oacute;n de configuraci&oacute;n de la aplicaci&oacute;n con los valores
	 * que se le pasan como par&aacute;metro.</p>
	 * 
	 * @param loginAttempts code>int</code> n&uacute;meros de intentos de acceso a la cuenta.
	 * @param activeCourse <code>int</code> curso actual (activo).
	 * @param adminEmail <code>String</code> email del administrador.
	 * @param smtpServer <code>String</code> servidor SMTP.
	 * @param smtpPort <code>int</code> puerto del servidor SMTP.
	 * @param smtpAuthentication <code>boolean</code> indica si el servidor SMTP requiero autentificaci&oacute;n
	 * @param smtpTLS <code>boolean</code> indica si el servidor SMTP admite el protocolo TLS.
	 * @param smtpUser <code>String</code> usuario servidor SMTP
	 * @param smtpPassword <code>String</code> password servidor SMTP.
	 * @param fpFrom <code>String</code> email de quien env&iacute;a el correo de recuperaci&oacute; de contrase&ntilde;a.
	 * @param fpSubject <code>String</code> asunto del correo de recuperaci&oacute; de contrase&ntilde;a.
	 * @param fpText <code>String</code> texto del correo de recuperaci&oacute; de contrase&ntilde;a.
	 * @param flFrom <code>String</code> email de quien env&iacute;a el correo de recuperaci&oacute; de login.
	 * @param flSubject <code>String</code> asunto del correo de recuperaci&oacute; de login.
	 * @param flText <code>String</code> texto del correo de recuperaci&oacute; de login.
	 * @return <code>TOConfig</code> con la informaci&oacute;n de configuraci&oacute;n actualizada.
	 * @throws DataStoreException si se produce un error a la hora de trabajar con la 
	 * capa de persistencia.
	 * @throws BusinessRulesViolationException si existe m&aacute;s de una configuraci&oacute;n
	 * activa dentro de la aplicaci&oacute;n.
	 */
	public abstract TOConfig modifyConfiguration(int loginAttempts, int activeCourse, String adminEmail,
			String smtpServer, int smtpPort, boolean smtpAuthentication, boolean smtpTLS, 
			String smtpUser, String smtpPassword, String fpFrom, String fpSubject, String fpText,
			String flFrom, String flSubject, String flText)
			throws DataStoreException, BusinessRulesViolationException;
	
}
