package edu.uoc.pfc.business.interfaces;

import java.util.List;

import edu.uoc.pfc.business.exceptions.DataStoreException;
import edu.uoc.pfc.business.exceptions.DuplicatedFieldException;
import edu.uoc.pfc.business.exceptions.InvalidIdException;
import edu.uoc.pfc.business.transfers.TOCurriculum;
import edu.uoc.pfc.business.transfers.TOSubject;

/**
 * <p>Interfaz que define todas las operaciones que la capa de negocio
 * ofrece a la capa de presentaci&oacute;n relacionadas con la gesti&oacute;n
 * de los planes de estudio.</p>
 * 
 * <p>En concreto esta interfaz define los m&eacute;todos para:</p>
 * <ul>
 * <li>Obtener una lista con todos los planes de estudios que est&aacute;n
 * registrados en el aplicaci&oacute;n agrupados por el tipo de plan del
 * que se trata (diplomatura, licenciatura, grado, tec.).</li>
 * </ul>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0 
 */
public interface IBCurriculumManager 
{
	
	// -------- MTODOS PBLICOS Y ABSTRACTOS.
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un administrador del sistema cuando quiera asignar 
	 * una asignatura a un plan de estudios.</p>
	 * 
	 * @param idCurriculum <code>int</code> identificador del Plan de Estudios.
	 * @param idSubject <code>int</code> identificador de la Asignatura.
	 * @param subjectType <code>String</code> tipo de Asignatura dentro del Plan de Estudios. 
	 * @param code <code>String</code> c&oacute;digo de la Asignatura.
	 * @param credits <code>double</code> n&uacute;mero de los cr&eeacute;ditos de la Asignatura.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de excepci&oacute;n a
	 * la hora de trabajar con la capa de persistencia.
	 * @throws InvalidIdException si el identificador del Plan de Estudios o de la Asignatura que se 
	 * le ha pasado es inv&aacute;lido.
	 * @throws DuplicatedFieldException si la asignaci&oacute;n de la Asignatura al Plan de Estudios
	 * ya haba sido realizada anteriormente.
	 */
	public abstract void addSubject(int idCurriculum, int idSubject, String subjectType, String code, 
			double credits) throws DataStoreException, InvalidIdException, DuplicatedFieldException;
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado por un administrador del sistema cuando quiera 
	 * registrar un nuevo Plan de Estudios dentro de la aplicaci&oacute;n.</p>
	 * 
	 * @param name <code>String</code> nombre del Plan de Estudios.
	 * @param description <code>String</code> descripci&oacute;n del Plan de Estudios.
	 * @param type <code>String</code> tipo del Plan de Estudios.
	 * @return <code>TOCurriculum</code> con toda la informaci&oacute;n correspondiente al 
	 * Plan de Estudios que se acaba de crear.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de excepci&oacute;n a
	 * la hora de trabajar con la capa de persistencia.
	 * @throws DuplicatedFieldException si dentro del sistema ya existe un Plan de Estudios 
	 * registrado cuyo nombre y tipo concidan con los valores que se pasan en los par&aacute;metros
	 * <i>name</i> y <i>type</i> respectivamente.
	 */
	public abstract TOCurriculum createPlan(String name, String description, String type) 
			throws DataStoreException, DuplicatedFieldException;
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado para eliminar del sistema el Plan de Estudios
	 * cuyo identificador se le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador del Plan de Estudios.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de excepci&oacute;n a
	 * la hora de trabajar con la capa de persistencia.
	 * @throws InvalidIdException si el identificador que se ha pasado como par&aacute;metro 
	 * no corresponde con el de ninguno de los Planes de Estudio que est&aacute;n registrados.
	 */
	public abstract void deletePlan(int id) throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado para obtener la informaci&oaute;n del Plan de 
	 * Estudios cuyo identificador se recibe como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador del Plan de Estudios.
	 * @return <code>TOCurriculum</code> con la informaci&oacute;n del Plan de Estudios cuyo
	 * identificador es igual al par&aacute;metro <i>id</i>.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de excepci&oacute;n a
	 * la hora de trabajar con la capa de persistencia.
	 * @throws InvalidIdException si el identificador que se ha pasado como par&aacute;metro 
	 * no corresponde con el de ninguno de los Planes de Estudio que est&aacute;n registrados.
	 */
	public abstract TOCurriculum getPlan(int id) throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>Obtiene la lista de asignaturas que todava no est&aacute;n asignadas al Plan 
	 * de Estudios cuyo identificador se le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador del Plan de Estudios.
	 * @return <code>List&lt;TOSubject&gt;</code> donde cada uno de los elementos ser&aacute;
	 * una asignatura no asignada al Plan de Estudios.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de excepci&oacute;n a
	 * la hora de trabajar con la capa de persistencia.
	 * @throws InvalidIdException si el identificador que se ha pasado como par&aacute;metro 
	 * no corresponde con el de ninguno de los Planes de Estudio que est&aacute;n registrados.
	 */
	public abstract List<TOSubject> getUnassignedSubjects(int id) 
			throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>Retorna una lista de objetos <code>TOCurriculum</code> cada uno de los cuales
	 * representar&aacute; a uno de los Planes de Estudios que est&aacute;n registrados
	 * dentro del sistema.</p>
	 * 
	 * @return <code>List&lt;TOCurriculum&gt;</code> con la informaci&oacute;n de los
	 * Planes de Estudio registrados.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de excepci&oacute;n a
	 * la hora de trabajar con la capa de persistencia.
	 */
	public abstract List<TOCurriculum> listPlan() throws DataStoreException;	
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un administrador del sistema cuando 
	 * desee modificar la informaci&oacute;n de un Plan de Estudios que est&aacute;.</p> 
	 * 
	 * @param id <code>int</code> identificador del Plan de Estudios.
	 * @param description <code>String</code> nueva descripci&oacute;n del Plan de Estudios.
	 * @return
	 * @throws DataStoreException si se produce alg&uacute;n tipo de excepci&oacute;n a
	 * la hora de trabajar con la capa de persistencia.
	 * @throws InvalidIdException  si el identificador que se ha pasado como par&aacute;metro 
	 * no corresponde con el de ninguno de los Planes de Estudio que est&aacute;n registrados.
	 */
	public abstract TOCurriculum modifyPlan(int id, String description) 
			throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un administrador del sistema cuando desee
	 * eliminar la asignaci&oacute;n de una Asignatura a un determinado Plan de Estudios.</p>
	 * 
	 * @param idCurriculum <code>int</code> identificador del Plan de Estudios.
	 * @param idSubject <code>int</code> identificador de la Asignatura.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de excepci&oacute;n a
	 * la hora de trabajar con la capa de persistencia.
	 * @throws InvalidIdException si no se ha realizado previamente la asignaci&oacute;n
	 * de la asignatura al Plan de Estudios previamente.
	 */
	public abstract void removeSubject(int idCurriculum, int idSubject) 
			throws DataStoreException, InvalidIdException;
	
}
