package edu.uoc.pfc.business.interfaces;

import java.util.List;

import edu.uoc.pfc.business.exceptions.BusinessRulesViolationException;
import edu.uoc.pfc.business.exceptions.DataStoreException;
import edu.uoc.pfc.business.exceptions.DuplicatedFieldException;
import edu.uoc.pfc.business.exceptions.InvalidIdException;
import edu.uoc.pfc.business.transfers.TOQuestion;
import edu.uoc.pfc.business.transfers.TOQuestionCategory;


/**
 * <p>Interfaz que define la API que la capa de negocio ofrecer&aacute; a la
 * capa de presentaci&oacute;n asociada con las transacciones que tienen que
 * ver con la gesti&oacute;n los Bancos de Preguntas de los Profesores.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public interface IBQuestionBankManager 
{
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera a&ntilde;adir una 
	 * nueva Categor&iaacute;a de Preguntas a su Banco de Preguntas.</p>
	 *
	 * @param idTeacher <code>int</code> identificador del profesor.
	 * @param name <code>String</code> nombre de la nueva Categor&iacute;a de Preguntas.
	 * @param description <code>String</code> descripci&oacute;n de la Categor&iacute;a de Preguntas.
	 * @return <code>TOQuestionCategory</code> con la informaci&oacute;n de la nueva Categor&iacute;a
	 * de Preguntas.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de trabajar con la 
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador de profesor proporcionado no corresponde 
	 * con el de ninguno de los Profesores que est&aacute;n registrados en el sistema.
	 * @throws DuplicatedFieldException si en el sistema el Profesor ya posee una Categor&iacute;a
	 * de Preguntas con el mismo nombre que el que se ha pasada.
	 */
	public abstract TOQuestionCategory createCategory(int idTeacher, String name, 
			String description) throws DataStoreException, InvalidIdException, 
			DuplicatedFieldException;
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quira a&ntilde;adir 
	 * una Preguntas a una de sus Cateogor&iacutes de Preguntas.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.  
	 * @param idCategory <code>int</code> identificador de la Categor&iacute;
	 * @param question <code>TOQuestion</code> con la informaci&oacute;n de la Pregunta.
	 * @return <code>TOQuestion</code> que contendr&aacute; toda al informaci&oacute;n de
	 * la pregunta una vez ha sido registrada en el sistema.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de trabajar con la 
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador de profesor proporcionado no corresponde 
	 * con el de ninguno de los Profesores que est&aacute;n registrados en el sistema o si el 
	 * identificador de la Categor&iacute; a de Preguntas no corresponde con el de ninguna de 
	 * las Categor&iacute;as de Preguntas definidas el por el Profesor.
	 * @throws DuplicatedFieldException si ya existe una pregunta dentro de la Categor&iacute;a
	 * con el mismo nombre y del mismo tipo que la que se pretende crear.
	 * @throws BusinessRulesViolationException si se intenta crear una pregunta de tipo Opci&oacute;n
	 * M&uacute;ltiple sin haber especificado al menos dos opciones de respuesta.
	 */
	public abstract TOQuestion createQuestion(int idTeacher, int idCategory, TOQuestion question)
			throws DataStoreException, InvalidIdException, DuplicatedFieldException,
			BusinessRulesViolationException;
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera eliminar una 
	 * Categor&iacute;a de Preguntas del sistema.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idCategory <code>int</code> identificador de la Categor&iacute;a de Preguntas.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de trabajar con la 
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador del Profesor no corresponde con el de un
	 * Usuario de Profesor registrado o si el identificador de la Categor&iacute;a no corresponde
	 * con el de una Categor&iacute;a de preguntas dentro del Banco de Preguntas del Profesor.
	 */
	public abstract void deleteCategory(int idTeacher, int idCategory) throws DataStoreException,
			InvalidIdException;
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera eliminar una Pregunta
	 * dentro de su Banco de Preguntas.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idQuestion <code>int</code> identificador de la Pregunta.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de trabajar con la 
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador del Profesor no corresponde con el de un
	 * Usuario de Profesor registrado o el identificador de la Pregunta no se corresponde con 
	 * el de una Pregunta que pertenece al Banco de Preguntas del Profesor.
	 */
	public abstract void deleteQuestion(int idTeacher, int idQuestion) throws DataStoreException,
			InvalidIdException;
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando obtener la informaci&oacute;n
	 * de una de sus Categor&iacute;a de Preguntas.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idCategory <code>int</code> identificador de la Categor&iacute;a de Preguntas.
	 * @return <code>TOQuestionCategory</code> con toda la informaci&oacute;n de la Categor&iacute;a.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de trabajar con la 
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador del Profesor no corresponde con el de un
	 * Usuario de Profesor registrado o si el identificador de la Categor&iacute;a no corresponde
	 * con el de una Categor&iacute;a de preguntas dentro del Banco de Preguntas del Profesor.
	 */
	public abstract TOQuestionCategory getCategory(int idTeacher, int idCategory) 
			throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>M&eacute;todo que permite recuperar la informaci&oacute;n de una Pregunta registrada en 
	 * el sistema y cuyo identificador se le pasa como par&aacute;metro.</p>
	 * 
	 * @param idQuestion <code>int</code> identificador de la Pregunta.
	 * @return <code>TOQuestion</code> con la informaci&oacute;n de la Pregunta.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con
	 * la capa de persistencia.
	 * @throws InvalidIdException si el identificador que se le pasa no se corresponde con el de
	 * ninguna de las Preguntas que est&aacute;n registradas en el sistema.
	 */
	public abstract TOQuestion getQuestion(int idQuestion) 
			throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera obtener la 
	 * informaci&oacute;n de una de sus Preguntas creadas en su Banco de Preguntas.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idQuestion <code>int</code> identificador de la Pregunta.
	 * @return <code>TOQuestion</code> con la informaci&oacute;n de la Pregunta.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de trabajar con la 
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador del Profesor no corresponde con el de un
	 * Usuario de Profesor registrado o si el identificador de la Pregunta no corresponde con
	 * el de una Pregunta que pertenece al Profesor:
	 */
	public abstract TOQuestion getQuestion(int idTeacher, int idQuestion)
			throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado para obtener la lista de Categor&iacute;as
	 * de Preguntas que el usuario couyo identificador se pasa en el par&aacute;metro <i>idTeacher</i>
	 * tiene definido dentro de su Bando de Preguntas.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del profesor.
	 * @return <code>List&lt;TOQuestionCategory&gt;</code> con todas las Categor&iacute;as de 
	 * Preguntas que el Usuario ha definido dentro de su Bando de Preguntas.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de trabajar con la 
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador de profesor proporcionado no corresponde 
	 * con el de ninguno de los Profesores que est&aacute;n registrados en el sistema.
	 */
	public abstract List<TOQuestionCategory> listCategories(int idTeacher) 
			throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un profesor cuando quiera modificar una 
	 * la informaci&oacute;n de una de sus Categor&iacute;as de Preguntas.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idCategory <code>int</code> identificador de la Categor&iacute;a de Preguntas.
	 * @param description <code>String</code> la nueva descripci&oacutge;n de la Categor&iacute;a.
	 * @return <code>TOQuestionCategory</code> con toda la informaci&oacute;n de la Categor&iacute;a
	 * una vez se ha modificado.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de trabajar con la 
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador del Profesor no corresponde con el de un
	 * Usuario de Profesor registrado o si el identificador de la Categor&iacute;a no corresponde
	 * con el de una Categor&iacute;a de preguntas dentro del Banco de Preguntas del Profesor.
	 */
	public abstract TOQuestionCategory modifyCategory(int idTeacher, int idCategory, 
			String description)	throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado para modificar la informaci&oacute;n de una
	 * Pregunta del Profesor cuyo identificador se le pasa como par&aacute;metro.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idQuestion <code>int</code> identificador de la Pregunta.
	 * @param question <code>TOQuestion</code> con la informaci&oacute;n de la pregunta.
	 * @return <code>TOQuestion</code> con la informaci&aocute;n de la Pregunta moficada.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de trabajar con la 
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador del Profesor no corresponde con el de un
	 * Usuario de Profesor registrado o si el identificador de la Pregunta no corresponde
	 * con el de una de las Preguntas que el Profesor tiene creadas en el sistema.
	 * @throws BusinessRulesViolationException si no se han especificado al menos dos opciones
	 * de respuesta para una pregunta de tipo "Opci&oacute;n M&uacute;ltiple".
	 */
	public abstract TOQuestion modifyQuestion(int idTeacher, int idQuestion, TOQuestion question)
			throws DataStoreException, InvalidIdException, BusinessRulesViolationException;
	
}
