package edu.uoc.pfc.business.interfaces;

import java.util.List;

import edu.uoc.pfc.business.exceptions.DataStoreException;
import edu.uoc.pfc.business.exceptions.DuplicatedFieldException;
import edu.uoc.pfc.business.exceptions.InvalidIdException;
import edu.uoc.pfc.business.transfers.TOCurriculum;
import edu.uoc.pfc.business.transfers.TOSubject;

/**
 * <p>Interfaz que define la serie de m&eeacute;todos que la capa de negocio
 * ofrece a la capa de presentaci&oacute;n y que estar&aacute;n relacionados
 * con la gesti&oacute;n de las asignaturas.</p>
 * 
 * <p>Estos m&eeacute;todos han de permitir:</p>
 * <ul>
 * <li>Obtener una lista de todas las asignaturas que est&aacute;n registradas
 * en la aplicaci&oacute;n.</li>
 * <li>Obtener los datos de una determinada asignatura.</li>
 * </ul>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0 
 */
public interface IBSubjectManager
{
	
	// -------- MTODOS PBLICOS Y ABSTRACTOS.
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un administrador del sistema cuando quiera asignar 
	 * una asignatura a un plan de estudios.</p>
	 * 
	 * @param idSubject <code>int</code> identificador de la Asignatura.
	 * @param idCurriculum <code>int</code> identificador del Plan de Estudios.
	 * @param subjectType <code>String</code> tipo de Asignatura dentro del Plan de Estudios. 
	 * @param code <code>String</code> c&oacute;digo de la Asignatura.
	 * @param credits <code>double</code> n&uacute;mero de los cr&eeacute;ditos de la Asignatura.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de excepci&oacute;n a
	 * la hora de trabajar con la capa de persistencia.
	 * @throws InvalidIdException si el identificador del Plan de Estudios o de la Asignatura que se 
	 * le ha pasado es inv&aacute;lido.
	 * @throws DuplicatedFieldException si la asignaci&oacute;n de la Asignatura al Plan de Estudios
	 * ya haba sido realizada anteriormente.
	 */
	public abstract void addCurriculum(int idSubject, int idCurriculum, String subjectType, String code, 
			double credits) throws DataStoreException, InvalidIdException, DuplicatedFieldException;
	
	/**
	 * <p>Crea una nueva asignatura dentro de la base de datos a partir de la informaci&oacute;n
	 * que se le pasa como pa&aacute;metro. En el caso de que ya exista una asignatura con el 
	 * mismo nombre (<i>name</i>) este m&eacute;todo lanzar&aacute; una <code>DuplicatedFieldException
	 * </code>. Si el proceso de creaci&oacute;n de la asignatura finaliza correctamente la 
	 * informaci&oacute; de la nueva asignatura ser&aacute; retornada dentro de un objeto de 
	 * tipo <code>TOSubject</code>.</p>
	 *  
	 * @param name <code>String</code> nombre de la asignatura.
	 * @param description <code>String</code> descripci&oacute;n de la asignatura.
	 * @return <code>TOSubject</code> con la informaci&oacute;n de la nueva asignatura.
	 * @throws DataStoreException si se produce al&uacute;n tipo de error dentro del
	 * marco de persistencia.
	 * @throws DuplicatedFieldException si ya existe una asignatura con el mismo nombre registrada
	 * dentro de la aplicaci&oacute;n.
	 */
	public abstract TOSubject createSubject(String name, String description)
			throws DataStoreException, DuplicatedFieldException;
	
	/**
	 * <p>Elimina del sistema la asignatura cuyo identificador se le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador de la asignatura.
	 * @throws DataStoreException si se produce al&uacute;n tipo de error dentro del
	 * marco de persistencia.
	 * @throws InvalidIdException si el identificador de la asignatura que se ha pasado no
	 * coincide con el de ninguna de las asignaturas que est&aacute;n registradas en el sistema.
	 */
	public abstract void deleteSubject(int id) throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>Obtiene la informaci&oacute;n de la asignatura cuyo identificador se le pasa como
	 * par&aacute;metro y la devuelve encapsulada dentro del un objeto <code>TOSubject</code>.</p>
	 * 
	 * @param id <code>int</code> identificador de la asignatura.
	 * @return <code>TOSubject</code> con la informaci&oacute;n de la asignatura.
	 * @throws DataStoreException si se produce al&uacute;n tipo de error dentro del
	 * marco de persistencia.
	 * @throws InvalidIdException si el identificador que se ha pasado no corresponde con el
	 * de ninguna de las asignaturas registradas dentro de la aplicaci&oacute;n.
	 */
	public abstract TOSubject getSubject(int id) throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>Obtiene una lista de objetos <code>TOCurriculum</code> donde cada uno de ellos recoge
	 * la informaci&oacute;n de un Plan de Estudios que no ha sido asignado a la Asignatura cuyo
	 * identificador se le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador de la Asignatura.
	 * @return <code>List&lt;TOCurriculum&gt;</code> donde cada elemento representa a un Plan
	 * de Estudios al que no ha sido asignado la asignatura.
	 * @throws DataStoreException si se produce al&uacute;n tipo de error dentro del
	 * marco de persistencia.
	 * @throws InvalidIdException si el identificador que se ha pasado no corresponde con el
	 * de ninguna de las asignaturas registradas dentro de la aplicaci&oacute;n.
	 */
	public abstract List<TOCurriculum> getUnassignedCurriculum(int id)
			throws DataStoreException, InvalidIdException;
			
	/**
	 * <p>Obtiene una lista de objetos <code>TOSubject</code> cada uno de los cuales
	 * contiene la informaci&oacute;n de una de las asignaturas que est&aacute;n 
	 * registradas en el sistema.</p>
	 * 
	 * @return List&lt;TOSubject&gt; con todas las asignaturas registradas.
	 * @throws DataStoreException si se produce al&uacute;n tipo de error dentro del
	 * marco de persistencia.
	 */
	public abstract List<TOSubject> listSubject() throws DataStoreException;
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado por un administrador del sistema cada vez
	 * que quiera modificar la informaci&oacute;n de la Asignatura cuyo identificador se
	 * le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador de a Asignatura.
	 * @param description <code>String</code> nueva descripci&oacute; para la asignatura.
	 * @return <code>TOSubject</code> con la informaci&oacute;n de la Asignatura ya modificada.
	 * @throws DataStoreException  si se produce al&uacute;n tipo de error dentro del
	 * marco de persistencia.
	 * @throws InvalidIdException si el identificador que se ha pasado no corresponde con el
	 * de ninguna de las asignaturas registradas dentro de la aplicaci&oacute;n.
	 */
	public abstract TOSubject modifySubject(int id, String description) 
			throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado en el momento en el que se quiera desasignar
	 * una Asignatura (identificada por "idSubject") de un Plan de Estudios (identificado 
	 * por "idCurriculum").</p>
	 * 
	 * @param idSubject <code>int</code> Identificador de la Asignatura.
	 * @param idCurriculum <code>int</code> Identificador del Plan de Estudios.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de excepci&oacute;n a
	 * la hora de trabajar con la capa de persistencia.
	 * @throws InvalidIdException si no se ha realizado previamente la asignaci&oacute;n
	 * de la asignatura al Plan de Estudios previamente.
	 */
	public abstract void removeCurriculum(int idSubject, int idCurriculum)
			throws DataStoreException, InvalidIdException;
	
}
