package edu.uoc.pfc.business.interfaces;

import java.util.List;

import edu.uoc.pfc.business.exceptions.DataStoreException;
import edu.uoc.pfc.business.exceptions.DuplicatedFieldException;
import edu.uoc.pfc.business.exceptions.InvalidIdException;
import edu.uoc.pfc.business.transfers.TOTest;
import edu.uoc.pfc.business.transfers.TOTestAnswer;
import edu.uoc.pfc.business.transfers.TOTestStats;


/**
 * <p>Interfaz que define la lista de m&eeacute;todos que la capa de negocio ofrece a 
 * la capa de presentaci&oacute;n relacionados con la gesti&oacute;n de los cuestionarios.</p>
 *  
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public interface IBTestManager 
{

	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera asignar una de las 
	 * Preguntas de su Banco de Preguntas a uno de sus Cuestionarios. En el caso de que la Pregunta
	 * ya haya sido asignada previamente al Cuestionario este m&eacute;todo actualizar&aacute; la 
	 * informaci&oacute;n.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idTest <code>int</code> identificador del Cuestionario.
	 * @param idQuestion <code>int</code> identificador de la Pregunta.
	 * @param value <code>double</code> valor de la Pregunta en el Cuestionario.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con
	 * la capa de persistencia.
	 * @throws InvalidIdException si el identificador del Profesor no se corresponde con el de un Profesor
	 * registrado en el sistema; si el identificador del Cuestionario no se corresponde con  el de un 
	 * Cuestionario del Profesor; o si el identificador de la Pregunta no se corresponde con el de una
	 * de las Preguntas dentro del Banco de Preguntas del Profesor.
	 */
	public abstract void addQuestion(int idTeacher, int idTest, int idQuestion, double value)
			throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor del sistema cada vez que quiera crear
	 * un nuevo Cuestionario.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param title <code>String</code> t&iacute;tulo del Cuestionario.
	 * @param introduction <code>String</code> texto de introducci&oacute;n.
	 * @param shuffleQuestions <code>boolean</code> indica si se barajar&aacute;n las Preguntas.
	 * @param shuffleAnswers <code>boolean</code> indica si se barajar&aacute;n las Respuestas.
	 * @param showSolution <code>boolean</code> indica si se muestra la soluci&oacute;n.
	 * @param showReview <code>indica si se muestra la revisi&oacute;n</code>
	 * @return <code>TOTest</code> con toda la informaci&oacute;n de Cuestionario que se acaba
	 * de crear en el sistema.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con
	 * la capa de persistencia.
	 * @throws DuplicatedFieldException si el Profesor ya tiene registrado en el sistema un Cuestionario
	 * cuyo t&iacute;tulo coincide con el que se le pasa como par&aacute;metro.
	 * @throws InvalidIdException si el identificador de usuario que se le pasa no se corresponde con
	 * el de ninguno de los profesores registrados en el sistema.
	 */
	public abstract TOTest createTest(int idTeacher, String title, String introduction, 
			boolean shuffleQuestions, boolean shuffleAnswers, boolean showSolution, boolean showReview) 
			throws DataStoreException, DuplicatedFieldException, InvalidIdException;
	
	/**
	 * <p>Elimina del sistema el Cuestionario cuyo indentificador es "idTest" siempre y cuando este
	 * Cuestionario pertenezca al Usuario cuyo identificador es "idTeacher".</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idTest <code>int</code> identificador del Cuestionario.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con
	 * la capa de persistencia.
	 * @throws InvalidIdException si "idTeacher" no es el de un usuario Profesor o si "idTest" no 
	 * corresponde con el identificador de ninguno de los Cuestionarios del sistema que pertenecen 
	 * al Profesor.
	 */
	public abstract void deleteTest(int idTeacher, int idTest)
			throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>Obtiene los datos del Cuestionario cuyo identificador se le pasa como par&aacute;metro.</p>
	 * 
	 * @param idTest <code>int</code> identificador del Cuestionario.
	 * @param idClassroom <code>int</code> identificador del Aula.
	 * @return <code>TOTest</code> con la informaci&oacute;n del Cuestionario.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con
	 * la capa de persistencia.
	 * @throws InvalidIdException si el identificador que se le pasa no se corresponde con el
	 * de ninguno de los Cuestionarios que est&aacute;n registrados en el sistema o si el identificador
	 * del Aula no corresponde a ninguna de las Aulas registradas en el sistema o si el test no ha
	 * sido asignado al aula.
	 */
	public abstract TOTest getTestClassroom(int idTest, int idClassroom) throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>Obtiene los datos del Cuestionario cuyo identificador se pasa en "idTest" siempre y cuando
	 * ese Cuestionario pertenezca al Profesor cuyo identificador se pasa en el par&aacute;metro "idTeacher".</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idTest <code>int</code> identificador del Cuestionario.
	 * @return <code>TOTest</code> con la informaci&oacute;n del Cuestionario.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con
	 * la capa de persistencia.
	 * @throws InvalidIdException si "idTeacher" no es el de un usuario Profesor o si "idTest" no 
	 * corresponde con el identificador de ninguno de los Cuestionarios del sistema que pertenecen 
	 * al Profesor.
	 */
	public abstract TOTest getTest(int idTeacher, int idTest) 
			throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>M&eacute;todo que obtiene las Estad&iacute;sticas detalladas de un Cuestionario de 
	 * un Profesor.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idTest <code>int</code> identificador del Cuestionario.
	 * @return <code>TOTestStats</code> con la informaci&oacute;n de las Estad&iacute;sticas.
	 * @throws DataStoreException  si se produce alg&uacute;n tipo de error a la hora de trabajar con
	 * la capa de persistencia.
	 * @throws InvalidIdException si "idTeacher" no es el de un usuario Profesor o si "idTest" no 
	 * corresponde con el identificador de ninguno de los Cuestionarios del sistema que pertenecen 
	 * al Profesor.
	 */
	public abstract TOTestStats getTestStats(int idTeacher, int idTest)
			throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado para obtener una lista de objetos <code>TOTest</code>
	 * que representar&aacute;n a los diferentes Cuestionarios que el Profesor cuyo identidicador
	 * se le pasa como par&aacute;metro, tiene registrados dentro del sistema.</p> 
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @return <code>List&lt;TOTest&gt;</code> donde cada elemento representa a un Cuestionario del 
	 * Profesor registrado en el sistema.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con
	 * la capa de persistencia.
	 * @throws InvalidIdException si el identificador de usuario que se le pasa no se corresponde con
	 * el de ninguno de los profesores registrados en el sistema.
	 */
	public abstract List<TOTest> listTest(int idTeacher)
			throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado para obtener una lista de objetos <code>TOTestStat</code>
	 * donde cada uno de ellos representar&aacute; las estad&iacute;sticas de resoluci&oacute;n de cada
	 * uno de los Cuestionarios que el Profesor cuyo identificador se recibe como par&aacute;metro tiene
	 * definido en el sistema.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @return <code>List&lt;TOTestStats&gt;</code> donde cada elemento representa a las estad&iacute;sticas
	 * de un Cuestionario del Profesor.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de trabajar con la capa de 
	 * persistenica.
	 * @throws InvalidIdException si el identificador de usuario que se le pasa no se corresponde con
	 * el de ninguno de los profesores registrados en el sistema.
	 */
	public abstract List<TOTestStats> listTestStats(int idTeacher)
			throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>Modifica los datos del Cuestionario identificado por el atributo "idTest" (Cuestionario que
	 * ha de pertenecer al Profesor identificador por "idTeacher") con la informaci&oacute;n suministrada
	 * en el resto de par&aacute;metros del m&eacute;todo.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idTest <code>int</code> identificador del Cuestionario.
	 * @param introduction <code>String</code> texto de introducci&oacute;n.
	 * @param shuffleQuestions <code>boolean</code> indica si se deben barajar las preguntas.
	 * @param shuffleAnswers <code>boolean</code> indica si se deben barajar las respuestas.
	 * @param showResult <code>boolean</code> indica si se mostrar&aacute; el resultado.
	 * @param showReview <code>boolean</code> indic asi se mostrar&aacute; la revisi&oacute;n.
	 * @return <code>TOTest</code> con la informaci&oacute;n del Cuestionario modificada.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con
	 * la capa de persistencia.
	 * @throws InvalidIdException si "idTeacher" no es el de un usuario Profesor o si "idTest" no 
	 * corresponde con el identificador de ninguno de los Cuestionarios del sistema que pertenecen 
	 * al Profesor.
	 */
	public abstract TOTest modifyTests(int idTeacher, int idTest, String introduction, 
			boolean shuffleQuestions, boolean shuffleAnswers, boolean showResult, boolean showReview)
			throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>M&eacute;todo que se encargar&aacute; de desasignar una Pregunta de un Cuestionario de un
	 * un Profesor. En el caso de que se llame a este m&eacute;todo y la pregunta no haya sido asignada
	 * previamente el Cuestionario, el m&eacuete;todo no realizar&aacute; ninguna acci&oacute;n.</p> 
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idTest <code>int</code> identificador del Cuestionario.
	 * @param idQuestion <code>int</code> identificador de la Pregunta.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con
	 * la capa de persistencia.
	 * @throws InvalidIdException si el identificador de Profesor no corresponde con el de un Profesor
	 * registrado en el sistema; si el identificador de Cuestionario no corresponde con el de un Cuestionario
	 * del Profesor; si el identificador de la Pregunta no corresponde con el de una Pregunta de Profesor.
	 */
	public abstract void removeQuestion(int idTeacher, int idTest, int idQuestion)
			throws DataStoreException, InvalidIdException;
	
	/**
	 * <p>M&eacute;todo que ser&aacute; el encargado de recoger dentro del sistema la informaci&oacute;n
	 * relativa a una respuesta de un Cuestionario por parte de un Alumno.</p>
	 * 
	 * @param idStudent <code>int</code> el identificador del Alumno.
	 * @param idTestClassroom <code>int</code> el identificador del Test una vez asociado a un Aula.
	 * @param score <code>double</code> la puntuaci&oacute;n obtenida.
	 * @param maxScore <code>double</code> la puntuaci&oacute;n m&aacute;xima que se pod&iacute;a obtener.
	 * @param totalQuestions <code>int</code> n&uacute;mero de Preguntas que forman el Cuestionario.
	 * @param correctQuestions <code>int</code> n$uacute;mero de Preguntas contestadas correctamente.
	 * @return <code>TOTestAnswer</code> con toda la informaci&oacute;n de la respuesta una vez que ha 
	 * sido grabada dentro del sistema.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con la
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador que se le pasa no corresponde con el de un usuario
	 * registrado como Alumno en el sistema o si el identificador del Cuestionario no corresponde con el
	 * identificador de ninguna de los Cuestionarios una vez han sido asignados a un Aula.
	 */
	public abstract TOTestAnswer saveAnswer(int idStudent, int idTestClassroom, double score,
			double maxScore, int totalQuestions, int correctQuestions) throws DataStoreException, InvalidIdException;
	
}
