package edu.uoc.pfc.business.objects;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import edu.uoc.pfc.business.transfers.TOClassroom;
import edu.uoc.pfc.business.transfers.TOTest;
import edu.uoc.pfc.business.transfers.TOUser;

/**
 * <p>Clase que recoge la informaci&oacute;n de las diferentes aulas dentro de la aplicaci&oacute;n.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class BOClassroom implements Serializable 
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Identificador del Aula.</p> */
	protected int id;
	/** <p>Plan de Estudios al que pertenece.</p> */
	protected BOCurriculum curriculum;
	/** <p>Asignatura que se imparte en el Aula.</p> */
	protected BOSubject subject;
	/** <p>Letra del Aula.</p> */
	protected String letter;
	/** <p>A&ntilde;o del Aula.</p> */
	protected int course;
	/** <p>Usuarios del Aula.</p> */
	protected Set<BOClassroomUser> users;
	/** <p>Cuestionarios asignados al Aula.</p> */
	protected Set<BOTestClassroom> test;
	
	// -------- CONSTANTES.
	/**
	 * <p>Crea un objeto <code>BOClassroom</code> e inicializa todos sus atributos a los
	 * valores por defecto.</p>
	 */
	public BOClassroom()
	{
		// Llamamos al constructor de la superclase.
		super();
	}
	
	/**
	 * <p>Crea un objeto <code>BOClassroom</code> e inicializa sus atributos a los valores
	 * de los par&aacute;metros.</p>
	 *  	
	 * @param curriculum <code>BOCurriculum</code> el Plan de Estudios del aula.
	 * @param subject <code>BOSubject</code> la Asignatura.
	 * @param letter <code>String</code> la letra del aula.
	 * @param course <code>int</code> el curso del aula.
	 */
	public BOClassroom(BOCurriculum curriculum, BOSubject subject,
			String letter, int course) {
		super();
		this.curriculum = curriculum;
		this.subject = subject;
		this.letter = letter;
		this.course = course;
	}

	// -------- MTODOS PBLICOS.
	/**
	 * <p>Retorna el objeto <code>TOClassroom</code> cuya informaci&oacute;n es equivalente
	 * a la que contiene el objeto <code>BOClassroom</code> sobre el que se invoca el m&eacute;todo.</p>
	 * 
	 * @return <code>TOClassroom</code> con la informaci&oacute;n equivalente al objeto
	 * sobre el que se invoca a este m&eacute;todo.
	 */
	public TOClassroom getTOClassroom()
	{
		// Creamos un objeto TOClassroom y lo retornamos.
		return (new TOClassroom(this.id, this.curriculum.getId(), this.curriculum.getName(), 
				this.subject.getId(), this.subject.getName(), this.letter, this.course));
	}
	
	/** 
	 * <p>Retorna un objeto <code>TOClassroom</code> con al informaci&oacute;n del Aula
	 * (incluyendo qui&eacute;n es el profesor y los aulumnos).</p>
	 * 
	 * @return <code>TOClassroom</code> con toda la informaci&oacute;n del Aula.
	 */
	public TOClassroom getTOClassroomComplete()
	{
		// Definimos una referencia al profesor.
		TOUser teacher = null;
		// Creamos una Lista de Alumnos.
		List<TOUser> students = new ArrayList<TOUser>();
		// Recorremos el conjunto de usuarios del aula.
		for (BOClassroomUser user : this.users)
		{
			// Vemos el rol del usuario.
			if (BOClassroomUser.ROLE_STUDENT.equals(user.getRole()))
			{
				// Se trata de un Alumno. Lo sacamos de la lista.
				TOUser toStudent = user.getTOUserPersonalData();
				// Indicamos que se deber ordenar por "name"
				toStudent.setOrderBy(TOUser.ORDER_BY_SURNAME);
				// Lo aadimos a la lista de alumnos.
				students.add(toStudent);
			}
			else
			{
				// Es el profesor. 
				teacher = user.getTOUserPersonalData();
			}
		}
		// Ordenamos la lista de alumnos.
		Collections.sort(students);
		// Creamos un objeto TOClassroom
		TOClassroom classroom = new TOClassroom(this.id, this.curriculum.getId(), this.curriculum.getName(), 
				this.subject.getId(), this.subject.getName(), this.letter, this.course,
				teacher, students);
		// Creamos una lista de objeto TOTest
		List<TOTest> list_test = new ArrayList<TOTest>();
		// Recorremos el conjunto de cuestionarios asignados al aula.
		for (BOTestClassroom test : this.test)
		{
			// Cada uno de los test lo aadimos a la lista como un objeto TOTest.
			list_test.add(test.getTOTest());
		}
		// Ordenamos la lista de objetos TOTest
		Collections.sort(list_test);
		// Asignamos la lista de Cuestionarios al Aula. 
		classroom.setTest(list_test);
		// Retornamos el objeto TOClassroom 
		return classroom;
	}
	
	// -------- GETTERS & SETTERS
	public int getId() 
	{
		return id;
	}
	public void setId(int id) 
	{
		this.id = id;
	}
	public BOCurriculum getCurriculum() 
	{
		return curriculum;
	}
	public void setCurriculum(BOCurriculum curriculum) 
	{
		this.curriculum = curriculum;
	}
	public BOSubject getSubject() 
	{
		return subject;
	}
	public void setSubject(BOSubject subject) 
	{
		this.subject = subject;
	}
	public String getLetter()
	{
		return letter;
	}
	public void setLetter(String letter)
	{
		this.letter = letter;
	}
	public int getCourse() 
	{
		return course;
	}
	public void setCourse(int course) 
	{
		this.course = course;
	}
	public Set<BOClassroomUser> getUsers() 
	{
		return users;
	}
	public void setUsers(Set<BOClassroomUser> users) 
	{
		this.users = users;
	}
	public Set<BOTestClassroom> getTest() 
	{
		return test;
	}
	public void setTest(Set<BOTestClassroom> test) 
	{
		this.test = test;
	}
	
}
