package edu.uoc.pfc.business.objects;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Set;

import edu.uoc.pfc.business.transfers.TOCurriculum;
import edu.uoc.pfc.business.transfers.TOCurriculumSubject;
import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Clase que se encargar de recoger la informaci&oacute;n de los diferentes planes
 * de estudio con los que se trabaja en la aplicaci&oacute;n.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class BOCurriculum implements Serializable
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Identificador del plan de estudios.</p> */
	protected int id;
	/** <p>Nombre del plan de estudios.</p> */
	protected String name;
	/** <p>Tipo de Asignatura.</p> */
	protected String type;
	/** <p>Descripci&oacute;n del plan de estudios.</p> */
	protected String description;
	/** <p>Fecha de creaci&oacute;n del plan de estudios.</p> */
	protected Date created;
	/** <p>Asignaturas que corresponden al plan de estudios.</p> */
	protected Set<BOSubjectCurriculum> subjects;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Constructor por defecto. Crea un objeto <code>BOCurriculum</code> e inicializa sus 
	 * atributos a los valores por defecto.</p>
	 */
	public BOCurriculum() 
	{
		// Llamamos al construtor de la superclase.
		super();
	}
	
	/**
	 * <p>Crea un objeto <code>BOCurriculum</code> e inicializa los valores de sus atributos a 
	 * sus hom&oacute;nimos que se le pasan como par&aacute;metro.</p>
	 *  
	 * @param name <code>String</code> nombre del Plan de Estudios.
	 * @param description <code>String</code> descripci&oacute;n del Plan de Estudios.
	 * @param type <code>String</code> tipo del Plan de Estudios.
	 */
	public BOCurriculum(String name, String description, String type)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Inicializamos los parmetros.
		this.name = WebUtils.stringToHTML(name);
		this.description = WebUtils.stringToHTML(description);
		this.type = type;
	}
	
	// -------- MTODOS PBLICOS
	/**
	 * <p>M&eeacute;todo que retorna un objeto <code>TOCurriculum</code> con la 
	 * informaci&oacute;n asociada al objeto <code>BOCurriculum</code> sobre el cual
	 * se invoca.</p>
	 * 
	 * @return <code>TOCurriculum</code> con la informaci&oacute;n de este objeto (this).
	 */
	public TOCurriculum getTOCurriculum()
	{
		// Construimos un objeto TOCurriculum a partir de la informacin de los atributos.
		return (new TOCurriculum(this.id, this.name, this.type, this.created, this.description));
	}
	
	/** 
	 * <p>Retorna un objeto <code>TOCurriculum</code> con la informaci&oacute;n completa del
	 * Plan de Estudios. Dicha informaci&oacute;n incluir&aacute; adem&aacute;s la lista de 
	 * asignaturas que est&aacute;n asociadas al Plan de Estudios.</p>
	 * 
	 * @return <code>TOCurriculum</code> con la informaci&oacute;n de este objeto (this).
	 */
	public TOCurriculum getTOCurriculumComplete()
	{
		// Creamos un objeto TOCurriculum a partir de la informacin de los parmetros.
		TOCurriculum curriculum = new TOCurriculum(this.id, this.name, this.type, this.created, this.description);
		// Creamos una lista que almacenar la informacin de las asignaturas del Plan de Estudios.
		List<TOCurriculumSubject> subjects = new ArrayList<TOCurriculumSubject>();
		// Recorremos el conjunto de asignaturas asignadas al Plan de Estudios.
		for (BOSubjectCurriculum bo : this.subjects)
		{
			// Creamos un objeto TOCurriculumSubject y lo aadimos a los 
			subjects.add(new TOCurriculumSubject(bo.getSubject().getId(), bo.getSubject().getName(), 
					bo.getCode(), bo.getCredits(), bo.getType()));
		}
		// Ordenamos la lista de asignaturas.
		Collections.sort(subjects);
		// Asignamos la lista de asignaturas a TOCurriculum.
		curriculum.setSubjects(subjects);
		// Retornamos el objeto TOCurriculum
		return curriculum;
	}
	
	
	// -------- GETTERS & SETTERS.
	public int getId() 
	{
		return id;
	}
	public void setId(int id) 
	{
		this.id = id;
	}
	public String getName() 
	{
		return name;
	}
	public void setName(String name) 
	{
		this.name = name;
	}
	public String getDescription() 
	{
		return description;
	}
	public void setDescription(String description) 
	{
		this.description = description;
	}
	public Date getCreated()
	{
		return created;
	}
	public void setCreated(Date created)
	{
		this.created = created;
	}
	public String getType() 
	{
		return type;
	}
	public void setType(String type) 
	{
		this.type = type;
	}
	public Set<BOSubjectCurriculum> getSubjects()
	{
		return subjects;
	}
	public void setSubjects(Set<BOSubjectCurriculum> subjects)
	{
		this.subjects = subjects;
	}
	
}
