package edu.uoc.pfc.business.objects;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Set;

import edu.uoc.pfc.business.transfers.TOSubject;
import edu.uoc.pfc.business.transfers.TOSubjectCurriculum;
import edu.uoc.pfc.lib.utils.DateUtils;
import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Clase que se encarga de recoger la informaci&oacute;n de las diferentes asignaturas que est&aacute;n
 * registradas dentro de la aplicaci&oacute;n.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class BOSubject implements Serializable
{

	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Identificador de la asignatura dentro de la aplicaci&oacute;n.</p> */
	protected int id;
	/** <p>Nombre de la asignatura.</p> */
	protected String name;
	/** <p>Fecha de creaci&oacute;n de la asignatura.</p> */
	protected Date created;
	/** <p>Descripci&oacute;n de la asignatura.</p> */
	protected String description;
	/** <p>Planes de Estudios donde se imparte la asignatura.</p> */
	protected Set<BOSubjectCurriculum> curriculum;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objecto <code>BOSubject</code>  e inicializa los atributos
	 * a sus valores por defecto.</p>
	 */
	public BOSubject()
	{
		// Llamamos al constructor de la superclase.
		super();
	}
	
	/**
	 * <p>Construye un objeto <code>BOSubject</code> a partir de la informaci&oacute;n recibida
	 * como par&aacute;tro.</p>
	 * 
	 * @param name <code>String</code> nombre de la asignatura.
	 * @param description <code>String</code> descripcin de la asignatura.
	 */
	public BOSubject(String name, String description)
	{
		// Llamamos al construtor de la superclase.
		super();
		// Asignamos los valoresa los atributos.
		this.name = WebUtils.stringToHTML(name);
		this.description = WebUtils.stringToHTML(description);
		// Asignamos a la fecha de creacin la fecha actual.
		this.created = DateUtils.now();
	}
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>Retorna un objeto <code>TOSubject</code> que contendr&aacute; la misma informaci&oacute;n
	 * que el objeto <code>BOSubject</code> sobre el que se invoca a este m&eacute;todo.</p>
	 * 
	 * @return <code>TOSubject</code> con la misma informaci&oacute;n que el objeto.
	 */
	public TOSubject getTOSubject()
	{
		// Creamos un objeto TOSubject a partir de la informacin de los atributos y lo retornamos.
		return (new TOSubject(this.id, this.name, this.created, this.description));
	}
	
	/**
	 * <p>Retorna un objeto <code>TOSubject</code> con toda la informaci&oacute;n de 
	 * que el objeto <code>BOSubject</code> sobre el cual se invoca incluyendo la informaci&oacute;n
	 * de los Planes de Estudio donde se imparte.</p>
	 * 
	 * @return <code>TOSubject</code> con la misma informaci&oacute;n que el objeto.
	 */
	public TOSubject getTOSubjectComplete()
	{
		// Creamos un objeto TOSubject.
		TOSubject subject = getTOSubject();
		// Creamos la lista de Planes de Estudio asociados a la asignatura.
		List<TOSubjectCurriculum> subjectCurriculum = new ArrayList<TOSubjectCurriculum>();
		// Recorremos la lista de Planes de Estudio.
		for (BOSubjectCurriculum bo : this.curriculum)
		{
			// Creamos un nuevo objeto TOSubjectCurriculum.
			TOSubjectCurriculum to = new TOSubjectCurriculum(bo.getCurriculum().getId(), 
						bo.getCurriculum().getName(), bo.getCode(), bo.getCredits(), 
						bo.getCurriculum().getType());
			// Almacenamos el elemento dentro de lista.
			subjectCurriculum.add(to);
		}
		// Ordenamos la lista de Planes de Estudio.
		Collections.sort(subjectCurriculum);
		// Asignamos la lista.
		subject.setCurriculum(subjectCurriculum);
		// Retornamos el objeto TOSubject
		return subject;
	}
	
	// -------- GETTERS & SETTERS.
	public int getId()
	{
		return id;
	}
	public void setId(int id)
	{
		this.id = id;
	}
	public String getName()
	{
		return name;
	}
	public void setName(String name)
	{
		this.name = name;
	}
	public String getDescription()
	{
		return description;
	}
	public void setDescription(String description)
	{
		this.description = description;
	}
	public Date getCreated()
	{
		return created;
	}
	public void setCreated(Date created)
	{
		this.created = created;
	}
	public Set<BOSubjectCurriculum> getCurriculum()
	{
		return curriculum;
	}
	public void setCurriculum(Set<BOSubjectCurriculum> curriculum)
	{
		this.curriculum = curriculum;
	}
	
}
