package edu.uoc.pfc.business.objects;

import java.io.Serializable;
import java.util.Date;

import edu.uoc.pfc.business.transfers.TOTestAnswer;
import edu.uoc.pfc.business.transfers.TOTestAnswerStats;
import edu.uoc.pfc.lib.utils.DateUtils;

/**
 * <p>Clase que modela las Respuestas que los Estudiantes realizan a los Cuestionarios que
 * tienen asignados a sus Aulas.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class BOTestAnswer implements Serializable 
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de la clase.</p> */
	private static final long serialVersionUID = 1L;
	/** <p>Nota final obtenida: Matr&iacute;cula de Honor.</p> */
	public static final String FINAL_SCORE_MATRICULA_HONOR = "M";
	/** <p>Nota final obtenida: Sobresaliente.</p> */
	public static final String FINAL_SCORE_SOBRESALIENTE = "EX";
	/** <p>Nota final obtenida: Notable.</p> */
	public static final String FINAL_SCORE_NOTABLE = "NO";
	/** <p>Nota final obtenida: Aprobado.</p> */
	public static final String FINAL_SCORE_APROBADO = "A";
	/** <p>Nota final obtenida: Suspenso.</p> */
	public static final String FINAL_SCORE_SUSPENSO = "SU";
	
	// -------- ATRIBUTOS.
	/** <p>Identificador de la respuesta.</p> */
	private int id;
	/** <p>El Estudiante que realiza la respuesta.</p> */
	private BOUser student;
	/** <p>La asignaci&ioacute;n del Cuestionario al Aula.</p> */
	private BOTestClassroom testClassroom;
	/** <p>Fecha de la respuesta.</p> */
	private Date answerDate;
	/** <p>Puntuaci&oacute;n obtenida en la respuesta.</p> */
	private double score;
	/** <p>Puntuaci&oacute;n m&aacute;xima que se pod&iacute;a obtener en el Cuestionario.</p>*/
	private double maxScore;
	/** <p>Nota final (en caracteres).</p> */
	private String finalScore;
	/** <p>N&uacute;mero total de Preguntas que ten&iacute;a el Cuestionario.</p> */
	private int totalQuestions;
	/** <p>N&uacute;mero de Preguntas acertadas por el Estudiante.</p> */
	private int correctQuestions;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objeto <code>BOTestAnswer</code> inicializando sus atributos a los valores
	 * por defecto.</p>
	 */
	public BOTestAnswer()
	{
		// Llamamos al constructor de la superclase.
		super();
	}
	
	/**
	 * <p>Construye un objeto <code>BOTestAnswer</code> a partir de la informaci&oacute;n que se le 
	 * pasa como par&aacute;metro.</p>
	 * 
	 * @param student <code>BOUser</code> el Estudiante que ha dado la respuesta.
	 * @param testClassroom <code>BOTestClass</code> la asignaci&oacute;n del Cuestionario a un Aula.
	 * @param score <code>double</code> la puntuaci&oacute;n obtenida (num&eacute;rica).
	 * @param maxScore <code>double</code> la puntuaci&oacute;n m&aacute;xima que se pod&iacute;a obtener.
	 * @param totalQuestions <code>int</code> n&uacute;mero total de preguntas del Cuestionario.
	 * @param correctQuestions <code>int</code> n&uacute;mero total de respuestas correctas.
	 */
	public BOTestAnswer(BOUser student, BOTestClassroom testClassroom, double score, double maxScore,
			int totalQuestions, int correctQuestions)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Establecemos los valores de los atributos.
		this.student = student;
		this.testClassroom = testClassroom;
		this.score = score;
		this.maxScore = maxScore; 
		this.totalQuestions = totalQuestions; 
		this.correctQuestions = correctQuestions;
		// Establecemos el valor de la fecha y hora de la respuesta.
		this.answerDate = DateUtils.now();
		// Establecemos el valor de la nota final en el test gracias a una regla de tres.
		double range = (score * 10.0) / maxScore;
		// Establecemos el valor de "finalScore" en funci&oacute;n de "range".
		if ((range >= 0) && (range < 5.0)) 
			this.finalScore = FINAL_SCORE_SUSPENSO;
		else if ((range >= 5.0) && (range < 7.0)) 
			this.finalScore = FINAL_SCORE_APROBADO;
		else if ((range >= 7.0) && (range < 9.0))
			this.finalScore = FINAL_SCORE_NOTABLE;
		else if ((range >= 9.0) && (range < 10.0))
			this.finalScore = FINAL_SCORE_SOBRESALIENTE;
		else 
			this.finalScore = FINAL_SCORE_MATRICULA_HONOR;
	}
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>Retorna un objeto <code>TOTestAnswer</code> cuya informaci&oacute;n es equivalente al objeto
	 * "this" sobre el que se ha invocado el m&eacute;todo.</p>
	 * 
	 * @return <code>TOTestAnswer</code> con la misma informaci&oacute;n que el objeto "this" sobre el
	 * cual se ha llamado a este m&eacute;todo.
	 */
	public TOTestAnswer getTOTestAnswer()
	{
		// Creamos un objeto TOTestAnswer a partir de la informacin de "this" y lo retornamos.
		return (new TOTestAnswer(this.id, this.answerDate, this.score, this.maxScore, this.finalScore, 
				this.totalQuestions, this.correctQuestions));
	}
	
	/**
	 * <p>Retorna un objeto <code>TOTestAnswerStats</code> cuya informaci&oacute;n es equivalente
	 * al objeto "this" sobre el que se invoca al m&eacute;todo.</p>
	 * 
	 * @return <code>TOTestAnswerStats</code> con la informaci&oacute;n equivalente al objeto
	 * sobre el cual se invoca a este m&eacute;todo.
	 */
	public TOTestAnswerStats getTOTestAnswerStats()
	{
		// Variable local que nos indicar si el Test est aprobado o no.
		boolean pass = true;
		// Comprobamos si el Test est suspenso.
		if (FINAL_SCORE_SUSPENSO.equals(this.finalScore)) pass = false;
		// Construimos un objeto TOTestAnswerStats y lo retornamos.
		return (new TOTestAnswerStats(pass, this.finalScore, this.score, this.maxScore));
	}
	
	// -------- GETTERS & SETTERS.
	public int getId() 
	{
		return id;
	}
	public void setId(int id) 
	{
		this.id = id;
	}
	public BOUser getStudent() 
	{
		return student;
	}
	public void setStudent(BOUser student) 
	{
		this.student = student;
	}
	public BOTestClassroom getTestClassroom() 
	{
		return testClassroom;
	}
	public void setTestClassroom(BOTestClassroom testClassroom) 
	{
		this.testClassroom = testClassroom;
	}
	public Date getAnswerDate() 
	{
		return answerDate;
	}
	public void setAnswerDate(Date answerDate) 
	{
		this.answerDate = answerDate;
	}
	public double getScore() 
	{
		return score;
	}
	public void setScore(double score) 
	{
		this.score = score;
	}
	public double getMaxScore() 
	{
		return maxScore;
	}
	public void setMaxScore(double maxScore) 
	{
		this.maxScore = maxScore;
	}
	public String getFinalScore() 
	{
		return finalScore;
	}
	public int getTotalQuestions()
	{
		return totalQuestions;
	}
	public void setTotalQuestions(int totalQuestions)
	{
		this.totalQuestions = totalQuestions;
	}
	public int getCorrectQuestions()
	{
		return correctQuestions;
	}
	public void setCorrectQuestions(int correctQuestions)
	{
		this.correctQuestions = correctQuestions;
	}
	public void setFinalScore(String finalScore)
	{
		this.finalScore = finalScore;
	}

}
