package edu.uoc.pfc.business.services;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.hibernate.Hibernate;
import org.hibernate.Query;
import org.hibernate.Session;

import edu.uoc.pfc.business.exceptions.AccountLockedException;
import edu.uoc.pfc.business.exceptions.BusinessRulesViolationException;
import edu.uoc.pfc.business.exceptions.DataStoreException;
import edu.uoc.pfc.business.exceptions.DuplicatedFieldException;
import edu.uoc.pfc.business.exceptions.InvalidIdException;
import edu.uoc.pfc.business.exceptions.SendEmailException;
import edu.uoc.pfc.business.interfaces.IBAccountManager;
import edu.uoc.pfc.business.objects.BOClassroomUser;
import edu.uoc.pfc.business.objects.BOConfig;
import edu.uoc.pfc.business.objects.BOQuestion;
import edu.uoc.pfc.business.objects.BOQuestionCategory;
import edu.uoc.pfc.business.objects.BOTest;
import edu.uoc.pfc.business.objects.BOTestAnswer;
import edu.uoc.pfc.business.objects.BOTestQuestion;
import edu.uoc.pfc.business.objects.BOUser;
import edu.uoc.pfc.business.transfers.TOUser;
import edu.uoc.pfc.lib.utils.DateUtils;
import edu.uoc.pfc.lib.utils.MailUtils;

/**
 * <p>Implementaci&oacute;n de la interfaz <code>IBAccountManager</code> por
 * lo que es la clase de la capa de negocio que ofrece a la capa de presentaci&oacute;n
 * todos los servicios relacionados con la gesti&oacute;n de cuentas de los
 * usuarios de la aplicaci&oacute;n.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0 
 * @see IBAccountManager
 */
public class IBAccountManagerImpl extends IBBaseImpl implements IBAccountManager 
{
	
	// -------- CONSTRUCTORES.
	/**
	 * <p><code>Construye un nuevo objeto <code>IBAccountManagerImpl</code> e inicializa el 
	 * marco de persistencia <i>Hibernate</i>.</p>
	 * @throws DataStoreException error al iniciar el marco de persistencia.
	 */
	public IBAccountManagerImpl() throws DataStoreException
	{
		// Constructor de la superclase.
		super();
	}
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado para crear una cuenta de usuario a 
	 * partir de la informaci&aocute;n recogida dentro del objeto <code>TOUser</code>
	 * que se le pasa como par&aacute;metro.</p>
	 * 
	 * @param user <code>TOUser</code> con la informaci&oacute;n para crear la cuenta.
	 * @return <code>TOUser</code> que contendr&aacute; la informaci&oacute;n completa
	 * de la cuenta del usuario una vez creada (incluir&aacute;, entre otras cosas, el
	 * identificador del usuario <i>id</i> dentro de la aplicaci&oacute;n).</p>
	 * @throws DataStoreException si se produce un error en el marco de persistencia.
	 * @throws DuplicatedFieldException si el login proporcionado est&aacute; repetido.
	 * @throws InvalidEmailException si el email proporcionado est&aacute; repetido.
	 * @see edu.uoc.pfc.business.interfaces.IBAccountManager
	 */
	@Override
	public TOUser createAccount(TOUser user) throws DataStoreException, DuplicatedFieldException 
	{
		// Comprobamos que el parmetro no es nulo.
		if (user == null) throw new NullPointerException("Parametro null.");
		// Abrimos la sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Vemos si el "login" est duplicado.
			if (duplicateField(session, Table.USERS, "login", user.getLogin()))
			{
				// Lanzamos una  InvalidLoginException.
				throw (new DuplicatedFieldException(DuplicatedFieldException.MSG_LOGIN_DUPLICATED));
			}
			else if (duplicateField(session, Table.USERS, "email", user.getEmail()))
			{
				// Dos usuarios con la misma direccin de correo electrnico.
				throw (new DuplicatedFieldException(DuplicatedFieldException.MSG_EMAIL_DUPLICATED));
			}
			// Si llegamos a este punto quiere decir que se podr crear la cuenta.
			// Creamos un objeto BOUse a partir del valor del parmetro.
			BOUser bo = new BOUser(user);
			// Establecemos la fecha de creacin.
			bo.setCreated(DateUtils.now());
			// Grabamos la informacin de la cuenta dentro de la base de datos. 
			session.saveOrUpdate(bo);
			// Consolidamos los cambios.
			session.flush();
			// Retornamos la informacin del usuario dentro del objeto TOUser.
			return bo.getTOUser();
		}
		catch (DuplicatedFieldException ile)
		{
			// Simplemente propagamos la excepcin.
			throw ile;
		}
		catch (Exception ex)
		{
			// Cualquier otro tipo de excepcin que se produzca la encapsulamos dentro
			// de una DataStoreException y la propagamos.
			throw (new DataStoreException(ex.getMessage()));
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado para cambiar el estado de la cuenta
	 * del usuario cuyo identificador se le pasa como parmetro. El sistema comprobar&aacute;
	 * que no la cuenta a cambiar no pertenece al &uacute;nico administrador del 
	 * sistema (siempre tiene que haber una cuenta de administrador activa).</p>
	 * 
	 * <p>Si el m&eeacute;todo se ejecuta correctamente el estado del usuario pasar&aacute;
	 * a ser el contrario del que ten&iacute;a antes de llamarlo. As&iacute; si el estado
	 * era "activo" pasar&aacute; a estar "inactivo" y viceversa.</p>
	 * 
	 * @param id_user <code>int</code> identificador del usuario.
	 * @return <code>TOUser</code> con la informaci&oacute;n del usuario una vez se le 
	 * ha cambiado el estado.
	 * @throws DataStoreException si se produce un error en el marco de persistencia.
	 * @throws InvalidIdException si el identificador de usuario que se ha recibido no
	 * pertenece a un usuario registrado en el sistema.
	 * @throws BusinessRulesViolationException si se intenta cambiar el estado del &uacute;nico
	 * administrador activo dentro de la aplicaci&oacute;n.
	 */
	@Override
	public TOUser changeState(int id_user) 
			throws DataStoreException, InvalidIdException, BusinessRulesViolationException 
	{
		// Abrimos la sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Vemos la existencia del usuario. Si existe 
			Object oUser = existFieldValue(session, Table.USERS, "id", id_user);
			// Vemos si es nulo o no y si es de tipo BOUser
			if ((oUser != null) && (oUser instanceof BOUser))
			{
				// Cast del resultado devuelto.
				BOUser bo = (BOUser) oUser;
				// Comprobamos que el usuario al menos es de un tipo.
				if ((!bo.isAdmin()) && (!bo.isTeacher()) && (!bo.isStudent()))
				{
					// Se est intentando activar una cuenta de usuario sin haberle 
					// asignado al menos un rol (se trata de una peticin de creacin
					// de cuenta que est pendiente) por lo que deberemos lanzar
					// una BusinessRulesViolationException
					throw (new BusinessRulesViolationException(BusinessRulesViolationException.MSG_MUST_HAVE_A_USER_TYPE));
				}
				// Vemos si el usuario es un administrador o no.
				if (bo.isAdmin())
				{
					// Se trata de un administrador del sistema. Vemos si es el nico activo.
					// Preparamos la sentencia HQL.
					Query query = session.createQuery("from BOUser where admin=? and active=?");
					// Establecemos los parmetros.
					query.setParameter(0, true, Hibernate.BOOLEAN);
					query.setParameter(1, true, Hibernate.BOOLEAN);
					// Ejecutamos la sentencia y comprobamos el nmero de registros.
					if (query.list().size() == 1)
					{
						// Se trata del nico administrador. Lanzamos una BusinessRulesViolationException
						throw (new BusinessRulesViolationException(BusinessRulesViolationException.MSG_ADMIN_CHANGE_STATE));
					}
				}
				//No es un administrador o no es el nico activo. Simplemente cambiamos su estado.
				bo.setActive(!bo.isActive());
				// Actualizamos el estado del usuario.
				session.saveOrUpdate(bo);
				// Consolidamos los cambios.
				session.flush();
				// Retornamos un objeto TOUser con la informacin actualizada.
				return bo.getTOUser();
			}
			// Si se llega a este punto querr decir que el identificador que se
			// ha facilitado no es vlido por lo que lanzamos una InvalidIdException
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_USER_ID));
		}
		catch (BusinessRulesViolationException brve)
		{
			// En este caso simplemente propagaremos la excepcin.
			throw brve;
		}
		catch (InvalidIdException iie)
		{
			// En este caso simplemente propagaremos la excepcin
			throw iie;
		}
		catch (Exception ex)
		{
			// Cualquier otro tipo de excepcin que se produzca la encapsulamos dentro
			// de una DataStoreException y la propagamos.
			throw (new DataStoreException(ex.getMessage()));
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}	
	
	/**
	 * <p>Borra del sistema la cuenta de usuario cuyo identificador se pasa en el par&aacute;metro
	 * <i>id</i>. El sistema comprobar&aacute; que se trata de un identificador v&aacute;lido 
	 * (pertenece a una cuenta) y adem&aacute;s que no es el identificador del &uacute;nico 
	 * administrador registrado.</p>
	 * 
	 * @param id_user <code>int</code> identificador del usuario.
	 * @throws DataStoreException si se produce un error en el marco de persistencia.
	 * @throws InvalidIdException si el identificador de usuario que se ha recibido no
	 * pertenece a un usuario registrado en el sistema.
	 * @throws BusinessRulesViolationException si se intenta borrar la cuenta del &uacute;nico 
	 * administrador registrador en el sistema.
	 */
	@Override
	public void deleteAccount(int id_user) throws DataStoreException,
			InvalidIdException, BusinessRulesViolationException
	{
		// Abrimos la sesin de Hibernate.
		Session session = sessionFactory.openSession();
		try
		{
			// Vemos si existe el usuario.
			Object oUser = existFieldValue(session, Table.USERS, "id", id_user);
			// Comprobamos que no es nulo y adems es de tipo BOUser.
			if ((oUser != null) && (oUser instanceof BOUser))
			{
				// Cast del objeto.
				BOUser bo = (BOUser) oUser;
				// Comprobamos si se trata de un administrador del sistema.
				if (bo.isAdmin())
				{
					// Comprobamos si se trata del nico administrador del sistema.
					// Creamos la sentencia HQL.
					Query query = session.createQuery("from BOUser where admin=? and active=?");
					// Establecemos los parmetros.
					query.setParameter(0, true, Hibernate.BOOLEAN);
					query.setParameter(1, true, Hibernate.BOOLEAN);
					// Ejecutamos la consulta y comprobamos el nmero de resultados.
					if (query.list().size() == 1)
					{
						// Se trata del nico administrador del sistema. Lanzamos una excepcin.
						throw (new BusinessRulesViolationException(BusinessRulesViolationException.MSG_ADMIN_DELETE));
					}
				}
				// En el caso de que el usuario sea un Profesor o un Alumno tenemos que la informacin
				// de las Aulas en las que participa.
				if ((bo.isTeacher()) || (bo.isStudent()))
				{
					// Obtenemos la lista de Aulas en las que participa el Usuario. 
					List<Object> list_classroom = getAll(session, Table.CLASSROOM_USER, "user", id_user);
					// Recorremos la lista de las Aulas.
					for (Object obj : list_classroom)
					{
						// Vemos si obj es de tipo BOClassroomUser
						if (obj instanceof BOClassroomUser)
						{
							// Eliminamos el objeto.
							session.delete((BOClassroomUser) obj);
							// Consolidamos la operacin.
							session.flush();
						}
					}
				}
				// En el caso de que sea un Profesor tenemos que borrar las categoras de preguntas
				// que haya creado en su Banco de Preguntas adems de la informacin de todos los
				// Cuestionarios que haya creado dentro del sistema.
				if (bo.isTeacher())
				{
					// Obtenemos la lista de categoras de preguntas.
					List<Object> list_categories = getAll(session, Table.QUESTION_CATEGORIES, "user", id_user);
					// Recorremos la lista de Categoras.
					for (Object obj : list_categories)
					{
						// Vemos si se trata de un objeto BOQuestionClassroom
						if (obj instanceof BOQuestionCategory)
						{
							// Cast de obj.
							BOQuestionCategory category = (BOQuestionCategory) obj;
							// Emiminamos las preguntas dentro de esa categora. 
							List<Object> list_objQuestions = getAll(session, Table.QUESTIONS, 
									"category", category.getId());
							// Recorremos la lista de preguntas.
							for (Object objQuestion : list_objQuestions)
							{
								// Comprobamos que es de tipo BOQuestion.
								if (objQuestion instanceof BOQuestion)
								{
									// Eliminamos la preguntas.
									session.delete((BOQuestion) objQuestion);
									// Consolidamos la operacin.
									session.flush();
								}
							}
							// Eliminamos el objeto
							session.delete(category);
							// Consolidamos la operacin.
							session.flush();
						}
					}
					// Obtenemos la lista de los Cuestionarios que ha creado.
					List<Object> list_obj_test = getAll(session, Table.TEST, "teacher", id_user);
					// Recorremos la lista de Cuestionarios.
					for (Object obj_test : list_obj_test)
					{
						// Comprobamos que obj_test es de tipo BOTest.
						if (obj_test instanceof BOTest)
						{
							// Cast del obj_test.
							BOTest test = (BOTest) obj_test;
							// Recorremos la lista de asignaciones de Preguntas.
							for (BOTestQuestion testQuestion : test.getQuestions())
							{
								// Borramos la asignacin.
								session.delete(testQuestion);
								// Consolidamos la operacin.
								session.flush();
							}
							// Borramos el Cuestionario.
							session.delete(test);
							// Consolidamos la operacin.
							session.flush();
						}
					}
				}
				// En el caso de que sea un Alumno tendremos que eliminar todas sus respuestas a Cuestionarios.
				if (bo.isStudent())
				{
					// Obtenemos la lista de de respuestas de Usuario. Sentencia HQL.
					Query query = session.createQuery("from BOTestAnswer where student=?");
					// Establecemos los valores de los parmetros.
					query.setParameter(0, bo.getId(), Hibernate.INTEGER);
					// Ejecutamos la consulta y recorremos los objetos que retorna.
					for (Object objTestAnswer : query.list())
					{
						// Comprobamos el tipo del objeto.
						if (objTestAnswer instanceof BOTestAnswer)
						{
							// Cast del objeto y lo eleminamos de la capa de persistencia.
							session.delete((BOTestAnswer) objTestAnswer);
							// Consolidamos la operacin.
							session.flush();
						}
					}
				}
				// Si llegamos a este punto se podr borrar la cuenta del usuario sin problemas.
				session.delete(bo);
				// Consolidamos el borrado.
				session.flush();
			}
			else 
			{
				// Si se llega a este punto querr decir que el identificador del usuario
				// que se ha pasado no es correcto por lo que lanzamos una InvalidIdException
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_USER_ID));
			}
		}		
		catch (InvalidIdException iie)
		{
			// En este caso simplemente propagamos la excepcin
			throw iie;
		}
		catch (BusinessRulesViolationException brve)
		{
			// En este caso simplemente propagamos la excepcin.
			throw brve;
		}
		finally 
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}

	/**
	 * <p>Env&iacute;a a la cuenta de correo electr&oacute;nico que se le ha pasado 
	 * como par&aacute;metro un email inform&aacute;ndole del nombre de usuario que 
	 * tiene en el sistema. En el caso de que la direcci&oacute;n de correo no corresponda
	 * con una cuenta del sistema este m&eeacute;todo no realizar&aacute; nada.</p>
	 * 
	 * @param email <code>String</code> direcci&oacute;n de correo electr&oacute;nico del 
	 * usuario para el cual se quiere recuperar su cuenta en el sistema.
	 * @throws DataStoreException su se produce alg&uacute;n error a la hora de acceder
	 * a la capa de persistencia para obtener la informaci&oacute;n.
	 * @throws BusinessRulesViolationException si se intenta recuperar el nombre de usuario
	 * pero est&aacute; deshabilitado en env&iacute;o de correos electr&oacute;nicos dentro
	 * de la aplicaci&oacute;n.
	 * @throws SendEmailException si se produce un error al enviar el correo electr&oacute;nico
	 * con la informaci&oacute;n del nombre de usuario.
	 */
	@Override
	public void forgotLogin(String email) throws DataStoreException, 
			BusinessRulesViolationException, SendEmailException
	{
		// Abrimos la sesin de Hibernate.
		Session session = sessionFactory.openSession();
		// Vemos si existe el usuario.
		Object oUser = existFieldValue(session, Table.USERS, "email", email);
		// Obtenemos la informacin de la configuracin activa.
		Object oConfig = existFieldValue(session, Table.CONFIG, "active", true);
		// Comprobamos que no es nulo y las clases a las que pertenecen.
		if ((oUser != null) && (oUser instanceof BOUser) && (oConfig != null) && (oConfig instanceof BOConfig))
		{
			// Cast al objeto con las informacin de usuario.
			BOUser user = (BOUser) oUser;
			// Cast al objeto con la informacin de configuracin.
			BOConfig config = (BOConfig) oConfig;
			// Comprobamos que s se que est activado el envo de email.
			if (!config.isSendEmail())
			{
				// No est habilitado el envo de correos electrnicos. 
				// Lanzamos una BusinessRulesViolationException
				throw (new BusinessRulesViolationException(BusinessRulesViolationException.MSG_FORGOT_LOGIN));
			}
			try
			{
				// Si llegamos a este punto es que s que se puede enviar el correo.
				MailUtils.sendSMTPEmail(config.getSmtpServer(), config.getSmtpPort(), config.isSmtpTLS(), 
						config.isSmtpAuthentication(), config.getSmtpUser(), config.getSmtpPassword(), 
						config.getForgotLoginFrom(), email, config.getForgotLoginSubject(), 
						config.getForgotLoginText() + " " + user.getLogin());
			}
			catch (Exception e)
			{
				// Cualquier tipo de excepcin que se produzca la encapsulamos dentro de 
				// una SendEmailException y la propagamos.
				throw (new SendEmailException(e.getMessage()));
			}
			finally
			{
				// Cerramos la sesin de Hibernate.
				closeSession(session);
			}
		}
	}
	
	/**
	 * <p>Env&iacute;a a la cuenta de correo electr&oacute;nico que se le ha pasado 
	 * como par&aacute;metro un email inform&aacute;ndole de la contrase&ntilde;a que 
	 * tiene en el sistema. En el caso de que la direcci&oacute;n de correo no corresponda
	 * con una cuenta del sistema este m&eeacute;todo no realizar&aacute; nada.</p>
	 * 
	 * @param email <code>String</code> direcci&oacute;n de correo electr&oacute;nico del 
	 * usuario para el cual se quiere recuperar su contrase&ntilde;a en el sistema.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de acceder
	 * a la capa de persistencia para obtener la informaci&oacute;n.
	 * @throws BusinessRulesViolationException si se intenta recuperar la contrase&ntilde;a
	 * pero est&aacute; deshabilitado en env&iacute;o de correos electr&oacute;nicos dentro
	 * de la aplicaci&oacute;n.
	 * @throws SendEmailException si se produce un error al enviar el correo electr&oacute;nico
	 * con la informaci&oacute;n de la contrase&ntilde;a.
	 */
	@Override
	public void forgotPassword(String email) throws DataStoreException,
			BusinessRulesViolationException, SendEmailException
	{
		// Abrimos la sesin de Hibernate.
		Session session = sessionFactory.openSession();
		// Vemos si existe el usuario.
		Object oUser = existFieldValue(session, Table.USERS, "email", email);
		// Obtenemos la informacin de la configuracin activa.
		Object oConfig = existFieldValue(session, Table.CONFIG, "active", true);
		// Comprobamos que no es nulo y las clases a las que pertenecen.
		if ((oUser != null) && (oUser instanceof BOUser) && (oConfig != null) && (oConfig instanceof BOConfig))
		{
			// Cast al objeto con las informacin de usuario.
			BOUser user = (BOUser) oUser;
			// Cast al objeto con la informacin de configuracin.
			BOConfig config = (BOConfig) oConfig;
			// Comprobamos que s se que est activado el envo de email.
			if (!config.isSendEmail())
			{
				// No est habilitado el envo de correos electrnicos. 
				// Lanzamos una BusinessRulesViolationException
				throw (new BusinessRulesViolationException(BusinessRulesViolationException.MSG_FORGOT_PASSWORD));
			}
			try
			{
				// Si llegamos a este punto es que s que se puede enviar el correo.
				MailUtils.sendSMTPEmail(config.getSmtpServer(), config.getSmtpPort(), config.isSmtpTLS(), 
						config.isSmtpAuthentication(), config.getSmtpUser(), config.getSmtpPassword(), 
						config.getForgotPasswordFrom(), email, config.getForgotPasswordSubject(), 
						config.getForgotPasswordText() + " " + user.getPassword());
			}
			catch (Exception e)
			{
				// Cualquier tipo de excepcin que se produzca la encapsulamos dentro de 
				// una SendEmailException y la propagamos.
				throw (new SendEmailException(e.getMessage()));
			}
			finally
			{
				// Cerramos la sesin de Hibernate.
				closeSession(session);
			}
		}		
	}
	
	/**
	 * <p>Retorna un objeto <code>TOUser</code> que contendr&aacute; la informaci&aocute;n de
	 * la cuenta de un usuario cuyo identificador se le ha pasado como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> el identificador del usuario.
	 * @return <code>TOUser</code> con la inforamci&oacute;n de la cuenta del usuario cuyo
	 * <i>id</i> es igual al que se ha recibido como par&aacute;metro.
	 * @throws DataStoreException si se produce una excepci&oacute;n al obtener la informaci&oacute;n
	 * de la capa de persistencia.
	 * @throws InvalidIdException si el identificador que se ha pasado no corresponde con el
	 * de ninguna de las cuentas de usuarios registradas en el sistema.
	 */
	@Override
	public TOUser getAccount(int id) throws DataStoreException,
			InvalidIdException
	{
		// Inicializamos la sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Obtenemos el usuario.
			Object oUser = existFieldValue(session, Table.USERS, "id", id);
			// Vemos si existe el usuario.
			if ((oUser != null) && (oUser instanceof BOUser))
			{
				// Cast del objeto.
				BOUser user = (BOUser) oUser;
				// Simplemente retornamos la informacin del usuario encapsulada
				// dentro de un objeto TOUser.
				return user.getTOUser();
			}
			// Si llegamos a este punto querr decir que no existe el usuario
			// por lo que lanzaremos una InvalidIdException.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_USER_ID));
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado por un administrador del sistema cuando desee
	 * modificar la informa&oacute;n de la cuenta de usuario cuyo identificador se le pasa 
	 * en el par&aacute;metro <i>id</i>.</p>
	 * 
	 * @param id <code>int</code> identificador de la cuenta de usuario.
	 * @param password <code>String</code> nueva contrase&ntilde;a para la cuenta.
	 * @param isAdmin <code>boolean</code> indica si es un administrador o no.
	 * @param isTeacher <code>boolean</code> indica si es un profesor o no.
	 * @param isStudent <code>boolean</code> indica si es un alumno o no.
	 * @param isActive <code>boolean</code> indica si la cuenta est&aacute; activa o no.
	 * @param name <code>String</code> nombre real del usuario.
	 * @param surname <code>String</code> apellidos del usuario.
	 * @return <code>TOUser</code> con toda la informaci&aocute;n de la cuenta del usuario
	 * ya actualizada.
	 * @throws DataStoreException si se produce un error a la hora de realizar la operaci&oacute;n
	 * dentro del marco de persistencia.
	 * @throws InvalidIdException si el identificador que se ha pasado no corresponde con el
	 * de ninguna de las cuentas de usuarios registradas en el sistema.
	 * @throws BusinessRulesViolationException si se viola alguna regla de negocio de la
	 * aplicaci&oacute; si se permitiese realizar la operaci&oacute;n.
	 */
	@Override
	public TOUser modifyAccount(int id, String password, boolean isAdmin, boolean isTeacher,
			boolean isStudent, boolean isActive, String name, String surname) 
			throws DataStoreException, InvalidIdException, BusinessRulesViolationException
	{
		// Abrimos la sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Vemos si existe el usuario.
			Object oUser = existFieldValue(session, Table.USERS, "id", id);
			// Comprobamos que no es nulo y adems es de tipo BOUser
			if ((oUser != null) && (oUser instanceof BOUser))
			{		
				// Cast de oUser.
				BOUser user = (BOUser) oUser;
				// Comprobamos si se trata de un administrador y se quiere desactivar.
				if ((user.isAdmin()) && (!isActive))
				{
					// Tenemos que ver cuntos administradores hay en el sistema activos.
					Query query = session.createQuery("from BOUser where admin=? and active=?");
					// Establecemos los parmetros.
					query.setParameter(0, true, Hibernate.BOOLEAN);
					query.setParameter(1, true, Hibernate.BOOLEAN);
					// Ejecutamos la consulta y vemos el nmero de objetos que retorna.
					if (query.list().size() == 1)
					{
						// Se est intentando desactivar al nico administrador.
						// Lanzamos una BusinessViolationException.
						throw (new BusinessRulesViolationException(BusinessRulesViolationException.MSG_ADMIN_CHANGE_STATE));
					}
				}
				else if ((user.isAdmin()) && (!isAdmin))
				{
					// En este caso se quiere cambiar la condicin de un administrador, es
					// decir, que el usuario deje de ser un administrador del sistema.
					// Tenemos que ver cuntos administradores hay en el sistema.
					List<Object> admins = getAll(session, Table.USERS, "admin", true);
					// Vemos el nmero de elementos retornados.
					if (admins.size() == 1)
					{
						// No se puede cambiar el rol del nico administrador del sistema
						// para que deje de serlo, por lo que lanzamos una BusinessRulesViolationException.
						throw (new BusinessRulesViolationException(BusinessRulesViolationException.MSG_ADMIN_CHANGE_ROLE_ADMIN));
					}
				}
				// Comprobamos si el usuario es un Profesor deja de serlo.
				if (user.isTeacher() && (!isTeacher))
				{
					// El usuario deja de ser un Profesor. Tenemos que desasignarlo de 
					// todas las Aulas en las que sea un profesor. Sentencia HQL
					Query query = session.createQuery("from BOClassroomUser where user=? and role=?");
					// Establecemos los parmetros.
					query.setParameter(0, user.getId(), Hibernate.INTEGER);
					query.setParameter(1, BOClassroomUser.ROLE_TEACHER, Hibernate.STRING);
					// Ejecutamos la sentencia y recorremos los resultados.
					for (Object obj : query.list())
					{
						// Comprobamos que el objeto es de tipo BOClassroomUser
						if (obj instanceof BOClassroomUser)
						{
							// Eliminamos la asignacin del Usuario como Profesor.
							session.delete((BOClassroomUser) obj);
							// Confirmamos la operacin.
							session.flush();
						}
					}
					// Tambin deberemos de eliminar todas las categoras de Preguntas 
					// que hubiese creado dentro de su banco de Preguntas.
					// Obtenemos la lista de categoras de preguntas.
					List<Object> list_categories = getAll(session, Table.QUESTION_CATEGORIES, "user", user.getId());
					// Recorremos la lista de Categoras.
					for (Object obj : list_categories)
					{
						// Vemos si se trata de un objeto BOQuestionClassroom
						if (obj instanceof BOQuestionCategory)
						{
							// Cast de obj.
							BOQuestionCategory category = (BOQuestionCategory) obj;
							// Emiminamos las preguntas dentro de esa categora. 
							List<Object> list_objQuestions = getAll(session, Table.QUESTIONS, 
									"category", category.getId());
							// Recorremos la lista de preguntas.
							for (Object objQuestion : list_objQuestions)
							{
								// Comprobamos que es de tipo BOQuestion.
								if (objQuestion instanceof BOQuestion)
								{
									// Eliminamos la preguntas.
									session.delete((BOQuestion) objQuestion);
									// Consolidamos la operacin.
									session.flush();
								}
							}
							// Eliminamos el objeto
							session.delete((BOQuestionCategory) obj);
							// Consolidamos la operacin.
							session.flush();
						}
					}
					// Obtenemos la lista de los Cuestionarios que ha creado.
					List<Object> list_obj_test = getAll(session, Table.TEST, "teacher", id);
					// Recorremos la lista de Cuestionarios.
					for (Object obj_test : list_obj_test)
					{
						// Comprobamos que obj_test es de tipo BOTest.
						if (obj_test instanceof BOTest)
						{
							// Cast del obj_test.
							BOTest test = (BOTest) obj_test;
							// Recorremos la lista de asignaciones de Preguntas.
							for (BOTestQuestion testQuestion : test.getQuestions())
							{
								// Borramos la asignacin.
								session.delete(testQuestion);
								// Consolidamos la operacin.
								session.flush();
							}
							// Borramos el Cuestionario.
							session.delete(test);
							// Consolidamos la operacin.
							session.flush();
						}
					}
				}
				// Comprobamos si el Usuario es un Alumno y deja de serlo.
				if (user.isStudent() && (!isStudent))
				{
					// El usuario deja de ser un Alumno. Tenemos que desasignarlo de todas
					// las aUlas en las que participe como Alumno. Sentencia HQL.
					Query query = session.createQuery("from BOClassroomUser where user=? and role=?");
					// Establecemos los parmetros.
					query.setParameter(0, user.getId(), Hibernate.INTEGER);
					query.setParameter(1, BOClassroomUser.ROLE_STUDENT, Hibernate.STRING);
					// Ejecutamos la sentencia y recorremos los resultados.
					for (Object obj : query.list())
					{
						// Comprobamos que el objeto es de tipo BOClassroomUser
						if (obj instanceof BOClassroomUser)
						{
							// Eliminamos la asignacin del Usuario como Alumno.
							session.delete((BOClassroomUser) obj);
							// Confirmamos la operacin.
							session.flush();
						}
					}
					// Obtenemos la lista de de respuestas de Usuario. Sentencia HQL.
					query = session.createQuery("from BOTestAnswer where student=?");
					// Establecemos los valores de los parmetros.
					query.setParameter(0, user.getId(), Hibernate.INTEGER);
					// Ejecutamos la consulta y recorremos los objetos que retorna.
					for (Object objTestAnswer : query.list())
					{
						// Comprobamos el tipo del objeto.
						if (objTestAnswer instanceof BOTestAnswer)
						{
							// Cast del objeto y lo eleminamos de la capa de persistencia.
							session.delete((BOTestAnswer) objTestAnswer);
							// Consolidamos la operacin.
							session.flush();
						}
					}
				}
				
				// Llegados a este punto tenemos que actualizar la informacin.
				// Establecemos el nuevo password.
				user.setPassword(password);
				// Establecemos si es un administrador.
				user.setAdmin(isAdmin);
				// Establecemos si es un profesor.
				user.setTeacher(isTeacher);
				// Establecemos si es un alumno.
				user.setStudent(isStudent);
				// Establecemos si est activo.
				user.setActive(isActive);
				// Establecemos el nombre real.
				user.setName(name);
				// Establecemos los apellidos.
				user.setSurname(surname);
				// Eliminamos los espacios en blanco de todas las propiedades.
				edu.uoc.pfc.lib.utils.BeanUtils.trimProperties(user);
				// Actualizamos la informacin del usuario.
				session.saveOrUpdate(user);
				// Consolidamos la transaccin.
				session.flush();
				// Retornamos un objeto TOUser con la informacin actualizada.
				return user.getTOUser();
			}
			// Se llegamos a este punto querr decir que se se ha pasado 
			// un identificador de usuario invlido. Lanzamos una excepcin.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_USER_ID));
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado cuando se quiera modificar al informaci&aocute;n
	 * de la cuenta correspondiente al usuario <i>user</i> con los datos que se pasan como
	 * par&aacute;metros.</p>
	 * 
	 * @param user <code>TOUser</code> el usurio a modificar.
	 * @param password <code>String</code> la nueva contrase&ntilde;a de la cuenta.
	 * @param name <code>String</code> el nombre real del usuario.
	 * @param surname <code>String</code> los apellidos del usuario.
	 * @return <code>TOUser</code> con la informaci&oacute;n modificada del usuario.
	 * @throws DataStoreException si se produce un error a la hora de realizar la operaci&oacute;n
	 * dentro del marco de persistencia.
	 * @throws InvalidIdException si el objeto <code>TOUser</code> que se ha pasado como parmetro
	 * no corresponde con nunguna de las cuentas registradas dentro del sistema.
	 */
	@Override
	public TOUser modifyAccount(TOUser user, String password, 
			String name, String surname)
			throws DataStoreException, InvalidIdException
	{
		// Comprobamos que el parmetro TOUser no es nulo. Si es as lanzamos una NullPointerException.
		if (user == null) throw new NullPointerException();
		// Abrimos la sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Vemos si existe el usuario con el "id" de TOUser.
			Object oUser = existFieldValue(session, Table.USERS, "id", user.getId());
			// Comprobamos que no sea null y adems de tipo BOUser.
			if ((oUser != null) && (oUser instanceof BOUser))
			{
				// Cast del usuario.
				BOUser boUser = (BOUser) oUser;
				// Establecemos la nueva contrasea.
				boUser.setPassword(password);
				// Establecemos el nombre real.
				boUser.setName(name);
				// Establecemos los apellidos.
				boUser.setSurname(surname);
				// Actualizamos la informacin de usuario.
				session.saveOrUpdate(boUser);
				// Consolidamos la operacin.
				session.flush();
				// Retornamos la informacin del usuario una vez modificada.
				return boUser.getTOUser();
			}
			// Si llegamos a este punto querr decir que no existe el usuario por lo 
			// lanzamos una InvalidIdException.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_USER_ID));
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>Retorna un objeto <code>List</code> formada por objetos de tipo <code>TOUser</code>
	 * donde cada uno de los elementos contendr&aacute; la informaci&oacute;n de una cuenta
	 * de usuario que est&aacute; registrada en el sistema (independientemente del estado
	 * en el que se encuentre).</p>
	 * 
	 * @return <code>List&lt;TOUser&gt;</code>
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de acceder
	 * a la capa de persistencia para obtener la informac&oacute;n.
	 */
	@Override
	public List<TOUser> listAccount() throws DataStoreException
	{
		// Abrimos una sesin de Hibernate.
		Session session = sessionFactory.openSession();
		try
		{
			// Obtenemos la lista de todos los usuarios registrados.
			List<Object> list = getAll(session, Table.USERS);
			// Construimos la lista con los objetos que sern retornados.
			List<TOUser> users = new ArrayList<TOUser>();
			// Recorremos todos los usuarios.
			for (Object obj : list)
			{
				// Comprobamos que el objeto es de tipo BOUser.
				if (obj instanceof BOUser)
				{
					// Realizamos el cast del objeto
					BOUser bo = (BOUser) obj;
					// Aadimos a la lista el TOUser equivalente a BOUser.
					users.add(bo.getTOUser());
				}
			}
			// Una vez tenemos todos los usuarios los ordenamos por "id"
			Collections.sort(users);
			// Retornamos la lista de usuarios.
			return users;
		}
		catch (Exception e)
		{
			// Cualquier tipo de excepcin que se produzca la encapsulamos
			// dentro de una DatastoreException.
			throw new DataStoreException(e.getMessage());
		}
		finally
		{
			// Cerramos la sesin de Hibernate
			closeSession(session);
		}
	}

	/**
	 * <p>Retorna un objeto <code>List</code> formada por objetos de tipo <code>TOUser</code>
	 * donde cada uno de los elementos contendr&aacute; la informaci&oacute;n de una cuenta
	 * de usuario que est&aacute; registrada en el sistema (independientemente del estado
	 * en el que se encuentre).</p>
	 * 
	 * @return <code>List&lt;TOUser&gt;</code>
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de acceder
	 * a la capa de persistencia para obtener la informac&oacute;n.
	 */
	@Override
	public List<TOUser> getAccounts() throws DataStoreException
	{
		// Abrimos una sesin de Hibernate.
		Session session = sessionFactory.openSession();
		try
		{
			// Obtenemos la lista de todos los usuarios registrados.
			List<Object> list = getAll(session, Table.USERS);
			// Construimos la lista con los objetos que sern retornados.
			List<TOUser> users = new ArrayList<TOUser>();
			// Recorremos todos los usuarios.
			for (Object obj : list)
			{
				// Comprobamos que el objeto es de tipo BOUser.
				if (obj instanceof BOUser)
				{
					// Realizamos el cast del objeto
					BOUser bo = (BOUser) obj;
					// Aadimos a la lista el TOUser equivalente a BOUser.
					users.add(bo.getTOUser());
				}
			}
			// Una vez tenemos todos los usuarios los ordenamos por "id"
			Collections.sort(users);
			// Retornamos la lista de usuarios.
			return users;
		}
		catch (Exception e)
		{
			// Cualquier tipo de excepcin que se produzca la encapsulamos
			// dentro de una DatastoreException.
			throw new DataStoreException(e.getMessage());
		}
		finally
		{
			// Cerramos la sesin de Hibernate
			closeSession(session);
		}
	}
	
	/**
	 * <p>Retonar una lista de objetos <code>TOUser</code> donde cada uno de ellos
	 * representa a un usuario registrado como un Profesor dentro de la aplicaci&oacute;n.</p>
	 * 
	 * @return <code>List&lt;TOUser&gt;</code> con todos los Profesores.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de acceder
	 * a la capa de persistencia para obtener la informac&oacute;n.
	 */
	@Override
	public List<TOUser> getTeachers() throws DataStoreException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Creamos la lista de objetos TOUser.
			List<TOUser> teacher_list = new ArrayList<TOUser>();
			// Obtenemos la lista de Profesores registrados.
			List<Object> list_users = getAll(session, Table.USERS, "teacher", true);
			// Recorremos la lista de Profesores.
			for (Object user : list_users)
			{
				// Comprobamos que "user" es de tipo BOUser.
				if (user instanceof BOUser)
				{
					// Cast del Usuario.
					BOUser teacher = (BOUser) user;
					// Aadimos el usuarios a la lista de profesores.
					teacher_list.add(teacher.getTOUser());
				}
			}
			// Ordenamos la lista de Profesores.
			Collections.sort(teacher_list);
			// Retornamos la lista de usuarios.
			return teacher_list;
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>Retonar una lista de objetos <code>TOUser</code> donde cada uno de ellos
	 * representa a un usuario registrado como un Alumno dentro de la aplicaci&oacute;n.</p>
	 * 
	 * @return <code>List&lt;TOUser&gt;</code> con todos los Alumnos.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de acceder
	 * a la capa de persistencia para obtener la informac&oacute;n.
	 */
	public List<TOUser> getStudents() throws DataStoreException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Creamos la lista de objetos TOUser.
			List<TOUser> student_list = new ArrayList<TOUser>();
			// Obtenemos la lista de Alumnos registrados.
			List<Object> list_users = getAll(session, Table.USERS, "student", true);
			// Recorremos la lista de Alumnos.
			for (Object user : list_users)
			{
				// Comprobamos que "user" es de tipo BOUser.
				if (user instanceof BOUser)
				{
					// Cast del Usuario.
					BOUser student = (BOUser) user;
					// Aadimos el usuarios a la lista de alumnos.
					student_list.add(student.getTOUser());
				}
			}
			// Ordenamos la lista de Alumnos.
			Collections.sort(student_list);
			// Retornamos la lista de usuarios.
			return student_list;
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eeacute;to que ser&aacute; llamado desde la capa de presentaci&oacute;n en el
	 * momento en que un usuario quiera acceder a su cuenta dentro del sistema.</p>
	 * 
	 * @param login <code>String</code> nombre del usuario (login).
	 * @param password <code>String</code> contrase&ntilde;a de acceso.
	 * @return <code>TOUser</code> con la informaci&oacute;n del usuario dentro del sistema.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de acceder
	 * a la capa de persistencia para obtener la informac&oacute;n.
	 * @throws InvalidIdException si los datos proporcionados no corresponden con ninguna
	 * de las cuentas del sistema.
	 * @throws AccountLockedException si la cuenta del usuario no est&aacute; activa.
	 */
	@SuppressWarnings("unchecked")
	@Override
	public TOUser signin(String login, String password) 
			throws DataStoreException, InvalidIdException, AccountLockedException
	{
		// Comprobamos que los parmetros no sean null. Si as fuese lanzamos una NullPointerException
		if ((login == null) || (password == null)) throw (new NullPointerException());
		// Abrimos la sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Creamos la sentencia HQL
			Query query = session.createQuery("from BOUser where login=? and password=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, login, Hibernate.STRING);
			query.setParameter(1, password, Hibernate.STRING);
			// Obtenemos la lista de resultados.
			List<BOUser> results = query.list();
			// Ejecutamos la consulta y vemos el nmero de resultados devueltos.
			if (results.size() == 1)
			{
				// Obtenemos el usuario.
				BOUser user = results.get(0);
				// Comprobamos si la cuenta de usuario est activa o no.
				if (user.isActive())
				{
					// Establecemos la fecha y hora de acceso.
					user.setLastLogin(DateUtils.now());
					// Guardamos la informacin de la fecha y hora.
					session.saveOrUpdate(user);
					// Confirmamos la transaccin
					session.flush();
					// Retornamos el objeto que representa al usuario.
					return user.getTOUser();
				}
				else
				{
					// El usuario no est activo por lo que lanzaremos una AccountLockedException
					throw (new AccountLockedException(AccountLockedException.ACCOUNT_LOCKED));
				}
			}
			// Si llegamos a este punto querr decir que los datos proporcionados
			// no corresponden a ninguna de las cuentas de usuario en el sistema.
			// Lanzamos una InvalidIdException.
			throw (new InvalidIdException(InvalidIdException.MSG_NO_USER_ACCOUNT));
		}
		finally
		{
			// Cerramos la sesin de Hibernate
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eeacute;todo que ser&acute; llamado desde la capa de presentaci&oacute;n 
	 * en el momento en que un usuario desee salir de su cuenta.</p>
	 * 
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora
	 * de acceder a la capa de persistencia para obtener la informacin.
	 */
	@Override
	public void signout() throws DataStoreException
	{
		// En la implementacin que hemos realizado este mtodo no tiene
		// que realizar nada ya que no hay ninguna informacin que se tenga
		// que gestionar dentro del sistema cuando un usuario finaliza la
		// su sesin dentro de la aplicacin (cuenta de usuario).
	}
	
}
