package edu.uoc.pfc.business.services;

import java.util.ArrayList;
import java.util.List;

import org.hibernate.Hibernate;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;

import edu.uoc.pfc.business.exceptions.DataStoreException;
import edu.uoc.pfc.lib.utils.ValidationUtils;

/**
 * <p>Clase abstracta de la cual heredar&aacute;n el resto de clases que implementan las
 * diferentes interfaces con los m&eeacute;todos que la capa de negocio ofrece a la capa
 * de presentaci&oacute;n.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0 
 */
public abstract class IBBaseImpl
{
	
	// -------- ENUMS
	/** <p>Tablas dentro de la base de datos.</p> */
	public enum Table {
		// Componentes.
		CLASSROOM("BOClassroom"),					// Objeto: BOClassroom
		CLASSROOM_USER("BOClassroomUser"),			// Objeto: BOClassroomUser
		CONFIG("BOConfig"),							// Objeto: BOConfig
		CURRICULUM("BOCurriculum"),					// Objeto: BOCurriculum
		QUESTIONS("BOQuestion"),					// Obteto: BOQuestion
		QUESTION_CATEGORIES("BOQuestionCategory"),	// Objeto: BOQuestionCategory
		SUBJECTS("BOSubject"),						// Objeto: BOSubjects
		SUBJECTS_CURRICULUM("BOSubjectCurriculum"),	// Objeto: BOSubjectCurriculum
		TEST("BOTest"),								// Objeto: BOTest
		USERS("BOUser");							// Objeto: BOUsers
		
		// Atributos.
		private String tableName;
		
		// Constructores
		/** 
		 * <p>Construye un <i>enum</i> y le asigna al atributo <i>tableName</i>
		 * el valor que recibe como par&aacute;metro.</p>
		 * @param tableName <code>String</code> nombre de la tabla. 
		 */
		Table(String tableName)
		{
			// Asignamos el valor del atributo.
			this.tableName = tableName;
		}
		
		// -------- MTODOS PBLICOS
		@Override
		public String toString()
		{
			return this.tableName;
		}
	}
	
	// -------- ATRIBUTOS DE LA CLASE.
	/** <p>SessionFactory</code> de <i>Hibernate</i>.</p> */
	protected SessionFactory sessionFactory;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Inicializa el marco de persistencia <i>Hibernate</i>.</p>
	 * 
	 * @throws DataStoreException si se produce una excepci&oacute;n a la hora de iniciar
	 * el marco de persistencia.
	 */
	public IBBaseImpl() throws DataStoreException
	{
		// Constructor de la superclase.
		super();
		try
		{
			// Creamos el objeto SessionFactory.
			sessionFactory = new Configuration().configure().buildSessionFactory();
		}
		catch (Exception ex)
		{
			// Se produce una excepcin. La encapsulamos dentro de una DataStoreException
			// y la lanzamos.
			throw new DataStoreException(ex.getMessage());
		}
	}
	
	// -------- MTODOS PROTEGIDOS.
	/**
	 * <p>Comprueba que dentro de la tabla correspondiente al enum <i>table</i> ya existe
	 * alg&uacute;n registro cuyo valor en el campo <i>field</i> sea igual a <i>value</i>. Para
	 * que el m&eacute;todo funcione correctamente ser&aacute; necesario que previamente la 
	 * sesi&oacute;n de Hibernate haya sido inicializada (si no fuese as&iacute; el m&eeacute;todo
	 * retornar&aacute; <code>true</code>).</p>
	 * 
	 * @param session <code>Session</code> sesi&oacute;n de Hibernate.
	 * @param clazz <code>Table</code> tabla donde buscar.
	 * @param field <code>String</code> campo a comprobar.
	 * @param value <code>String</code> valor a comprobar.
	 * @return <code>true</code> si ya existe el objeto o <code>false</code> en caso contrario.
	 */
	@SuppressWarnings("rawtypes")
	protected boolean duplicateField(Session session, Table table, String field, String value)
	{
		// Comprobamos que ninguna de los parmetros es null y adems que la sesin 
		// ha sido abierta previamente.
		if ((session != null) && (session.isOpen()) && (table != null) && 
				(!ValidationUtils.isEmpty(field)) && (!ValidationUtils.isEmpty(value)))
		{
			// Preparamos la consulta HQL
			Query query = session.createQuery("from " + table + " where " + field + "=?");
			// Establecemos el valor del parmetro.
			query.setParameter(0, value, Hibernate.STRING);
			// Ejecutamos la consulta.
			List results = query.list();
			// Comprobamos el nmero de resultados devueltos.
			if (results.size() == 0)
			{
				// No existe el registro por lo que retornamos false.
				return false;
			}
		}
		// Si llegamos a este punto deberemos retornar true.
		return true;
	}
	
	/**
	 * <p>Comprueba que si dentro de la tabla <i>table</i> existe un registro cuyo valor
	 * en el campo <i>field</i> coincide con el par&aacute;metro <i>value</i> y, en el caso
	 * de que exista lo retorna. Para que el m&eacute;todo funcione correctamente ser&aacute; 
	 * necesario que previamente la sesi&oacute;n de Hibernate haya sido inicializada (si no 
	 * fuese as&iacute; el m&eeacute;todo retornar&aacute; <code>null</code>).
	 *  
	 * @param session <code>Session</code> sesi&oacute;n de Hibernate.
	 * @param table <code>Table</code> la tabla de la base de datos.
	 * @param field <code>String</code> el campo a comprobar.
	 * @param value <code>int</code> el valor a buscar.
	 * @return <code>Object</code> que recoge la informaci&oacute;n del objeto dentro de 
	 * la base de datos o <code>null</code> si no existe.
	 */
	@SuppressWarnings("rawtypes")
	protected Object existFieldValue(Session session, Table table, String field, int value)
	{
		// Comprobamos que los parmetros no son null y adems que se ha inicializado la sesin
		if ((session.isOpen()) && (table != null) && (!ValidationUtils.isEmpty(field)))
		{
			// Preparamos la consulta HQL.
			Query query = session.createQuery("from " + table + " where " + field + "=?");
			// Establecemos el valor del atributo.
			query.setParameter(0, value, Hibernate.INTEGER);
			// Ejecutamos la consulta.
			List results = query.list();
			// Comprobamos el nmero de resultados.
			if (results.size() == 1)
			{
				// Si el nmero de elementos es 1 ser que s existe el valor.
				return results.get(0);
			}
		}
		// Llegados a este punto ser debido a que no se ha encontrado el valor o alguno
		// de los parmetros era nulo por lo que retornamos null.
		return null;
	}
	
	/**
	 * <p>Comprueba que si dentro de la tabla <i>table</i> existe un registro cuyo valor
	 * en el campo <i>field</i> coincide con el par&aacute;metro <i>value</i> y, en el caso
	 * de que exista lo retorna. Para que el m&eacute;todo funcione correctamente ser&aacute; 
	 * necesario que previamente la sesi&oacute;n de Hibernate haya sido inicializada (si no 
	 * fuese as&iacute; el m&eeacute;todo retornar&aacute; <code>null</code>).
	 *  
	 * @param session <code>Session</code> sesi&oacute;n de Hibernate.
	 * @param table <code>Table</code> la tabla de la base de datos.
	 * @param field <code>String</code> el campo a comprobar.
	 * @param value <code>String</code> el valor a buscar.
	 * @return <code>Object</code> que recoge la informaci&oacute;n del objeto dentro de 
	 * la base de datos o <code>null</code> si no existe.
	 */
	@SuppressWarnings("rawtypes")
	protected Object existFieldValue(Session session, Table table, String field, String value)
	{
		// Comprobamos que los parmetros no son null y adems que se ha inicializado la sesin
		if ((session.isOpen()) && (table != null) && (!ValidationUtils.isEmpty(field)))
		{
			// Preparamos la consulta HQL.
			Query query = session.createQuery("from " + table + " where " + field + "=?");
			// Establecemos el valor del atributo.
			query.setParameter(0, value, Hibernate.STRING);
			// Ejecutamos la consulta.
			List results = query.list();
			// Comprobamos el nmero de resultados.
			if (results.size() == 1)
			{
				// Si el nmero de elementos es 1 ser que s existe el valor.
				return results.get(0);
			}
		}
		// Llegados a este punto ser debido a que no se ha encontrado el valor o alguno
		// de los parmetros era nulo por lo que retornamos null.
		return null;
	}
	
	/**
	 * <p>Comprueba que si dentro de la tabla <i>table</i> existe un registro cuyo valor
	 * en el campo <i>field</i> coincide con el par&aacute;metro <i>value</i> y, en el caso
	 * de que exista lo retorna. Para que el m&eacute;todo funcione correctamente ser&aacute; 
	 * necesario que previamente la sesi&oacute;n de Hibernate haya sido inicializada (si no 
	 * fuese as&iacute; el m&eeacute;todo retornar&aacute; <code>null</code>).
	 *  
	 * @param session <code>Session</code> sesi&oacute;n de Hibernate.
	 * @param table <code>Table</code> la tabla de la base de datos.
	 * @param field <code>String</code> el campo a comprobar.
	 * @param value <code>boolean</code> el valor a buscar.
	 * @return <code>Object</code> que recoge la informaci&oacute;n del objeto dentro de 
	 * la base de datos o <code>null</code> si no existe.
	 */
	@SuppressWarnings("rawtypes")
	protected Object existFieldValue(Session session, Table table, String field, boolean value)
	{
		// Comprobamos que los parmetros no son null y adems que se ha inicializado la sesin
		if ((session.isOpen()) && (table != null) && (!ValidationUtils.isEmpty(field)))
		{
			// Preparamos la consulta HQL.
			Query query = session.createQuery("from " + table + " where " + field + "=?");
			// Establecemos el valor del atributo.
			query.setParameter(0, value, Hibernate.BOOLEAN);
			// Ejecutamos la consulta.
			List results = query.list();
			// Comprobamos el nmero de resultados.
			if (results.size() == 1)
			{
				// Si el nmero de elementos es 1 ser que s existe el valor.
				return results.get(0);
			}
		}
		// Llegados a este punto ser debido a que no se ha encontrado el valor o alguno
		// de los parmetros era nulo por lo que retornamos null.
		return null;
	}
	
	/**
	 * <p>Retorna todos los objetos dentro de la tabla <i>table</i> que poseen el valor 
	 * <i>value</i> dentro del campo <i>field</i>. Para que el m&eeacute;todo funcione 
	 * correctamente previamente se habr&aacute; tenido que inicializar la sesi&aocute;n de 
	 * Hibernate. Si no se ha inicializado el m&eacute;todo retornar&aacute; una lista 
	 * vac&iacute;.</p>
	 * 
	 * @param <code>Session</code> sesi&oacute;n de Hibernate.
	 * @param table <code>Table</code> la tabla de la base de datos.
	 * @param field <code>String</code> el campo a comprobar.
	 * @param value <code>boolean</code> el valor a buscar.
	 * @return <code>List&lt;Object&gt;</code> con todos los objetos que cumplen 
	 * la condicin.
	 */
	@SuppressWarnings("unchecked")
	protected List<Object> getAll(Session session, Table table, String field, boolean value)
	{
		// Comprobamos los parmetros
		if ((session.isOpen()) && (table != null) && (!ValidationUtils.isEmpty(field)))
		{
			// Preparamos la sentencia HQL.
			Query query = session.createQuery("from " + table + " where " + field +"=?");
			// Establecemos los parmetros.
			query.setParameter(0, value, Hibernate.BOOLEAN);
			// Ejecutamos la consulta y retonramos el resultado.
			return query.list(); 
		}
		// Si llegamos a este punto no habremos podido hay objetos que cumplan
		// la condicin por lo que retornamos una lista vaca.
		return (new ArrayList<Object>());
	}
	
	/**
	 * <p>Retorna todos los objetos dentro de la tabla <i>table</i> que poseen el valor 
	 * <i>value</i> dentro del campo <i>field</i>. Para que el m&eeacute;todo funcione 
	 * correctamente previamente se habr&aacute; tenido que inicializar la sesi&aocute;n de 
	 * Hibernate. Si no se ha inicializado el m&eacute;todo retornar&aacute; una lista 
	 * vac&iacute;.</p>
	 * 
	 * @param <code>Session</code> sesi&oacute;n de Hibernate.
	 * @param table <code>Table</code> la tabla de la base de datos.
	 * @param field <code>String</code> el campo a comprobar.
	 * @param value <code>int</code> el valor a buscar.
	 * @return <code>List&lt;Object&gt;</code> con todos los objetos que cumplen 
	 * la condicin.
	 */
	@SuppressWarnings("unchecked")
	protected List<Object> getAll(Session session, Table table, String field, int value)
	{
		// Comprobamos los parmetros
		if ((session.isOpen()) && (table != null) && (!ValidationUtils.isEmpty(field)))
		{
			// Preparamos la sentencia HQL.
			Query query = session.createQuery("from " + table + " where " + field +"=?");
			// Establecemos los parmetros.
			query.setParameter(0, value, Hibernate.INTEGER);
			// Ejecutamos la consulta y retonramos el resultado.
			return query.list(); 
		}
		// Si llegamos a este punto no habremos podido hay objetos que cumplan
		// la condicin por lo que retornamos una lista vaca.
		return (new ArrayList<Object>());
	}
	
	/**
	 * <p>Retorna todos los objetos de la tabla <i>table</i>. Para que el m&etodo funcione
	 * correctamente previamente se habr&aacute; tenido que inicializar la sesi&oacute;n de
	 * Hibernate. Si no se ha inicializado el m&eeacut;todo retornar&aacute; una lista 
	 * vac&iacute;a.</p>
	 * 
	 * @param <code>Session</code> sesi&oacute;n de Hibernate.
	 * @param table <code>Table</code> la tabla de la base de datos.
	 * @return <code>List&lt;Object&gt;</code> con todos los objetos de la tabla.
	 */
	@SuppressWarnings("unchecked")
	protected List<Object> getAll(Session session, Table table)
	{
		// Comprobamos que los parmetros no son nulos y que la sesin est abierta.
		if ((session != null) && (session.isOpen()) && (table != null))
		{
			// Creamos la sentencia HQL.
			Query query = session.createQuery("from " + table);
			// Retornamos la lista tras ejecutar la sentencia.
			return query.list();
		}
		// Se llegamos a este punto querr decir que no se han cumplido las condiciones
		// sobre los parmetros as que retornamos una lista vaca.
		return (new ArrayList<Object>());
	}
	
	/**
	 * <p>Cierra la sesi&oacute;n de Hibernate que se le pasa como par&aacute;metro.</p> 
	 * 
	 * @param session <code>Session</code> la sesi&oacute;n que se quiere cerrar.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora
	 * de cerrar la sesi&oacute;n de Hibernate.
	 */
	protected void closeSession(Session session) throws DataStoreException
	{
		// Comprobamos que el parmetro no es nulo y adems que la sesin est abierta.
		if ((session != null) && (session.isOpen()))
		{
			try
			{
				// Intentamos cerrar la sesin
				session.close();
			}
			catch (Exception ex)
			{
				// Cualquier excepcin que se produzca la encapsulamos dentro de
				// una DataStoreException y la propagamos.
				throw (new DataStoreException(ex.getMessage()));
			}
		}
	}
	
}
