package edu.uoc.pfc.business.services;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.hibernate.Hibernate;
import org.hibernate.Query;
import org.hibernate.Session;

import edu.uoc.pfc.business.exceptions.BusinessRulesViolationException;
import edu.uoc.pfc.business.exceptions.DataStoreException;
import edu.uoc.pfc.business.exceptions.DuplicatedFieldException;
import edu.uoc.pfc.business.exceptions.InvalidIdException;
import edu.uoc.pfc.business.interfaces.IBClassroomManager;
import edu.uoc.pfc.business.objects.BOClassroom;
import edu.uoc.pfc.business.objects.BOClassroomUser;
import edu.uoc.pfc.business.objects.BOCurriculum;
import edu.uoc.pfc.business.objects.BOSubject;
import edu.uoc.pfc.business.objects.BOTest;
import edu.uoc.pfc.business.objects.BOTestAnswer;
import edu.uoc.pfc.business.objects.BOTestClassroom;
import edu.uoc.pfc.business.objects.BOUser;
import edu.uoc.pfc.business.transfers.TOClassroom;
import edu.uoc.pfc.business.transfers.TOTest;
import edu.uoc.pfc.business.transfers.TOUser;

public class IBClassroomManagerImpl extends IBBaseImpl implements IBClassroomManager 
{
	
	// -------- CONSTRUCTORES.
	/**
	 * <p><code>Construye un nuevo objeto <code>IBClassroomManagerImpl</code> e inicializa el 
	 * marco de persistencia <i>Hibernate</i>.</p>
	 * 
	 * @throws DataStoreException si se produce una excepci&oacute;n a la hora de iniciar
	 * el marco de persistencia.
	 */
	public IBClassroomManagerImpl() throws DataStoreException
	{
		// Constructor de la superclase.
		super();
	}
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cada vez que quiera asignar
	 * uno de sus Cuestionarios a un Aula. Si el Cuestionarios ya hab&iacute;a sido previamente
	 * asignado al Aula este m&eacute;todo no har&aacute; nada y finaliza su ejecuci&oacute;n 
	 * sin ning&uacute;n tipo de error.</p>
	 * 
	 * @param idClassroom <code>int</code> identificador del Aula.
	 * @param idTest <code>int</code> identificador del Cuestionario.
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param openDate <code>Date</code> fecha de publicaci&oacute;n.
	 * @param closeDate <code>Date</code> fecha de cierre.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de acceder
	 * a la capa de persistencia.
	 * @throws InvalidIdException si el identificador del profesor no corresponde con el de
	 * ninguno de los profesores registrados en el sistema, si el identificador del aula no 
	 * corresponde con el de ninguna de las aulas asignadas al profesor o si el identificador
	 * del cuestionario no se corresponde con el de ninguno de los cuestionarios del profesor.
	 */
	public void addTest(int idClassroom, int idTest, int idTeacher, Date openDate, 
			Date closeDate) throws DataStoreException, InvalidIdException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos que "idTeacher" es de un Profesor. Sentencia HQL.
			Query query = session.createQuery("from BOUser where id=? and teacher=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, idTeacher, Hibernate.INTEGER);
			query.setParameter(1, true, Hibernate.BOOLEAN);
			// Ejecutamos la consulta y vemos el total de resultados y el tipo.
			if ((query.list().size() != 1) || (!(query.list().get(0) instanceof BOUser)))
			{
				// Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
			}
			// Comprobamos que el usuario es un profesor del aula. Sentencia HQL.
			query = session.createQuery("from BOClassroomUser where classroom=? and user=? and role=?");
			// Establecemos los parmetros.
			query.setParameter(0, idClassroom, Hibernate.INTEGER);
			query.setParameter(1, idTeacher, Hibernate.INTEGER);
			query.setParameter(2, BOClassroomUser.ROLE_TEACHER, Hibernate.STRING);
			// Ejecutamos la consulta y vemos el total de resultados y el tipo.
			if ((query.list().size() != 1) || (!(query.list().get(0) instanceof BOClassroomUser)))
			{
				// Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_CLASSROOM_ID));
			}
			// Cast para obtener el objeto BOClassroomUser.
			BOClassroomUser boClassroomUser = (BOClassroomUser) query.list().get(0);
			// Comprobamos que el test es del usuario. Sentencia HQL.
			query = session.createQuery("from BOTest where id=? and teacher=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, idTest, Hibernate.INTEGER);
			query.setParameter(1, idTeacher, Hibernate.INTEGER);
			// Ejecutamos la consulta y vemos el total de resultados y el tipo.
			if ((query.list().size() != 1) || (!(query.list().get(0) instanceof BOTest)))
			{
				// Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEST_ID));
			}
			// Cast de BOTest.
			BOTest boTest = (BOTest) query.list().get(0);
			// Llegados a este punto vamos a ver si se ha hecho anteriormente la asignacin
			// del test al aula (habra que actualizar). Sentencia HQL.
			query = session.createQuery("from BOTestClassroom where test=? and classroom=?");
			// Valores de los parmetros.
			query.setParameter(0, idTest, Hibernate.INTEGER);
			query.setParameter(1, idClassroom, Hibernate.INTEGER);
			// Referencia local BOTestClassroom que contendr la informacin a almacenar.
			BOTestClassroom boTestClassroom = null;
			// Comprobamos el nmero de elementos retornados y el tipo.
			if ((query.list().size() == 1) && (query.list().get(0) instanceof BOTestClassroom))
			{
				// Tenemos que actulizar la informacin. Cast del objeto.
				boTestClassroom = (BOTestClassroom) query.list().get(0);
				// Asignamos los nuevos valores.
				boTestClassroom.setOpenDate(openDate);
				boTestClassroom.setCloseDate(closeDate);
			}
			else
			{
				// Tenemos que crear una nueva asignacin.
				boTestClassroom = new BOTestClassroom(boTest, boClassroomUser.getClassroom(), 
						openDate, closeDate);
			}
			// Guardamos la informacin en la capa de persistencia.
			session.saveOrUpdate(boTestClassroom);
			// Confirmamos la operacin.
			session.flush();
			// Una vez asignamos el test al aula tenemos que establecer el estado del test a "publicado".
			boTest.setPublished(true);
			// Modificamos la informacin del test.
			session.saveOrUpdate(boTest);
			// Consolidamos la operacin
			session.flush();
		}
		finally 
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado cuando se quiera a&ntilde;adir un Alumno
	 * a un Aula dentro de la aplicaci&oacute;n.</p>
	 * 
	 * @param idClassroom <code>int</code> identificador del Aula.
	 * @param idStudent <code>int</code> identificador del Estudiante.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de acceder
	 * a la capa de persistencia.
	 * @throws InvalidIdException si el identificador del Aula que se pasa como par&aacute;metro
	 * no corresponde con el de ninguna de las Aulas que est&aacute;n registradas en la aplicaci&oacute;n
	 * o si el identificador del Estudante que se pasa como par&aacute;metro tampoco corresponde a 
	 * ninguno de los Estudiantes que est&aacute;n registrados en la aplicaci&oacute;n.
	 * @throws BusinessRulesViolationException si el Estudiante ya ha sido asignado previamente al
	 * Aula como Profesor. 
	 */
	@Override
	public void assignStudent(int idClassroom, int idStudent)
			throws DataStoreException, InvalidIdException, BusinessRulesViolationException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Obtenemos la informacin del Aula.
			Object objClassroom = existFieldValue(session, Table.CLASSROOM, "id", idClassroom);
			// Comprobamos que no es null y adems que es de tipo BOClassroom.
			if ((objClassroom == null) || (!(objClassroom instanceof BOClassroom)))
			{
				// Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_CLASSROOM_ID));
			}
			// Guardamos el objeto BOClassroom (cast).
			BOClassroom boClassroom = (BOClassroom) objClassroom;
			// Vamos si "idStudent" corresponde con el identificador de un Estudiante. Sentencia HQL.
			Query queryIsStudent = session.createQuery("from BOUser where id=? and student=?");
			// Asignamos los parmetros.
			queryIsStudent.setParameter(0, idStudent, Hibernate.INTEGER);
			queryIsStudent.setParameter(1, true, Hibernate.BOOLEAN);
			// Comprobamos si existe el Aulumno.
			if ((queryIsStudent.list().size() == 0) || (!(queryIsStudent.list().get(0) instanceof BOUser)))
			{
				// Lanzamos una InvalidIdException (no existe el Alunmno).
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_STUDENT_ID));
			}
			// Cast del objeto Alumno.
			BOUser boStudent = (BOUser) queryIsStudent.list().get(0);
			// Vamos a comprobar que el Alumno no ha sido asignado previamente como Profesor al Aula.
			// Sentencia HQL para comprobarlo.
			Query queryIsClassroomTeacher = session.createQuery("from BOClassroomUser where classroom=? " +
					"and user=? and role=?"); 
			// Establecemos los valores de los parmetros.
			queryIsClassroomTeacher.setParameter(0, idClassroom, Hibernate.INTEGER);
			queryIsClassroomTeacher.setParameter(1, idStudent, Hibernate.INTEGER);
			queryIsClassroomTeacher.setParameter(2, BOClassroomUser.ROLE_TEACHER, Hibernate.STRING);
			// Ejecutamos la consulta.
			if (queryIsClassroomTeacher.list().size() != 0)
			{
				// El Usuario ha sido asignado como el Profesor del Aula por lo que lanzamos una excepcin.
				throw (new BusinessRulesViolationException(BusinessRulesViolationException.MSG_STUDENT_WAS_ASSIGNED_AS_TEACHER));
			}
			// Vamos a ver si ya se ha asignado previamente el Alumno. Sentencia HQL
			Query queryIsClassroomStudent = session.createQuery("from BOClassroomUser where classroom=? " +
					"and user=? and role=?");
			// Establecemos los valores de los parmetros.
			queryIsClassroomStudent.setParameter(0, idClassroom, Hibernate.INTEGER);
			queryIsClassroomStudent.setParameter(1, idStudent, Hibernate.INTEGER);
			queryIsClassroomStudent.setParameter(2, BOClassroomUser.ROLE_STUDENT, Hibernate.STRING);
			// Ejecutamos la sentencia y vemos el nmero de resultados.
			if (queryIsClassroomStudent.list().size() == 0)
			{
				// Tenemos que asignar el Alumno al Aula (si ya estaba asignado no haremos nada).
				// Creamos un objeto BOClassroomUser.
				BOClassroomUser boClassroomUser = new BOClassroomUser(boStudent, boClassroom, 
						BOClassroomUser.ROLE_STUDENT);
				// Guardamos la informacin
				session.saveOrUpdate(boClassroomUser);
				// Consolidamos la operacin.
				session.flush();
			}
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	
	/**
	 * <p>M&eacute;todo que ser&aacute; cuando se quiera a&ntilde;adir un Profesor a un 
	 * Aula dentro de la aplicaci&oacute;n.</p>
	 * 
	 * @param idClassroom <code>int</code> identificador del Aula.
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de acceder
	 * a la capa de persistencia.
	 * @throws InvalidIdException si el identificador del Aula que se pasa como par&aacute;metro
	 * no corresponde con el de ninguna de las Aulas que est&aacute;n registradas en la aplicaci&oacute;n
	 * o si el identificador del Profesor que se pasa como par&aacute;metro tampoco corresponde a 
	 * ninguno de los Profesores que est&aacute;n registrados en la aplicaci&oacute;n.
	 * @throws BusinessRulesViolationException si el Profesor ya ha sido asignado previamente al
	 * Aula como Alumno.
	 */
	@Override
	@SuppressWarnings("rawtypes")
	public void assignTeacher(int idClassroom, int idTeacher) 
			throws DataStoreException, InvalidIdException, BusinessRulesViolationException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Obtenemos la informacin del Aula.
			Object objClassroom = existFieldValue(session, Table.CLASSROOM, "id", idClassroom);
			// Comprobamos que no es null y adems que es de tipo BOClassroom.
			if ((objClassroom == null) || (!(objClassroom instanceof BOClassroom)))
			{
				// Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_CLASSROOM_ID));
			}
			// Guardamos el objeto BOClassroom (cast).
			BOClassroom boClassroom = (BOClassroom) objClassroom;
			// Obtenemos la informacin del Profesor. Preparamos la sentencia HQL
			Query queryExistTeacher = session.createQuery("from BOUser where id=? and teacher=?");
			// Establecemos los valores de los parmetro.
			queryExistTeacher.setParameter(0, idTeacher, Hibernate.INTEGER);
			queryExistTeacher.setParameter(1, true, Hibernate.BOOLEAN);
			// Ejecutamos la sentecia y vemos el nmero de registros retornados y si el elemento
			// que ocupa la posicin "0" es de tipo BOUser
			if ((queryExistTeacher.list().size() != 1) || 
					(!(queryExistTeacher.list().get(0) instanceof BOUser)))
			{
				// Lanzamos una InvalidIdException (no existe el profesor).
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
			}
			// Guardamos la referencia al profesor.
			BOUser boTeacher = (BOUser) queryExistTeacher.list().get(0);
			// Comprobamos si el usuario ha sido asignado previamente como Alumno al Aula.
			// Preparamos la sentecia HQL.
			Query queryIsStudent = session.createQuery("from BOClassroomUser where user=? and role=? and classroom=?");
			// Establecemos los valores de los parmetros.
			queryIsStudent.setParameter(0, idTeacher, Hibernate.INTEGER);
			queryIsStudent.setParameter(1, BOClassroomUser.ROLE_STUDENT, Hibernate.STRING);
			queryIsStudent.setParameter(2, idClassroom, Hibernate.INTEGER);
			// Ejecutamos la consulta y vemos el nmero de resultados que devuelve.
			if (queryIsStudent.list().size() != 0)
			{
				// Ya est asignado el usuario al aula como un Alumno.
				// Lanzamos una excepcin.
				throw (new BusinessRulesViolationException(
						BusinessRulesViolationException.MSG_TEACHER_WAS_ASSIGNED_AS_STUDENT));
			}			
			// Comprobamos si ya se ha realizado la asignacin del Profesor al Aula.
			Query queryIsTeacher = session.createQuery("from BOClassroomUser where user=? and role=? and classroom=?");
			// Establecemos los valores de los parmetros.
			queryIsTeacher.setParameter(0, idTeacher, Hibernate.INTEGER);
			queryIsTeacher.setParameter(1, BOClassroomUser.ROLE_TEACHER, Hibernate.STRING);
			queryIsTeacher.setParameter(2, idClassroom, Hibernate.INTEGER);
			// Ejecutamos la consulta y vemos el nmero de resultados.
			if (queryIsTeacher.list().size() == 0)
			{
				// No se ha asignado anteriormente el Profesor al Aula.
				// Vamos a ver si el Aula ya tena un profesor asignado. Sentencia HQL
				Query queryHaveTeacher = session.createQuery("from BOClassroomUser where classroom=? and role=?");
				// Establecemos los parmetros.
				queryHaveTeacher.setParameter(0, idClassroom, Hibernate.INTEGER);
				queryHaveTeacher.setParameter(1, BOClassroomUser.ROLE_TEACHER, Hibernate.STRING);
				// Ejecutamos la consulta.
				List list_teacher = queryHaveTeacher.list();
				// Vemos el nmero de elementos y que el nico elemento que tiene sea de tipo BOClassroomUser
				if ((list_teacher.size() == 1) && (list_teacher.get(0) instanceof BOClassroomUser))
				{
					// Tenemos que modificar la informacin de la asignacin del Profesor. Obtenemos 
					// el objeto BOClassroomUser con la informacin hasta ahora en la capa de persistencia
					BOClassroomUser boClassroomUser = (BOClassroomUser) list_teacher.get(0);
					// Creamos un objeto BOUser con la informacin del profesor.
					BOUser teacher = new BOUser(idTeacher);
					// Establecemos el nuevo Profesor.
					boClassroomUser.setUser(teacher);
					// Actualizamos la informacin en la sesin.
					session.saveOrUpdate(boClassroomUser);
				}
				else
				{
					// El proceso est bien (incluso si previamente ha sido asignado el Profesor
					// al Aula lo volvemos a ejecutar). Creamos un objeto BOClassroomUser
					BOClassroomUser boClassroomUser = new BOClassroomUser(boTeacher, 
							boClassroom, BOClassroomUser.ROLE_TEACHER);
					// Guardamos el objeto en la capa de persistencia.
					session.saveOrUpdate(boClassroomUser);
				}
				// Consolidamos la operacin.
				session.flush();
			}
			// En el caso de que el Profesor ya hubiese sido asignado se considera
			// que la operacin es correcta aunque no se realiza ninguna operacin.
		}
		finally 
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado siempre que se quiera crear un nuevo Aula dentro
	 * del sistema.</p>
	 * 
	 * @param idCurriculum <code>int</code> identificador del Plan de Estudios del Aula.
	 * @param idSubject <code>int</code> identificador de la Asignatura que se imparte en el Aula.
	 * @param course <code>int</code> a&ntilde;o acad&eacute;mico.
	 * @param letter <code>String</code> letra que le corresponde al Aula.
	 * @return <code>TOClassroom</code> con la informaci&oacute;n correspondiente al nuevo Aula creado.
	 * @throws DataStoreException si se produce un error a la hora de acceder a la capa de persistencia.
	 * @throws InvalidIdException si el identificador del Plan de Estudios o de la Asignatura no se 
	 * corresponde con el de ning&uacute;n Plan de Estudios y Asignatura registrada.
	 * @throws DuplicatedFieldException si el Aula que se quiere registrar ya est&aacute; registrada 
	 * dentro del sistema.
	 */
	@Override
	public TOClassroom createClassroom(int idCurriculum, int idSubject, int course, String letter)
			throws DataStoreException, InvalidIdException, DuplicatedFieldException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos si existe el Plan de Estudios.
			Object objCurriculum = existFieldValue(session, Table.CURRICULUM, "id", idCurriculum);
			// Vemos si es nulo o no es de la clase BOCurriculum.
			if ((objCurriculum == null) || (!(objCurriculum instanceof BOCurriculum)))
			{
				// No existe un Plan de Estudios con el "id" pasado. Lanzamos una excepcin.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_CURRICULUM_ID));
			}
			// Comprobamos que existe la Asignatura.
			Object objSubject = existFieldValue(session, Table.SUBJECTS, "id", idSubject);
			// Vemos si es nulo o no es de la clase BOSubject
			if ((objSubject == null) || (!(objSubject instanceof BOSubject)))
			{
				// No existe la asignatura. Lanzamos una excepcin.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_SUBJECT_ID));
			}
			// Vemos si la asignatura ha sido asignada al Plan de Estudios. Sentencia HQL.
			Query query = session.createQuery("from BOSubjectCurriculum where curriculum=? and subject=?");
			// Establecemos los parmetros.
			query.setParameter(0, idCurriculum, Hibernate.INTEGER);
			query.setParameter(1, idSubject, Hibernate.INTEGER);
			// Ejecutamos la consulta y vemos el nmero de resultados obtenidos.
			if (query.list().size() == 1)
			{
				// Tenemos que comprobar que si el Aula ya ha sido creado. Sentencia HQL
				Query query2 = session.createQuery("from BOClassroom where curriculum=? and subject=? and course=? and letter=?");
				// Establecemos los valores de los parmetros.
				query2.setParameter(0, idCurriculum, Hibernate.INTEGER);
				query2.setParameter(1, idSubject, Hibernate.INTEGER);
				query2.setParameter(2, course, Hibernate.INTEGER);
				query2.setParameter(3, letter, Hibernate.STRING);
				// Ejecutamos la consulta y vemos el total de resultados.
				if (query2.list().size() == 1)
				{
					// Ya existe el Aula. Lanzamos una DuplicatedFieldException.
					throw (new DuplicatedFieldException(DuplicatedFieldException.MSG_CLASSROOM_DUPLICAD));
				}
				// Si llegamos a este punto tendremos que crear el aula.
				// Cast a los objeto BOCurriculum y BOSubject.
				BOCurriculum boCurriculum = (BOCurriculum) objCurriculum;
				BOSubject boSubject = (BOSubject) objSubject;
				// Creamos un objeto BOClassroom con la informacin del aula.
				BOClassroom boClassroom = new BOClassroom(boCurriculum, boSubject, letter, course);
				// Guardamos el objeto en la capa de persistencia.
				session.saveOrUpdate(boClassroom);
				// Consolidamos la operacin.
				session.flush();
				// Retornamos el objeto TOClassroom equivalente al que acabamos de crear.
				return (boClassroom.getTOClassroom());
			}
			// Si se llega a este punto querr decir que no la Asignatura no ha sido 
			// asignada al Plan de Estudios por lo que lanzamos una InvalidIdException.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_SUBJECT_ASSIGN_CURRICULUM));
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo para eliminar del sistema la informaci&oacute;n del Aula cuyo
	 * identificador se le pasa como par&aacute;metro.</p>
	 * 
	 * @param idClassroom <code>int</code> identificador del Aula.
	 * @throws DataStoreException si se produce un error a la hora de trabajar con la capa de persistencia.
	 * @throws InvalidIdException si el identificador de Aula pasado no corresponde con el de ninguna
	 * de las Aulas que est&aacute;n registradas dentro de la aplicaci&oacute;n.
	 */
	@Override
	public void deleteClassroom(int idClassroom) throws DataStoreException, InvalidIdException
	{
		// Obtenemos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos si existe un Aula cuyo identificador sea el que se recibe en "idClassroom".
			Object objClassroom = existFieldValue(session, Table.CLASSROOM, "id", idClassroom);
			// Comprobamos que no se null y que es de tipo BOClassroom
			if ((objClassroom != null) && (objClassroom instanceof BOClassroom))
			{
				// Obtenemos la lista de todas las asignaciones de Usuarios al Aula (profesores
				// y alumnos).
				List<Object> list_users = getAll(session, Table.CLASSROOM_USER, "classroom", idClassroom);
				// Recorremos la lista de resultados.
				for (Object objUser : list_users)
				{
					// Comprobamos que se trata de un objeto BOClassroomUser
					if (objUser instanceof BOClassroomUser)
					{
						// Cast del objeto
						BOClassroomUser bo = (BOClassroomUser) objUser;
						// Eliminamos el registro.
						session.delete(bo);
						// Confirmamos la operacin.
						session.flush();
					}
				}
				// Una vez eliminados los Alumnos y Profesores eliminamos el Aula.
				session.delete((BOClassroom)objClassroom);
				// confirmamos la operacin.
				session.flush();
			}
			else
			{
				// No existe un Aual cuyo identificador coincide con el que se 
				// ha pasado como parmetro. Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_CLASSROOM_ID));
			}
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo para ser&aacute; llamado para obtener la informaci&oacute;n de un
	 * Aula del sistema (aquella cuyo identificador se le pasa como par&aacute;metro).</p>
	 * 
	 * @param id <code>int</code> identificador del Aula.
	 * @return <code>TOClassroom</code> con la informaci&oacute;n del Aula.
	 * @throws DataStoreException si se produce un error a la hora de acceder a la capa de persistencia.
	 * @throws InvalidIdException si el identificador que se ha pasado no se corresponde 
	 * con el de ninguna de las Aulas registradas en el sistema.
	 */
	@Override
	public TOClassroom getClassroom(int id) throws DataStoreException, InvalidIdException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Obtenemos la informacin del Aula.
			Object objClassroom = existFieldValue(session, Table.CLASSROOM, "id", id);
			// Comprobamos que no nulo y adems que es de tipo BOClassroom.
			if ((objClassroom != null) && (objClassroom instanceof BOClassroom))
			{
				// Cast del objeto.
				BOClassroom boClassroom = (BOClassroom) objClassroom;
				// Retornamos el objeto TOClassroom equivalente.
				return boClassroom.getTOClassroomComplete();
			}
			// Si llegamos a este punto quiere decir que el identificar que se ha pasado
			// no corresponde con el ninguno de los Aulas registradas en el sistema.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_CLASSROOM_ID));
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo que retornar&aacute; un objeto <code>TOClassroom</code> con la informaci&oacute;n
	 * del Aula cuyo identificador se le pasa como par&aacute;metro siempre y cuando ese Aula corresponda
	 * al Estudiante cuyo identificador se le pasa como par&aacute;metro (es decir, que el Estudiante est&eacute;
	 * asignado como tal al Aula).</p>
	 * 
	 * @param idStudent <code>int</code> identificador del Estudiante.
	 * @param idClassroom <code>int</code> identificador del Aula.
	 * @return <code>TOClassroom</code> con toda la informaci&oacute;n del Aula.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con la
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador de Estudiante que se pasa no corresponde con el de un
	 * Estudiante registrado en el sistema o si el identificador del Aula no corresponde con el de ninguna 
	 * de las Aulas que el Estudiante tiene asignadas.
	 */
	@Override
	public TOClassroom getClassroomStudent(int idStudent, int idClassroom)
			throws DataStoreException, InvalidIdException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamo que existe un usuario con el identificador que se le pasa al mtodo.
			Object objUser = existFieldValue(session, Table.USERS, "id", idStudent);
			// Comprobamos que no sea nulo y que sea de tipo BOUser.
			if ((objUser == null) || (!(objUser instanceof BOUser)))
			{
				// El identificador que se pasa no corresponde con el de un usuario. Lanzamos un excepcin.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_STUDENT_ID));
			}
			// Cast de objUser.
			BOUser boUser = (BOUser) objUser;
			// Comprobamos si se trata de un Alumno.
			if (!boUser.isStudent())
			{
				// No es un Estudiante as que lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_STUDENT_ID));
			}
			// Comprobamos si el Aulumno est asignado como tal al Aula. Sentencia HQL.
			Query query = session.createQuery("from BOClassroomUser where classroom=? and user=? and role=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, idClassroom, Hibernate.INTEGER);
			query.setParameter(1, idStudent, Hibernate.INTEGER);
			query.setParameter(2, BOClassroomUser.ROLE_STUDENT, Hibernate.STRING);
			// Ejecutamos la consulta. Vemos el nmero de objetos retornados y el tipo.
			if ((query.list().size() != 1) || (!(query.list().get(0) instanceof BOClassroomUser)))
			{
				// El Alumno no est asignado al Aula. Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_CLASSROOM_ID));
			}
			// Cast del nico objeto retornado en la consulta.
			BOClassroomUser boClassroomUser = (BOClassroomUser) query.list().get(0);
			// Obtenemos el objeto TOClassroom con la informacin del Aula.
			TOClassroom toClassroom = boClassroomUser.getClassroom().getTOClassroomComplete();
			// Una vez tenemos todos los test que se han publicado en Aula vamos a obtener 
			// la informacin de aquellos que han sido contestados por el Estudiante. 
			// Recorremos el conjunto de test asignados al Aula.
			for (BOTestClassroom boTestClassroom : boClassroomUser.getClassroom().getTest())
			{
				// Sentencia HQL para ver si el estudiante ha contestado al test o no.
				query = session.createQuery("from BOTestAnswer where student=? and test_classroom=?");
				// Establecemos los valores de los parmetros.
				query.setParameter(0, idStudent, Hibernate.INTEGER);
				query.setParameter(1, boTestClassroom.getId(), Hibernate.INTEGER);
				// Ejecutamos la sentencia HQL y recorremos los resultados.
				for (Object objAnswer : query.list())
				{
					// Comprobamos que objAnswer es de tipo BOTestAnswer.
					if (objAnswer instanceof BOTestAnswer)
					{
						// Cast del objeto.
						BOTestAnswer boTestAnswer = (BOTestAnswer) objAnswer;
						// Recorremos la lista de Test del Aula para asignar la informacin
						// de la respuesta del alumno al objeto TOTest.
						for (TOTest toTest : toClassroom.getTest())
						{
							// Comparamos los identificadores de los Test.
							if (toTest.getId() == boTestAnswer.getTestClassroom().getTest().getId())
							{
								// Establecemos los valores de los atributos de "toTest"
								toTest.setAnswerDate(boTestAnswer.getAnswerDate());
								toTest.setScore(boTestAnswer.getScore());
								toTest.setMaxScore(boTestAnswer.getMaxScore());
								toTest.setFinalScore(boTestAnswer.getFinalScore());
							}
						}
					}
				}
			}
			// Retornamos la informacin completa del Aula.
			return toClassroom;
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo que retornar&aacute; un objeto <code>TOClassroom</code> con la informaci&oacute;n
	 * del Aula cuyo identificador se le pasa como par&aacute;metro siempre y cuando ese Aula corresponda
	 * al Profesor cuyo identificador se le pasa como par&aacute;metro (es decir, que el Profesor est&eacute;
	 * asignado como tal al Aula).</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idClassroom <code>int</code> identificador del Aula.
	 * @return <code>TOClassroom</code> con toda la informaci&oacute;n del Aula.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con la
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador de Profesor que se pasa no corresponde con el de un
	 * Profesor registrado en el sistema o si el identificador del Aula no corresponde con el de ninguna 
	 * de las Aulas que el Profesor tiene asignadas.
	 */
	@Override
	public TOClassroom getClassroomTeacher(int idTeacher, int idClassroom) 
			throws DataStoreException, InvalidIdException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos que el "idTeacher" corresponde al de un Profesor.
			Object objTeacher = existFieldValue(session, Table.USERS, "id", idTeacher);
			// Comprobamos que "objTeacher" no es nulo y adems que es de tipo BOUser.
			if ((objTeacher == null) || (!(objTeacher instanceof BOUser)))
			{
				// Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
			}
			// Comprobamos que el profesor ha sido asignado como tal al Aula. Sentencia HQL.
			Query query = session.createQuery("from BOClassroomUser where classroom=? and user=? and role=?");
			// Asignamos los valores a los parmetros.
			query.setParameter(0, idClassroom, Hibernate.INTEGER);
			query.setParameter(1, idTeacher, Hibernate.INTEGER);
			query.setParameter(2, BOClassroomUser.ROLE_TEACHER, Hibernate.STRING);
			// Ejecutamos la consulta, vemos el nmero de elementos obtenido y el tipo.
			if ((query.list().size() != 1) || (!(query.list().get(0) instanceof BOClassroomUser)))
			{
				// Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_CLASSROOM_ID));
			}
			// Cast del objeto BOClassroomUser resultado de la consulta.
			BOClassroomUser boClassroomUser = (BOClassroomUser) query.list().get(0);
			// Comprobamos el valor de su atributo "classroom".
			if (boClassroomUser.getClassroom() == null)
			{
				// No tenemos la informacin del Aula as que lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_CLASSROOM_ID));
			}
			// Si llegamos a este punto querr decir que s existe el Aula as que retornamos toda su informacin.
			return (boClassroomUser.getClassroom().getTOClassroomComplete());
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>Retorna una lista de objectos <code>TOUser</code> donde cada uno de ellos recoge la informaci&oacute;n
	 * de un Alumno que no ha sido asignado al Aula cuyo identificador se le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador del Aula.
	 * @return <code>List&lt;TOUser&gt;</code> con la informaci&oacute;n de todos los Alumnos que todav&iacute;a
	 * no han sido asignados al Aula.
	 * @throws DataStoreException si se produce un error a la hora de acceder a la capa de persistencia.
	 * @throws InvalidIdException si el identificador que se ha pasado no se corresponde 
	 * con el de ninguna de las Aulas registradas en el sistema.
	 */
	@Override
	public List<TOUser> getUnassignedStudents(int id) throws DataStoreException, InvalidIdException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Obtenemos la informacin del Aula.
			Object objClassroom = existFieldValue(session, Table.CLASSROOM, "id", id);
			// Comprobamos si objClassroom es nulo y adems de tipo BOClassroom.
			if ((objClassroom == null) || (!(objClassroom instanceof BOClassroom)))
			{
				// No existe el aula. Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_CLASSROOM_ID));
			}
			// Llegados a este punto sabemos que existe el Aula. Cast a un BOClassroom
			BOClassroom boClassroom = (BOClassroom) objClassroom;
			// Obtenemos la lista de todos los Alumnos del sistema.
			List<Object> list_students = getAll(session, Table.USERS, "student", true);
			// Definimos un conjunto para almacenar todos los Alumnos.
			Set<BOUser> set_students = new HashSet<BOUser>();
			// Recorremos la lista de objetos que nos ha retornado.
			for (Object objStudent : list_students)
			{
				// Comprobamos que objStudent es de tipo BOUser.
				if (objStudent instanceof BOUser)
				{
					// Aadimos el objeto al conjunto de estudiantes (tras hacer el cast).
					set_students.add((BOUser) objStudent);
				}
			}
			// Recorremos la lista de usuarios asignados al Aula.
			for (BOClassroomUser boClassroomUser : boClassroom.getUsers())
			{				
				// Se trate de un Profesor o de un Alumno lo deberemso eliminar de la lista
				// de posibles alumnos a asignar al Aula.
				set_students.remove(boClassroomUser.getUser());
			}
			// Creamos una lista con los Estudiantes que retornaremos.
			List<TOUser> list_students_assign = new ArrayList<TOUser>();
			// Recorremos el conjunto.
			for (BOUser user_student : set_students)
			{
				// Obtenemos el objeto TOUser equivalente al BOUser almacenado.
				TOUser toUser = user_student.getTOUser();
				// Indicamos que queremos ordenarlo por apellidos.
				toUser.setOrderBy(TOUser.ORDER_BY_SURNAME);
				// Aadimos "toUser" a la lista a retornar.
				list_students_assign.add(toUser);
			}
			// Ordenamos la lista de Alumnos.
			Collections.sort(list_students_assign);
			// Retornamos la lista ordenada.
			return list_students_assign;
		}
		finally 
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>Retorna una lista de objectos <code>TOClassroom</code> donde cada uno 
	 * de los elementos representa a un Aula registrada en el sistema.</p>
	 * 
	 * @return <code>List&lt;TOClassroom&gt;</code> donde cada uno de los elementos
	 * representa a un Aula registrada en el sistema.
	 * @throws DataStoreException si se produce un error a la hora de trabajar con
	 * la capa de persistencia.
	 */
	@Override
	public List<TOClassroom> listClassroom() throws DataStoreException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Obtenemos la lista de todas las aulas registradas.
			List<Object> list_objects = getAll(session, Table.CLASSROOM);
			// Creamos la lista de objetos TOClassroom
			List<TOClassroom> classroomList = new ArrayList<TOClassroom>();
			// Recorremos la lista de objetos "list_objects"
			for(Object obj : list_objects)
			{
				// Comprobamos que el objeto es de tipo BOClassroom
				if (obj instanceof BOClassroom)
				{
					// Cast del objeto.
					BOClassroom classroom = (BOClassroom) obj;
					// Aadimos le objeto TOClassroom equivalente a la lista
					classroomList.add(classroom.getTOClassroom());
				}
			}
			// Ordenamos la lista con las aulas.
			Collections.sort(classroomList);
			// Retornamos la lista de objetos TOClassroom.
			return classroomList;
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo que permite obtener una lista de objetos <code>TOClassroom</code> donde cada uno
	 * de ellos representa un Aula donde el Estudiante cuyo identificador se le pasa como par&aacute;metro
	 * est&aacute; asignado.</p>
	 * 
	 * @param idStudent <code>int</code> identificador del Estudiante.
	 * @return <code>List&lt;TOClassroom&gt;</code> con todas las Aulas donde el usuario est&aacute;
	 * asignado como Estudiante.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con la
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador de usuario que ser recibe no corresponde con el 
	 * de ninguno de los Estudiantes que est&aacute;n registrados en el sistema.
	 */
	@Override
	public List<TOClassroom> listClassroomStudent(int idStudent) 
			throws DataStoreException, InvalidIdException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos que el identificador que se pasa es el de un Usuario.
			Object objStudent = existFieldValue(session, Table.USERS, "id", idStudent);
			// Comprobamos que no sea nulo y que es de tipo BOUser.
			if ((objStudent == null) || (!(objStudent instanceof BOUser)))
			{
				// Lanzamos una InvalidIdException puesto que no es un identificador vlido.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_USER_ID));
			}
			// Cast al objeto.
			BOUser boStudent = (BOUser) objStudent;
			// Comprobamos si se trata de un Estudiante o no.
			if (!(boStudent.isStudent()))
			{
				// No es un Estudiante as que lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_STUDENT_ID));
			}
			// Se trata de un Estudiante. Obtenemos la lista de Aulas en las que est asignado.
			// Sentencia HQL.
			Query query = session.createQuery("from BOClassroomUser where user=? and role=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, idStudent, Hibernate.INTEGER);
			query.setParameter(1, BOClassroomUser.ROLE_STUDENT, Hibernate.STRING);
			// Creamos una lista de objeto TOClassroom que contendr las Aulas del Estudiante.
			List<TOClassroom> list_classroom = new ArrayList<TOClassroom>();
			// Recorremos el resultado de la consulta.
			for (Object objClassroomUser : query.list())
			{
				// Comprobamos que objClassroomUser es de tipo BOClassroomUser
				if (objClassroomUser instanceof BOClassroomUser)
				{
					// Cast de objClassroomUser.
					BOClassroomUser boClassroomUser = (BOClassroomUser) objClassroomUser;
					// Comprobamos que la informacin del Aula no es nula.
					if (boClassroomUser.getClassroom() != null)
					{
						// Aadimos el objeto TOClassroom equivalente a la lista de Aulas.
						list_classroom.add(boClassroomUser.getClassroom().getTOClassroom());
					}
				}
			}
			// Ordenamos la lista de Aulas obtenida.
			Collections.sort(list_classroom);
			// Retornamos la lista de Aulas del Estudiante.
			return list_classroom;
		}
		finally 
		{
			// Cerramos la sesin de la Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo que proporciona una lista de objetos <code>TOClassromm</code> donde 
	 * cada uno de ellos representa un Aula donde est&aacute; asignado como profesor el usuario
	 * cuyo identificador se le pasa como parmetro.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @return <code>List&lt;TOClassroom&gt;</code> con todas las Aulas donde el usuario est&aacute;
	 * asignado como Profesor.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con la
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador de usuario que ser recibe no corresponde con el 
	 * de ninguno de los Profesores que est&aacute;n registrados en el sistema.
	 */
	@Override
	public List<TOClassroom> listClassroomTeacher(int idTeacher) 
			throws DataStoreException, InvalidIdException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos que el identificador que se recibe es de un Profesor.
			Object objTeacher = existFieldValue(session, Table.USERS, "id", idTeacher);
			// Comprobamos que el objeto no es nulo y es de tipo BOUser.
			if ((objTeacher == null) || (!(objTeacher instanceof BOUser)))
			{
				// No se trata de un Profesor. Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
			}
			// Llegados a este punto tenemos que obtener la lista de Aulas del Profesor. Sentencia HQL.
			Query query = session.createQuery("from BOClassroomUser where user=? and role=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, idTeacher, Hibernate.INTEGER);
			query.setParameter(1, BOClassroomUser.ROLE_TEACHER, Hibernate.STRING);
			// Creamos un ArrayList de objeto TOClassroom que ser el que retornaremos.
			List<TOClassroom> list_classroom = new ArrayList<TOClassroom>();
			// Ejecutamos la consulta.
			for (Object objClassroomUser : query.list())
			{
				// Comprobamos que objClassroomUser es de tipo BOClassroomUser.
				if (objClassroomUser instanceof BOClassroomUser)
				{
					// Cast de objClasroomUser.
					BOClassroomUser bo = (BOClassroomUser) objClassroomUser;
					// Comrprobamos que tenemos la informacin del Aula.
					if (bo.getClassroom() != null)
					{
						// Aadimos a la lista a retornar el objeto TOClassroom equivalente.
						list_classroom.add(bo.getClassroom().getTOClassroom());
					}
				}
			}
			// Ordenamos la lista de Aulas.
			Collections.sort(list_classroom);
			// Retornamos la lista de Aulas.
			return list_classroom;
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado para desasignar un Cuestionario de un Aula.</p> 
	 * 
	 * @param idTest <code>int</code> identificador del Cuestionario.
	 * @param idClassroom <code>int</code> identificador del Aula.
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con la
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador del profesor no corresponde con el de
	 * ninguno de los profesores registrados en el sistema, si el identificador del aula no 
	 * corresponde con el de ninguna de las aulas asignadas al profesor o si el identificador
	 * del cuestionario no se corresponde con el de ninguno de los cuestionarios del profesor.
	 */
	@Override
	public void removeTest(int idTest, int idClassroom, int idTeacher)
			throws DataStoreException, InvalidIdException
	{
		// Creamos una sessin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos que "idTeacher" es de un Profesor. Sentencia HQL.
			Query query = session.createQuery("from BOUser where id=? and teacher=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, idTeacher, Hibernate.INTEGER);
			query.setParameter(1, true, Hibernate.BOOLEAN);
			// Ejecutamos la consulta y vemos el total de resultados y el tipo.
			if ((query.list().size() != 1) || (!(query.list().get(0) instanceof BOUser)))
			{
				// Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
			}
			// Comprobamos que el usuario es un profesor del aula. Sentencia HQL.
			query = session.createQuery("from BOClassroomUser where classroom=? and user=? and role=?");
			// Establecemos los parmetros.
			query.setParameter(0, idClassroom, Hibernate.INTEGER);
			query.setParameter(1, idTeacher, Hibernate.INTEGER);
			query.setParameter(2, BOClassroomUser.ROLE_TEACHER, Hibernate.STRING);
			// Ejecutamos la consulta y vemos el total de resultados y el tipo.
			if ((query.list().size() != 1) || (!(query.list().get(0) instanceof BOClassroomUser)))
			{
				// Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_CLASSROOM_ID));
			}
			// Comprobamos que el test es del usuario. Sentencia HQL.
			query = session.createQuery("from BOTest where id=? and teacher=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, idTest, Hibernate.INTEGER);
			query.setParameter(1, idTeacher, Hibernate.INTEGER);
			// Ejecutamos la consulta y vemos el total de resultados y el tipo.
			if ((query.list().size() != 1) || (!(query.list().get(0) instanceof BOTest)))
			{
				// Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEST_ID));
			}
			// Cast de BOTest.
			BOTest boTest = (BOTest) query.list().get(0);
			// Llegados a este punto vamos a ver si se ha hecho anteriormente la asignacin
			// del test al aula (habra que actualizar). Sentencia HQL.
			query = session.createQuery("from BOTestClassroom where test=? and classroom=?");
			// Valores de los parmetros.
			query.setParameter(0, idTest, Hibernate.INTEGER);
			query.setParameter(1, idClassroom, Hibernate.INTEGER);
			// Comprobamos el nmero de elementos retornados y el tipo.
			if ((query.list().size() == 1) && (query.list().get(0) instanceof BOTestClassroom))
			{
				// Tenemos que actulizar la informacin. Cast del objeto.
				BOTestClassroom boTestClassroom = (BOTestClassroom) query.list().get(0);
				// Tenemos que eliminar cualquier respuesta que se haya dado del test en el Aula.
				// Sentencia HQL para obtener la lista de asignaciones del Test.
				query = session.createQuery("from BOTestAnswer where test_classroom=?");
				// Establecemos los valores de los parmetros.
				query.setParameter(0, boTestClassroom.getId(), Hibernate.INTEGER);
				// Recorremos la lista de asignaciones.
				for (Object objTestAnswer : query.list())
				{
					// Comprobamos que el elemento es de tipo BOTestAnswer.
					if (objTestAnswer instanceof BOTestAnswer)
					{
						// Cast del objeto.
						BOTestAnswer boTestAnswer = (BOTestAnswer) objTestAnswer;
						// Eliminamos la respuesta. 
						session.delete(boTestAnswer);
						// Consolidamos la operacin.
						session.flush();
					}
				}
				// Eliminamos la asignacin.
				session.delete(boTestClassroom);
				// Consolidamos la operacin.
				session.flush();
				// Tenemos que ver si el Test est asignado a otro Aula. Sentencia HQL.
				query = session.createQuery("from BOTestClassroom where test=?");
				// Establecemos los parmetros.
				query.setParameter(0, idTest, Hibernate.INTEGER);
				// Ejecutamos la consulta y contamos los resultados.
				if (query.list().size() == 0)
				{
					// El test ya no est asignado a ningn Aula. Tenemos que establecerlo 
					// como "no publicado".
					boTest.setPublished(false);
					// Actualizamos la informacin del test.
					session.saveOrUpdate(boTest);
					// Consolidamos la operacin.
					session.flush();
				}
			}
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>Desasigna el Alumno cuyo identificador se le pasa en el par&aacute;metro <i>idStudent</i>
	 * del Aula cuyo identificador se le pasa en el par&aacute;metro <i>idClassroom</i>.</p>
	 * 
	 * @param idClassroom <code>int</code> identificador del Aula.
	 * @param idStudent <code>int</code> identificador del Alumno.
	 * @throws DataStoreException si se produce un error a la hora de acceder a la capa de persistencia.
	 * @throws InvalidIdException si <i>idStudent</i> no se corresponde con ninguno el ninguno
	 * de los estudiantes asignados al Aula o si <i>idClassroom</i> no se corresponde con el
	 * identificador de ninguna de las Aulas.
	 */
	@Override
	public void unassignStudent(int idClassroom, int idStudent) 
			throws DataStoreException, InvalidIdException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos si existe existe el Aula.
			Object objClassroom = existFieldValue(session, Table.CLASSROOM, "id", idClassroom);
			// Vemos si es nulo o no es de la clase BOClassromm
			if ((objClassroom == null) || (!(objClassroom instanceof BOClassroom)))
			{
				// Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_CLASSROOM_ID));
			}
			// Comprobamos si existe el alumno. Sentencia HQL.
			Query queryIsStudent = session.createQuery("from BOUser where id=? and student=?");
			// Asignamos los parmetros.
			queryIsStudent.setParameter(0, idStudent, Hibernate.INTEGER);
			queryIsStudent.setParameter(1, true, Hibernate.BOOLEAN);
			// Ejecutamos la consulta.
			if ((queryIsStudent.list().size() == 0) || (!(queryIsStudent.list().get(0) instanceof BOUser)))
			{
				// No existe el usuario o no se trata de un Alumno. Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_STUDENT_ID));
			}
			// Vemos si el Alumno ha sido asignado al Aula. Sencia HQL.
			Query queryIsAssigned = session.createQuery("from BOClassroomUser where classroom=? " +
					"and user=? and role=?");
			// Asignamos los parmetros.
			queryIsAssigned.setParameter(0, idClassroom, Hibernate.INTEGER);
			queryIsAssigned.setParameter(1, idStudent, Hibernate.INTEGER);
			queryIsAssigned.setParameter(2, BOClassroomUser.ROLE_STUDENT, Hibernate.STRING);
			// Ejecutamos la consulta.
			if ((queryIsAssigned.list().size() == 1) && (queryIsAssigned.list().get(0) instanceof BOClassroomUser))
			{
				// Obtenemos el objeto BOClassroomUser.
				BOClassroomUser bo = (BOClassroomUser) queryIsAssigned.list().get(0);
				// Eliminamos la asignacin.
				session.delete(bo);
				// Confirmamos la operacin.
				session.flush();
			}
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>Desasigna el Profesor que est&aacute; asignado al Aula cuyo identificador se 
	 * le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador del Aula.
	 * @throws DataStoreException si se produce un error a la hora de acceder a la capa de persistencia.
	 * @throws InvalidIdException si el identificador que se ha pasado no se corresponde 
	 * con el de ninguna de las Aulas registradas en el sistema.
	 */
	@Override
	public void unassignTeacher(int id) throws DataStoreException, InvalidIdException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos si existe el Aula.
			Object objClassroom = existFieldValue(session, Table.CLASSROOM, "id", id);
			// Vemos que si es nulo o no es de tipo BOClassroom.
			if ((objClassroom != null) && (objClassroom instanceof BOClassroom))
			{
				// Obtenemos el objeto que es el Profesor del Aula. Sentencia HQL.
				Query query = session.createQuery("from BOClassroomUser where classroom=? and role=?");
				// Asignamos los parmetros.
				query.setParameter(0, id, Hibernate.INTEGER);
				query.setParameter(1, BOClassroomUser.ROLE_TEACHER, Hibernate.STRING);
				// Ejecutamos la consulta y vemos el nmero de resultados y que es de tipo BOClassroomUser
				if ((query.list().size() == 1) && (query.list().get(0) instanceof BOClassroomUser))
				{
					// Cast del resultado.
					BOClassroomUser bo = (BOClassroomUser) query.list().get(0);
					// Borramos el Profesor del Aula.
					session.delete(bo);
					// Consolidamos la operacin.
					session.flush();
				}
			}
			else
			{
				// El identificador del Aula no corresponde con las de un Aula registrado.
				// Lanzamos una excepcin.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_CLASSROOM_ID));
			}
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
}
