package edu.uoc.pfc.business.services;

import org.hibernate.Session;

import edu.uoc.pfc.business.exceptions.BusinessRulesViolationException;
import edu.uoc.pfc.business.exceptions.DataStoreException;
import edu.uoc.pfc.business.interfaces.IBConfigManager;
import edu.uoc.pfc.business.objects.BOConfig;
import edu.uoc.pfc.business.transfers.TOConfig;

/**
 * <p>Implementaci&oacute;n de la interfaz <code>IBConfigManager</code> por lo que ser&aacute;
 * la clase de la capa de negocio que ofrecer&aacute; a la capa de presentaci&oacute;n toda
 * la funcionalidad de la aplicaci&oacute;n relacionada con la gesti&oacute;n de la 
 * configuraci&oacute;n.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0 
 * @see IBConfigManager
 */
public class IBConfigManagerImpl extends IBBaseImpl implements IBConfigManager
{
	
	// -------- CONSTRUCTORES.
	/**
	 * <p><code>Construye un nuevo objeto <code>IBAccountManagerImpl</code> e inicializa el 
	 * marco de persistencia <i>Hibernate</i>.</p>
	 * 
	 * @throws DataStoreException si se produce una excepci&oacute;n a la hora de iniciar
	 * el marco de persistencia.
	 */
	public IBConfigManagerImpl() throws DataStoreException
	{
		// Constructor de la superclase.
		super();
	}
	
	// -------- MTODOS PBLICOS
	/**
	 * <p>Retorna un objeto <code>TOConfig</code> con la informaci&oacute;n de configuraci&oacute;n
	 * que est&aacute; activa dentro de la aplicaci&oacute;n.</p>
	 * 
	 * @return <code>TOConfig</code> con la informaci&oacute;n de configuraci&oacute;n.
	 * @throws DataStoreException no se ha podido acceder a la capa de persistencia.
	 * @throws BusinessRulesViolationException si existe m&aacute;s de una configuraci&oacute;n
	 * activa dentro de la aplicaci&oacute;n.
	 */
	@Override
	public TOConfig getConfiguration() throws DataStoreException, BusinessRulesViolationException
	{
		// Abrimos la sesin de Hibernate.
		Session session = sessionFactory.openSession();
		try
		{
			// Vemos si existen los datos de la configuracin active.
			Object objConfig = existFieldValue(session, Table.CONFIG, "active", true);
			// Comprobamos que no es null y es de tipo BOConfig.
			if ((objConfig != null) && (objConfig instanceof BOConfig))
			{
				// Retornamos el objeto TOConfig equivalente a objConfig.
				return (((BOConfig) objConfig).getTOConfig());				
			}
			// No existe una configuracin activa as que lanzamos una excepcin. 
			throw (new BusinessRulesViolationException(
					BusinessRulesViolationException.MSG_NO_ACTIVE_CONFIGURATION));
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}

	/**
	 * <p>Modifica la informaci&oacute;n de configuraci&oacute;n de la aplicaci&oacute;n con los
	 * valores que se le pasan como par&aacute;metros.</p>
	 * 
	 * @param loginAttempts <code>int</code> n&uacute;meros de intentos de acceso a la cuenta.
	 * @param activeCourse <code>int</code> curso actual (activo).
	 * @param adminEmail <code>String</code> email del administrador.
	 * @return <code>TOConfig</code> con la informaci&oacute;n de configuraci&oacute;n actualizada.
	 * @throws DataStoreException si se produce un error a la hora de trabajar con la 
	 * capa de persistencia.
	 * @throws BusinessRulesViolationException si existe m&aacute;s de una configuraci&oacute;n
	 * activa dentro de la aplicaci&oacute;n.
	 */
	@Override
	public TOConfig modifyConfiguration(int loginAttempts, int activeCourse, String adminEmail) 
			throws DataStoreException, BusinessRulesViolationException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Obtenemos los datos de la configuracin activa.
			Object objConfiguration = existFieldValue(session, Table.CONFIG, "active", true);
			// Comprobamos que existe la configuracin y que es de tipo BOConfig.
			if ((objConfiguration != null) && (objConfiguration instanceof BOConfig))
			{
				// Cast al objeto con la configuracin activa.
				BOConfig boConfig = (BOConfig) objConfiguration;
				// Establecemos los nuevos valores de los parmetros.
				boConfig.setNoSMTPEmail(loginAttempts, activeCourse, adminEmail);
				// Modificamos la informacin.
				session.saveOrUpdate(boConfig);
				// Confirmamos la operacin
				session.flush();
				// Retornamos el objeto TOConfig equivalente tras la modificacin.
				return boConfig.getTOConfig();
			}
			// No existe una configuracin activa as que lanzamos una excepcin. 
			throw (new BusinessRulesViolationException(
					BusinessRulesViolationException.MSG_NO_ACTIVE_CONFIGURATION));
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>Modifica la informaci&oacute;n de configuraci&oacute;n de la aplicaci&oacute;n con los valores
	 * que se le pasan como par&aacute;metro.</p>
	 * 
	 * @param loginAttempts code>int</code> n&uacute;meros de intentos de acceso a la cuenta.
	 * @param activeCourse <code>int</code> curso actual (activo).
	 * @param adminEmail <code>String</code> email del administrador.
	 * @param smtpServer <code>String</code> servidor SMTP.
	 * @param smtpPort <code>int</code> puerto del servidor SMTP.
	 * @param smtpAuthentication <code>boolean</code> indica si el servidor SMTP requiero autentificaci&oacute;n
	 * @param smtpTLS <code>boolean</code> indica si el servidor SMTP admite el protocolo TLS.
	 * @param smtpUser <code>String</code> usuario servidor SMTP
	 * @param smtpPassword <code>String</code> password servidor SMTP.
	 * @param fpFrom <code>String</code> email de quien env&iacute;a el correo de recuperaci&oacute; de contrase&ntilde;a.
	 * @param fpSubject <code>String</code> asunto del correo de recuperaci&oacute; de contrase&ntilde;a.
	 * @param fpText <code>String</code> texto del correo de recuperaci&oacute; de contrase&ntilde;a.
	 * @param flFrom <code>String</code> email de quien env&iacute;a el correo de recuperaci&oacute; de login.
	 * @param flSubject <code>String</code> asunto del correo de recuperaci&oacute; de login.
	 * @param flText <code>String</code> texto del correo de recuperaci&oacute; de login.
	 * @return <code>TOConfig</code> con la informaci&oacute;n de configuraci&oacute;n actualizada.
	 * @throws DataStoreException si se produce un error a la hora de trabajar con la 
	 * capa de persistencia.
	 * @throws BusinessRulesViolationException si existe m&aacute;s de una configuraci&oacute;n
	 * activa dentro de la aplicaci&oacute;n.
	 */
	@Override
	public TOConfig modifyConfiguration(int loginAttempts, int activeCourse, String adminEmail,
			String smtpServer, int smtpPort, boolean smtpAuthentication, boolean smtpTLS, 
			String smtpUser, String smtpPassword, String fpFrom, String fpSubject, String fpText,
			String flFrom, String flSubject, String flText)
			throws DataStoreException, BusinessRulesViolationException
	{
		// Iniciamos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Obtenemos los datos de la configuracin activa.
			Object objConfiguration = existFieldValue(session, Table.CONFIG, "active", true);
			// Comprobamos que existe la configuracin y que es de tipo BOConfig.
			if ((objConfiguration != null) && (objConfiguration instanceof BOConfig))
			{
				// Cast al objeto con la configuracin activa.
				BOConfig boConfig = (BOConfig) objConfiguration;
				// Establecemos los nuevos valores de los parmetros.
				boConfig.setSMTPEmail(loginAttempts, activeCourse, adminEmail, smtpServer, 
						smtpPort, smtpAuthentication, smtpTLS, smtpUser, smtpPassword, 
						fpFrom, fpSubject, fpText, flFrom, flSubject, flText);
				// Modificamos la informacin.
				session.saveOrUpdate(boConfig);
				// Confirmamos la operacin
				session.flush();
				// Retornamos el objeto TOConfig equivalente tras la modificacin.
				return boConfig.getTOConfig();
			}
			// No existe una configuracin activa as que lanzamos una excepcin. 
			throw (new BusinessRulesViolationException(
					BusinessRulesViolationException.MSG_NO_ACTIVE_CONFIGURATION));
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
}
