package edu.uoc.pfc.business.services;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.hibernate.Hibernate;
import org.hibernate.Query;
import org.hibernate.Session;

import edu.uoc.pfc.business.exceptions.DataStoreException;
import edu.uoc.pfc.business.exceptions.DuplicatedFieldException;
import edu.uoc.pfc.business.exceptions.InvalidIdException;
import edu.uoc.pfc.business.interfaces.IBCurriculumManager;
import edu.uoc.pfc.business.objects.BOClassroom;
import edu.uoc.pfc.business.objects.BOCurriculum;
import edu.uoc.pfc.business.objects.BOSubject;
import edu.uoc.pfc.business.objects.BOSubjectCurriculum;
import edu.uoc.pfc.business.transfers.TOCurriculum;
import edu.uoc.pfc.business.transfers.TOSubject;
import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Implementaci&oacute;n de la interfaz <code>IBCurriculumManager</code> que la
 * capa de negocio ofrece a la de presentaci&oacute;n con todos los m&eeacute;todos 
 * asociados a la gesti&oacute;n de planes de estudios.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0 
 * @see IBCurriculumManager
 */
public class IBCurriculumManagerImpl extends IBBaseImpl implements IBCurriculumManager 
{
	
	// -------- CONSTRUCTORES.
	/**
	 * <p><code>Construye un nuevo objeto <code>IBCurriculumManagerImpl</code> e inicializa el 
	 * marco de persistencia <i>Hibernate</i>.</p>
	 * 
	 * @throws DataStoreException si se produce una excepci&oacute;n a la hora de iniciar
	 * el marco de persistencia.
	 */
	public IBCurriculumManagerImpl() throws DataStoreException
	{
		// Constructor de la superclase.
		super();
	}
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un administrador del sistema cuando quiera asignar 
	 * una asignatura a un plan de estudios.</p>
	 * 
	 * @param idCurriculum <code>int</code> identificador del Plan de Estudios.
	 * @param idSubject <code>int</code> identificador de la Asignatura.
	 * @param subjectType <code>String</code> tipo de Asignatura dentro del Plan de Estudios. 
	 * @param code <code>String</code> c&oacute;digo de la Asignatura.
	 * @param credits <code>double</code> n&uacute;mero de los cr&eeacute;ditos de la Asignatura.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de excepci&oacute;n a
	 * la hora de trabajar con la capa de persistencia.
	 * @throws InvalidIdException si el identificador del Plan de Estudios o de la Asignatura que se 
	 * le ha pasado es inv&aacute;lido.
	 * @throws DuplicatedFieldException si la asignaci&oacute;n de la Asignatura al Plan de Estudios
	 * ya haba sido realizada anteriormente.
	 */
	@Override
	public void addSubject(int idCurriculum, int idSubject, String subjectType, String code, 
			double credits) throws DataStoreException, InvalidIdException, DuplicatedFieldException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos que existe el Plan de Estudios.
			Object objectCurriculum = existFieldValue(session, Table.CURRICULUM, "id", idCurriculum);
			// Vemos que no es nulo y de tipo BOCurriculum
			if ((objectCurriculum == null) || (!(objectCurriculum instanceof BOCurriculum)))
			{
				// Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_CURRICULUM_ID));
			}
			// Comprobamos la existencia de la Asignatura.
			Object objectSubject = existFieldValue(session, Table.SUBJECTS, "id", idSubject);
			// Vemos que no es nulo y adems es de tipo BOSubject
			if ((objectSubject == null) || (!(objectSubject instanceof BOSubject)))
			{
				// Lanzamos una InvalidIdException
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_SUBJECT_ID));
			}
			// Vemos si previamente se haba realizado la asignacin. Creamos la sentencia HQL.
			Query query = session.createQuery("from BOSubjectCurriculum where subject=? and curriculum=?");
			// Establecemos los parmetros.
			query.setParameter(0, idSubject, Hibernate.INTEGER);
			query.setParameter(1, idSubject, Hibernate.INTEGER);
			// Ejecutamos la sentencia y vemos el nmero de resultados.
			if (query.list().size() == 0)
			{
				// No existe el registro dentro de la capa de persistencia.
				// Cast del objeto BOCurriculum
				BOCurriculum boCurriculum = (BOCurriculum) objectCurriculum;
				// Cast del objeto BOSubject
				BOSubject boSubject = (BOSubject) objectSubject;
				// Creamos un objeto BOSubjectCurriculum
				BOSubjectCurriculum boSubjectCurriculum = new BOSubjectCurriculum(boSubject, 
						boCurriculum, code, credits, subjectType);
				// Guardamos el objeto en la capa de persistencia.
				session.saveOrUpdate(boSubjectCurriculum);
				// Consolidamos la operacin.
				session.flush();
			}
			else
			{
				// Ya exista un registro con esta informacin as que lanzamos una DuplicatedFieldException.
				throw (new DuplicatedFieldException(DuplicatedFieldException.MSG_SUBJECT_CURRICULUM_DUPLICATED));
			}
		}
		finally 
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado por un administrador del sistema cuando quiera 
	 * registrar un nuevo Plan de Estudios dentro de la aplicaci&oacute;n.</p>
	 * 
	 * @param name <code>String</code> nombre del Plan de Estudios.
	 * @param description <code>String</code> descripci&oacute;n del Plan de Estudios.
	 * @param type <code>String</code> tipo del Plan de Estudios.
	 * @return <code>TOCurriculum</code> con toda la informaci&oacute;n correspondiente al 
	 * Plan de Estudios que se acaba de crear.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de excepci&oacute;n a
	 * la hora de trabajar con la capa de persistencia.
	 * @throws DuplicatedFieldException si dentro del sistema ya existe un Plan de Estudios 
	 * registrado cuyo nombre y tipo concidan con los valores que se pasan en los par&aacute;metros
	 * <i>name</i> y <i>type</i> respectivamente.
	 */
	@Override
	public TOCurriculum createPlan(String name, String description, String type) 
			throws DataStoreException, DuplicatedFieldException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Vamos a comprobar si existe algn Plan de Estudios registrado cuyo nombre y tipo
			// coincidan con los valores que se le pasan como parmetro. Preparamos la consulta HQL.
			Query query = session.createQuery("from BOCurriculum where name=? and type=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, WebUtils.stringToHTML(name), Hibernate.STRING);
			query.setParameter(1, type, Hibernate.STRING);
			// Ejecutamos la consulta y vemos el resultado.
			if (query.list().size() > 0)
			{
				// Lanzamos una DuplicatedFieldException.
				throw (new DuplicatedFieldException(DuplicatedFieldException.MSG_CURRICULUM_DUPLICATED));
			}
			// No existe el Plan de Estudios. Construimos un objeto BOCurriculum a partir de la 
			// informacin que se ha pasado como parmetro.
			BOCurriculum curriculum = new BOCurriculum(name, description, type);
			// Guardamos el Plan de Estudios en la base de datos.
			session.saveOrUpdate(curriculum);
			// Consolidamos la operacin.
			session.flush();
			// Retornamos el objeto TOCurriculum equivalente al que acabamos de guardar.
			return (curriculum.getTOCurriculum());
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado para eliminar del sistema el Plan de Estudios
	 * cuyo identificador se le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador del Plan de Estudios.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de excepci&oacute;n a
	 * la hora de trabajar con la capa de persistencia.
	 * @throws InvalidIdException si el identificador que se ha pasado como par&aacute;metro 
	 * no corresponde con el de ninguno de los Planes de Estudio que est&aacute;n registrados.
	 */
	@Override
	public void deletePlan(int id) throws DataStoreException, InvalidIdException
	{
		// Creamos un sesin de Hibernate
		Session session = this.sessionFactory.openSession();
		try
		{
			// Vemos si existe un Plan de Estudios con el identificador que se ha pasado.
			Object object = existFieldValue(session, Table.CURRICULUM, "id", id);
			// Comprobamos si existe y adems es de tipo BOCurriculum
			if ((object != null) && (object instanceof BOCurriculum))
			{
				// Hacemos un cast del objeto.
				BOCurriculum curriculum = (BOCurriculum) object;
				// Obtenemos la lista de Asignaturas asignadas. 
				List<Object> list_subjects = getAll(session, Table.SUBJECTS_CURRICULUM, "curriculum", id);
				// Recorremos la lista de Asignaturas.
				for (Object obj : list_subjects)
				{
					// Comprobamos que es de tipo BOSubjectCurriculum
					if (obj instanceof BOSubjectCurriculum)
					{
						// Eliminamos el objeto.
						session.delete(((BOSubjectCurriculum) obj));
						// Consolidamos la operacin
						session.flush();
					}
				}
				// Obtenemos la lista de Aulas creadas en el Plan de Estudios.
				List<Object> list_classroom = getAll(session, Table.CLASSROOM, "curriculum", id);
				// Recorremos la lista de Aula.
				for (Object obj : list_classroom)
				{
					// Comprobamos que el elemento es de tipo BOClassroom.
					if (obj instanceof BOClassroom)
					{
						// Eliminamos el objeto.
						session.delete(((BOClassroom) obj));
						// Consolidamos la operacin.
						session.flush();
					}
				}
				// Eliminamos el plan de estudios del sistema.
				session.delete(curriculum);
				// Consolidamos la operacin.
				session.flush();
			}
			else
			{
				// No existe el Plan de Estudios dentro del sistema. Lanzamos una excepcin.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_CURRICULUM_ID));
			}
		}
		finally
		{
			// Intentamos cerrar la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado para obtener la informaci&oaute;n del Plan de 
	 * Estudios cuyo identificador se recibe como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador del Plan de Estudios.
	 * @return <code>TOCurriculum</code> con la informaci&oacute;n del Plan de Estudios cuyo
	 * identificador es igual al par&aacute;metro <i>id</i>.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de excepci&oacute;n a
	 * la hora de trabajar con la capa de persistencia.
	 * @throws InvalidIdException si el identificador que se ha pasado como par&aacute;metro 
	 * no corresponde con el de ninguno de los Planes de Estudio que est&aacute;n registrados.
	 */
	@Override
	public TOCurriculum getPlan(int id) throws DataStoreException, InvalidIdException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Vemos si existe un Plan de Estudios con ese identificador.
			Object object = existFieldValue(session, Table.CURRICULUM, "id", id);
			// Comprobamos que no sea nulo y adems que sea de tipo BOCurriculum
			if ((object != null) && (object instanceof BOCurriculum))
			{
				// Catch del objeto.
				BOCurriculum curriculum = (BOCurriculum) object;
				// Retornamos el TOCurriculum equivalente.
				return curriculum.getTOCurriculumComplete();
			}
			else
			{
				// No existe el Plan de Estudios as que lanzamos una excepcin.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_CURRICULUM_ID));
			}
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>Obtiene la lista de asignaturas que todava no est&aacute;n asignadas al Plan 
	 * de Estudios cuyo identificador se le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador del Plan de Estudios.
	 * @return <code>List&lt;TOSubject&gt;</code> donde cada uno de los elementos ser&aacute;
	 * una asignatura no asignada al Plan de Estudios.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de excepci&oacute;n a
	 * la hora de trabajar con la capa de persistencia.
	 * @throws InvalidIdException si el identificador que se ha pasado como par&aacute;metro 
	 * no corresponde con el de ninguno de los Planes de Estudio que est&aacute;n registrados.
	 */
	@Override
	public List<TOSubject> getUnassignedSubjects(int id) 
			throws DataStoreException, InvalidIdException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos si existe el Plan de Estudios.
			Object object = existFieldValue(session, Table.CURRICULUM, "id", id);
			// Vemos si "object" es distinto de null y de tipo BOCurriculum
			if ((object != null) && (object instanceof BOCurriculum))
			{
				// Cast a "object"
				BOCurriculum curriculum = (BOCurriculum) object;
				// Obtenemos la lista de todas las asignaturas registradas.
				List<Object> all_subjects_obj = getAll(session, Table.SUBJECTS);
				// Creamos un Set donde almacenaremos las asignaturas.
				Set<BOSubject> set_subjects = new HashSet<BOSubject>();
				// Recorremos uno a uno "all_subjects_obj"
				for (Object obj : all_subjects_obj)
				{
					// Comprobamos que "obj" es de tipo BOSubject
					if (obj instanceof BOSubject)
					{
						// Cast de obj y lo aadimos al conjunto.
						set_subjects.add((BOSubject) obj);
					}
				}
				// Recorremos la lista de asignaturas que ya estn asignadas al Plan.
				for (BOSubjectCurriculum bo : curriculum.getSubjects())
				{
					// Eliminamos la asignatura del conjunto.
					set_subjects.remove(bo.getSubject());
				}
				// Creamos la lista de Asignaturas a retornar.
				List<TOSubject> list_subjects = new ArrayList<TOSubject>();
				// Recorremos todos los elementos del conjunto.
				for (BOSubject bo : set_subjects)
				{
					// Aadimos el asignatura a la lista a retornar.
					list_subjects.add(bo.getTOSubject());
				}
				// Ordenamos la lista de resultados.
				Collections.sort(list_subjects);
				// Retornamos la lista.
				return list_subjects;
			}
			// Si llegamos a este punto querr decir que el "id" no corresponde con el 
			// de un Plan de Estudios registrado por lo que lanzamos una excepcin.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_CURRICULUM_ID));
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>Retorna una lista de objetos <code>TOCurriculum</code> cada uno de los cuales
	 * representar&aacute; a uno de los Planes de Estudios que est&aacute;n registrados
	 * dentro del sistema.</p>
	 * 
	 * @return <code>List&lt;TOCurriculum&gt;</code> con la informaci&oacute;n de los
	 * Planes de Estudio registrados.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de excepci&oacute;n a
	 * la hora de trabajar con la capa de persistencia.
	 */
	@Override
	public List<TOCurriculum> listPlan() throws DataStoreException
	{
		// Inicializamos la sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Obtenemos la lista con todos los Planes de Estudio.
			List<Object> list = getAll(session, Table.CURRICULUM);
			// Creamos la lista de objeto TOCurriculum que ser retornada.
			List<TOCurriculum> curriculumList = new ArrayList<TOCurriculum>();
			// Recorremos todos los elementos de "list".
			for (Object o : list)
			{
				// Comprobamos que el elemento es de tipo BOCurriculum
				if (o instanceof BOCurriculum)
				{
					// Realizamos un cast de "o"
					BOCurriculum bo = (BOCurriculum) o;
					// Aadimos el objeto TOCurriculum equivalente a la lista.
					curriculumList.add(bo.getTOCurriculum());
				}
			}
			// Una vez aadidos todos los planes los ordenamos por nombre.
			Collections.sort(curriculumList);
			// Retornamos la lista de objetos TOCurriculum.
			return curriculumList;
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un administrador del sistema cuando 
	 * desee modificar la informaci&oacute;n de un Plan de Estudios que est&aacute;.</p> 
	 * 
	 * @param id <code>int</code> identificador del Plan de Estudios.
	 * @param description <code>String</code> nueva descripci&oacute;n del Plan de Estudios.
	 * @return
	 * @throws DataStoreException si se produce alg&uacute;n tipo de excepci&oacute;n a
	 * la hora de trabajar con la capa de persistencia.
	 * @throws InvalidIdException  si el identificador que se ha pasado como par&aacute;metro 
	 * no corresponde con el de ninguno de los Planes de Estudio que est&aacute;n registrados.
	 */
	@Override
	public TOCurriculum modifyPlan(int id, String description) 
			throws DataStoreException, InvalidIdException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos la existencia del Plan de Estudios.
			Object object = existFieldValue(session, Table.CURRICULUM, "id", id);
			// Comprobamos que "object" no es null y adems que es de tipo "BOCurriculum"
			if ((object != null) && (object instanceof BOCurriculum))
			{
				// Realizamos un cast a "object".
				BOCurriculum curriculum = (BOCurriculum) object;
				// Establecemos el nuevo valor de la descripcin.
				curriculum.setDescription(WebUtils.stringToHTML(description));
				// Actualizamos los datos.
				session.saveOrUpdate(curriculum);
				// Consolidamos la operacin.
				session.flush();
				// Retornamos el objeto equivalente.
				return curriculum.getTOCurriculum();
			}
			else
			{
				// Si llegamos a este punto querr decir que el identificador que 
				// se ha recibido no es el de un Plan de Estudios registrado por lo
				// que lanzaremos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_CURRICULUM_ID));
			}
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un administrador del sistema cuando desee
	 * eliminar la asignaci&oacute;n de una Asignatura a un determinado Plan de Estudios.</p>
	 * 
	 * @param idCurriculum <code>int</code> identificador del Plan de Estudios.
	 * @param idSubject <code>int</code> identificador de la Asignatura.
	 * @throws DataStoreException  si se produce alg&uacute;n tipo de excepci&oacute;n a
	 * la hora de trabajar con la capa de persistencia.
	 * @throws InvalidIdException si no se ha realizado previamente la asignaci&oacute;n
	 * de la asignatura al Plan de Estudios previamente.
	 */
	@Override
	@SuppressWarnings("unchecked")
	public void removeSubject(int idCurriculum, int idSubject) throws DataStoreException, InvalidIdException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Preparamos la sentencia HQL para comprobar que se ha realiza previamente la Asignatura al
			// Plan de Estudios.
			Query query = session.createQuery("from BOSubjectCurriculum where curriculum=? and subject=?");
			// Asignamos los parmetros.
			query.setParameter(0, idCurriculum, Hibernate.INTEGER);
			query.setParameter(1, idSubject, Hibernate.INTEGER);
			// Ejecutamos la sentencia.
			List<BOSubjectCurriculum> listBOSubjectCurriculum = query.list();
			// Comprobamos el nmero de registros retornados.
			if (listBOSubjectCurriculum.size() == 1)
			{
				// Borramos el registro.
				session.delete(listBOSubjectCurriculum.get(0));
				// Consolidamos la operacin.
				session.flush();
			}
			else
			{
				// No se haba realizado la asignacin as que lanzamos una excepcin.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_SUBJECT_ASSIGN_CURRICULUM));
			}
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
}
