package edu.uoc.pfc.business.services;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.hibernate.Hibernate;
import org.hibernate.Query;
import org.hibernate.Session;

import edu.uoc.pfc.business.exceptions.BusinessRulesViolationException;
import edu.uoc.pfc.business.exceptions.DataStoreException;
import edu.uoc.pfc.business.exceptions.DuplicatedFieldException;
import edu.uoc.pfc.business.exceptions.InvalidIdException;
import edu.uoc.pfc.business.interfaces.IBQuestionBankManager;
import edu.uoc.pfc.business.objects.BOQuestion;
import edu.uoc.pfc.business.objects.BOQuestionCategory;
import edu.uoc.pfc.business.objects.BOQuestionMatch;
import edu.uoc.pfc.business.objects.BOQuestionMatchAnswer;
import edu.uoc.pfc.business.objects.BOQuestionMultipleChoices;
import edu.uoc.pfc.business.objects.BOQuestionMultipleChoicesAnswer;
import edu.uoc.pfc.business.objects.BOQuestionNumeric;
import edu.uoc.pfc.business.objects.BOQuestionShortAnswer;
import edu.uoc.pfc.business.objects.BOQuestionTrueOrFalse;
import edu.uoc.pfc.business.objects.BOUser;
import edu.uoc.pfc.business.transfers.TOQuestion;
import edu.uoc.pfc.business.transfers.TOQuestionCategory;
import edu.uoc.pfc.business.transfers.TOQuestionMatch;
import edu.uoc.pfc.business.transfers.TOQuestionMatchAnswer;
import edu.uoc.pfc.business.transfers.TOQuestionMultipleChoices;
import edu.uoc.pfc.business.transfers.TOQuestionMultipleChoicesAnswer;
import edu.uoc.pfc.business.transfers.TOQuestionNumeric;
import edu.uoc.pfc.business.transfers.TOQuestionShortAnswer;
import edu.uoc.pfc.business.transfers.TOQuestionTrueOrFalse;
import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Clase de negocio que implementa a la interfaz <code>IBQuestionManager</code>
 * la cual ofrece los servicios que la capa de negocio proporciona a la capa de 
 * presentaci&oacute;n.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class IBQuestionBankManagerImpl extends IBBaseImpl implements IBQuestionBankManager {
	
	// -------- CONSTRUCTORES.
	/**
	 * <p><code>Construye un nuevo objeto <code>IBQuestionManagerImpl</code> e inicializa el 
	 * marco de persistencia <i>Hibernate</i>.</p>
	 * 
	 * @throws DataStoreException si se produce una excepci&oacute;n a la hora de iniciar
	 * el marco de persistencia.
	 */
	public IBQuestionBankManagerImpl() throws DataStoreException
	{
		// Constructor de la superclase.
		super();
	}
	
	// -------- MTOODOS PRIVADOS.
	/**
	 * <p>Comprueba si el identificador de usuario que recibe corresponde con el de 
	 * un usuario definido como un Profesor dentro de la aplicaci&oacute;n.</p>
	 * 
	 * @param idTeacher <code>int</code> el identificador del Usuario a comprobar.
	 * @return <code>true</code> si el identificador del usaurio corresponde con el de
	 * un Profesor o <code>false</code> en caso contrario.
	 * @thows DataStoreException si se produce alg&uacute;n error a la hora de trabajar
	 * con la capa de persistencia.
	 * @throws InvalidIdException siempre que el identificador recibido no corresponde
	 * con el de ninguno de los Usuarios que est&aacute;n registrados dentro de la 
	 * aplicaci&oacute;n.
	 */
	private boolean isTeacher(int idUser) throws DataStoreException, InvalidIdException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos si existe un Usuario con el identificador "idUser".
			Object objUser = existFieldValue(session, Table.USERS, "id", idUser);
			// Comprobamos que objUser no sea null y sea de tipo BOUser.
			if ((objUser != null) && (objUser instanceof BOUser))
			{
				// Retornamos valor del atributo "teacher" del objeto.
				return ((BOUser) objUser).isTeacher();
			}
			// Si llegamos a este punto querr decir que no existe el usuario por
			// lo que lanzamos una InvalidIdException.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_USER_ID));
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera a&ntilde;adir una 
	 * nueva Categor&iaacute;a de Preguntas a su Banco de Preguntas.</p>
	 *
	 * @param idTeacher <code>int</code> identificador del profesor.
	 * @param name <code>String</code> nombre de la nueva Categor&iacute;a de Preguntas.
	 * @param description <code>String</code> descripci&oacute;n de la Categor&iacute;a de Preguntas.
	 * @return <code>TOQuestionCategory</code> con la informaci&oacute;n de la nueva Categor&iacute;a
	 * de Preguntas.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de trabajar con la 
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador de profesor proporcionado no corresponde 
	 * con el de ninguno de los Profesores que est&aacute;n registrados en el sistema.
	 * @throws DuplicatedFieldException si en el sistema el Profesor ya posee una Categor&iacute;a
	 * de Preguntas con el mismo nombre que el que se ha pasada.
	 */
	@Override
	public TOQuestionCategory createCategory(int idTeacher, String name, 
			String description) throws DataStoreException, InvalidIdException, 
			DuplicatedFieldException
	{
		// Comprobamos que quien ha llamado al mtodo es un Profesor.
		if (isTeacher(idTeacher))
		{
			// Abrimos una sesin de Hibernate.
			Session session = this.sessionFactory.openSession();
			try
			{
				// Vamos a comprobar si el Profesor ya posee una Categora de Preguntas
				// con el mismo nombre. Sentencia HQL.
				Query query = session.createQuery("from BOQuestionCategory where user=? and name=?");
				// Establecemos los valores de los parmetros.
				query.setParameter(0, idTeacher, Hibernate.INTEGER);
				query.setParameter(1, WebUtils.stringToHTML(name), Hibernate.STRING);
				// Ejecutamos la consulta, vemos el nmero de resultados.
				if (query.list().size() != 0)
				{
					// Ya se ha creado una categora de pregunta con el nombre pasado as que tenemos
					// que lanzar una DuplicatedFieldException.
					throw (new DuplicatedFieldException(DuplicatedFieldException.MSG_QUESTION_CATEGORY_DUPLICATED));
				}
				// Si llegamos a este punto querr decir que tenemos que crear la categora de preguntas.
				// Creamos un objeto BOUser a partir del identificador.
				BOUser teacher = new BOUser(idTeacher);
				// Creamos un objeto BOQuestionCategory.
				BOQuestionCategory category = new BOQuestionCategory(teacher, WebUtils.stringToHTML(name), 
						WebUtils.stringToHTML(description));
				// Guardamos la informacin en la capa de persistencia.
				session.saveOrUpdate(category);
				// Confirmamos la operacin.
				session.flush();
				// Retornamos el objeto TOQuestionCategory equivalente al que acabamos de registrar.
				return category.getTOQuestionCategory();
			}
			finally
			{
				// Cerramos la sesin de Hibernate.
				closeSession(session);
			}
		}
		// Si llegamos a este punto querr decir que quien ha llamado al mtodo no 
		// es un Profesor as que lanzamos una InvalidIdException.
		throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
	}	
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quira a&ntilde;adir 
	 * una Preguntas a una de sus Cateogor&iacutes de Preguntas.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.  
	 * @param idCategory <code>int</code> identificador de la Categor&iacute;
	 * @param question <code>TOQuestion</code> con la informaci&oacute;n de la Pregunta.
	 * @return <code>TOQuestion</code> que contendr&aacute; toda al informaci&oacute;n de
	 * la pregunta una vez ha sido registrada en el sistema.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de trabajar con la 
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador de profesor proporcionado no corresponde 
	 * con el de ninguno de los Profesores que est&aacute;n registrados en el sistema o si el 
	 * identificador de la Categor&iacute; a de Preguntas no corresponde con el de ninguna de 
	 * las Categor&iacute;as de Preguntas definidas el por el Profesor.
	 * @throws DuplicatedFieldException si ya existe una pregunta dentro de la Categor&iacute;a
	 * con el mismo nombre y del mismo tipo que la que se pretende crear.
	 * @throws BusinessRulesViolationException si se intenta crear una pregunta de tipo Opci&oacute;n
	 * M&uacute;ltiple sin haber especificado al menos dos opciones de respuesta.
	 */
	@Override
	public TOQuestion createQuestion(int idTeacher, int idCategory, TOQuestion question)
			throws DataStoreException, InvalidIdException, DuplicatedFieldException,
			BusinessRulesViolationException
	{
		// Vemos que "question" no sea nulo.
		if (question == null)
		{
			// Lanzamos una NullPointerException.
			throw (new NullPointerException("Parametro TOQuestion nulo."));
		}
		// Comprobamos que quien ha llamado al mtodo es un Profesor.
		if (!isTeacher(idTeacher))
		{
			// Al no se un Profesor lanzamos una excepcin.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
		}
		// Llegados a este punto sabemos que tenemos a un Profesor. Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Vamos a comprobar que la categora de preguntas que se quiere eliminar es
			// del usuario que ha llamado al mtodo. Sentencia HQL.
			Query query = session.createQuery("from BOQuestionCategory where id=? and user=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, idCategory, Hibernate.INTEGER);
			query.setParameter(1, idTeacher, Hibernate.INTEGER);
			// Ejecutamos la consulta y vemos el nmero de resultados devueltos.
			if ((query.list().size() != 1) || (!(query.list().get(0) instanceof BOQuestionCategory)))
			{
				// El identificador de la categora no corresponde con el de una categora
				// de preguntas definida por el Profesor. Lanzamos una excepcin.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_QUESTION_CATEGORY_ID));
			}
			// Cast al objeto BOQuestionCategory
			BOQuestionCategory category = (BOQuestionCategory) query.list().get(0);
			// Tenemos que comprobar si ya existe la pregunta dentro del sistema. Sentencia HQL.
			query = session.createQuery("from BOQuestion where category=? and title=? and type=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, idCategory, Hibernate.INTEGER);
			query.setParameter(1, WebUtils.stringToHTML(question.getTitle()), Hibernate.STRING);
			// Variable local para guardar el objeto TOQuestion equivalente al que se crea.
			TOQuestion toReturn = null;
			// Tenemos que ver el tipo de pregunta que hemos recibido.
			if (question instanceof TOQuestionTrueOrFalse)
			{
				// Establecemos el valor del ltimo parmetro.
				query.setParameter(2, BOQuestion.TYPE_TRUE_OR_FALSE, Hibernate.STRING);
				// Ejecutamos la consulta y contamos los resultados.
				if (query.list().size() != 0)
				{
					// Ya existe la pregunta. Lanzamos una DuplicatedFieldException.
					throw (new DuplicatedFieldException(DuplicatedFieldException.MSG_QUESTION_DUPLICATED));
				}
				// Estamos ante una pregunta de tipo verdadero o falso. Cast.
				TOQuestionTrueOrFalse toQuestionTF = (TOQuestionTrueOrFalse) question;
				// Creamos un objeto BOQuestionTrueOrFalse a partir de TOQuestion.
				BOQuestionTrueOrFalse questionTF = new BOQuestionTrueOrFalse(toQuestionTF);
				// Le asignamos la categora.
				questionTF.setCategory(category);
				// Guardamos la pregunta.
				session.saveOrUpdate(questionTF);
				// Consolidamos la operaci&oacute;
				session.flush();
				// Establecemos el valor TOQuestion a retornar.
				toReturn = questionTF.getTOQuestion();
			}
			else if (question instanceof TOQuestionNumeric)
			{
				// Estamos ante una pregunta numrica. Establecemos el valor del ltimo parmetro.
				query.setParameter(2, BOQuestion.TYPE_NUMERIC, Hibernate.STRING);
				// Ejecutamos la consulta y contamos los resultados.
				if (query.list().size() != 0)
				{
					// Ya existe la pregunta. Lanzamos una DuplicatedFieldException.
					throw (new DuplicatedFieldException(DuplicatedFieldException.MSG_QUESTION_DUPLICATED));
				}
				// Cast del tipo de preguta.
				TOQuestionNumeric toQuestionNU = (TOQuestionNumeric) question;
				// Creamos un objeto BOQuestionNumeric a partir de toQuestionNU
				BOQuestionNumeric questionNU = new BOQuestionNumeric(toQuestionNU);
				// Le asignamos la categora.
				questionNU.setCategory(category);
				// Guardamos la pregunta.
				session.saveOrUpdate(questionNU);
				// Consolidamos la operacin.
				session.flush();
			}
			else if (question instanceof TOQuestionShortAnswer)
			{
				// Estamos ante una pregunta de respuesta corta. Establecemos el valor del parmetro.
				query.setParameter(2, BOQuestion.TYPE_SHORT_ANSWER, Hibernate.STRING);
				// Ejecutamos la consulta y contamos los resultados.
				if (query.list().size() != 0)
				{
					// Ya existe la pregunta. Lanzamos una DuplicatedFieldException.
					throw (new DuplicatedFieldException(DuplicatedFieldException.MSG_QUESTION_DUPLICATED));
				}
				// Cast del tipo de pregunta.
				TOQuestionShortAnswer toQuestionSA = (TOQuestionShortAnswer) question;
				// Creamos un objeto BOQuestionShortAnswer a partir de toQuestionSA.
				BOQuestionShortAnswer boQuestionSA = new BOQuestionShortAnswer(toQuestionSA);
				// Le asignamos la categora.
				boQuestionSA.setCategory(category);
				// Guardamos la pregunta.
				session.saveOrUpdate(boQuestionSA);
				// Consolidamos la operacin.
				session.flush();
			}
			else if (question instanceof TOQuestionMultipleChoices)
			{
				// Estamos ante una pregunta de mltiples respuestas. Establecemos el valor del parmetro.
				query.setParameter(2, BOQuestion.TYPE_MULTIPLE_CHOICES, Hibernate.STRING);
				// Ejecutamos la consulta y contamos los resultados.
				if (query.list().size() != 0)
				{
					// Ya existe la pregunta. Lanzamos una DuplicatedFieldException.
					throw (new DuplicatedFieldException(DuplicatedFieldException.MSG_QUESTION_DUPLICATED));
				}
				// Cast del tipo de pregunta.
				TOQuestionMultipleChoices toMC = (TOQuestionMultipleChoices) question;
				// Creamos un objeto BOQuestionMultipleChoices a partir de toMC.
				BOQuestionMultipleChoices boMC = new BOQuestionMultipleChoices(toMC);
				// Establecemos la categora.
				boMC.setCategory(category);
				// Guardamos la pregunta.
				session.saveOrUpdate(boMC);
				// Consolidamos la operacin.
				session.flush();
				// Comprobamos que hay al menos dos opciones de respuesta.
				if ((toMC.getAnswers() != null) && (toMC.getAnswers().size() >= 2))
				{
					// Recorremos la lista de respuestas.
					for (TOQuestionMultipleChoicesAnswer toAnswer : toMC.getAnswers())
					{
						// Creamos un objeto BOQuestionMultipleChoicesAnswer a partir de toAnswer.
						BOQuestionMultipleChoicesAnswer boAnswer = 
								new BOQuestionMultipleChoicesAnswer(boMC, toAnswer);
						// Guardamos la informacin de la opcin de respuesta.
						session.saveOrUpdate(boAnswer);
						// Consolidamos la operacin.
						session.flush();
					}
				}
				else
				{
					// No hay opciones de respuesta. Lanzamos una BusinessRulesViolationException;
					throw (new BusinessRulesViolationException(BusinessRulesViolationException.MSG_NO_ANSWERS_EXPECIFIED));
				}
			}
			else if (question instanceof TOQuestionMatch)
			{
				// Estamos ante una pregunta de mltiples respuestas. Establecemos el valor del parmetro.
				query.setParameter(2, BOQuestion.TYPE_MATCH, Hibernate.STRING);
				// Ejecutamos la consulta y contamos los resultados.
				if (query.list().size() != 0)
				{
					// Ya existe la pregunta. Lanzamos una DuplicatedFieldException.
					throw (new DuplicatedFieldException(DuplicatedFieldException.MSG_QUESTION_DUPLICATED));
				}
				// Cast del tipo de pregunta.
				TOQuestionMatch toMA = (TOQuestionMatch) question;
				// Creamos un objeto BOQuestionMatch a partir de la informacin de "toMA"
				BOQuestionMatch boMA = new BOQuestionMatch(toMA);
				// Establecemos la categora.
				boMA.setCategory(category);
				// Guardamos la pregunta.
				session.saveOrUpdate(boMA);
				// Consolidamos la operacin.
				session.flush();
				// Comprobamos que hay al menos dos parejas de respuestas.
				if ((toMA.getAnswers() != null) && (toMA.getAnswers().size() >= 2))
				{
					// Recorremos la lista de parejas de respuestas.
					for (TOQuestionMatchAnswer answer : toMA.getAnswers())
					{
						// Construimo el objeto BOQuestionMatchAnswer.
						BOQuestionMatchAnswer boAnswer = new BOQuestionMatchAnswer(boMA, answer);
						
						System.out.println("question = " + boAnswer.getQuestion().getId());
						System.out.println("text 1   = " + boAnswer.getText1());
						System.out.println("text 2   = " + boAnswer.getText2());
						System.out.println("posicion = " + boAnswer.getPosition());
						System.out.println("refuerzo = " + boAnswer.getExplain());
						
						// Guardamos la informacin de la respuesta.
						session.saveOrUpdate(boAnswer);
						// Consolidamos la operacin.
						session.flush();
					}
				}
				else
				{
					// No hay opciones de respuesta. Lanzamos una BusinessRulesViolationException;
					throw (new BusinessRulesViolationException(BusinessRulesViolationException.MSG_NO_ANSWERS_EXPECIFIED));
				}
			}
			// Retornamos el objeto TOQuestion equivalente al que acabamos de guardar.
			return toReturn;
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera eliminar una 
	 * Categor&iacute;a de Preguntas del sistema.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idCategory <code>int</code> identificador de la Categor&iacute;a de Preguntas.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de trabajar con la 
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador del Profesor no corresponde con el de un
	 * Usuario de Profesor registrado o si el identificador de la Categor&iacute;a no corresponde
	 * con el de una Categor&iacute;a de preguntas dentro del Banco de Preguntas del Profesor.
	 */
	@Override
	public void deleteCategory(int idTeacher, int idCategory) throws DataStoreException,
			InvalidIdException
	{
		// Comprobamos que quien ha llamado al mtodo es un Profesor.
		if (!isTeacher(idTeacher))
		{
			// Al no se un Profesor lanzamos una excepcin.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
		}
		// Llegados a este punto sabemos que tenemos a un Profesor. Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Vamos a comprobar que la categora de preguntas que se quiere eliminar es
			// del usuario que ha llamado al mtodo. Sentencia HQL.
			Query query = session.createQuery("from BOQuestionCategory where id=? and user=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, idCategory, Hibernate.INTEGER);
			query.setParameter(1, idTeacher, Hibernate.INTEGER);
			// Ejecutamos la consulta y vemos el nmero de resultados devueltos.
			if ((query.list().size() != 1) || (!(query.list().get(0) instanceof BOQuestionCategory)))
			{
				// El identificador de la categora no corresponde con el de una categora
				// de preguntas definida por el Profesor. Lanzamos una excepcin.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_QUESTION_CATEGORY_ID));
			}
			else
			{
				// Obtenemos el objeto BOQuestionCategory que representa a la Categora
				BOQuestionCategory category = (BOQuestionCategory) query.list().get(0);
				// Tenemos que borrar todas las preguntas que pertenezcan a dicha categora
				List<Object> list_objQuestion = getAll(session, Table.QUESTIONS, "category", 
						category.getId());
				// Recorremos la lista de resultados.
				for (Object objQuestion : list_objQuestion)
				{
					// Comprobamos que objQuestion es de tipo BOQuestion.
					if (objQuestion instanceof BOQuestion)
					{
						// Borramos el objeto.
						session.delete((BOQuestion) objQuestion);
						// Consolidamos la operacin.
						session.flush();
					}
				}
				// Borramos el objeto de la capa de persistencia.
				session.delete(category);
				// Confirmamos la operacin.
				session.flush();
			}
		}
		finally
		{
			// Cerramos la sesin de Hibernate
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera eliminar una Pregunta
	 * dentro de su Banco de Preguntas.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idQuestion <code>int</code> identificador de la Pregunta.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de trabajar con la 
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador del Profesor no corresponde con el de un
	 * Usuario de Profesor registrado o el identificador de la Pregunta no se corresponde con 
	 * el de una Pregunta que pertenece al Banco de Preguntas del Profesor.
	 */
	@Override
	public void deleteQuestion(int idTeacher, int idQuestion) throws DataStoreException,
			InvalidIdException
	{
		// Comprobamos que quien ha llamado al mtodo es un Profesor.
		if (!isTeacher(idTeacher))
		{
			// Al no se un Profesor lanzamos una excepcin.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
		}
		// Llegados a este punto sabemos que tenemos a un Profesor. Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos si existe la Pregunta dentro del sistema.
			Object objQuestion = existFieldValue(session, Table.QUESTIONS, "id", idQuestion);
			// Comprobamos que el objeto no es nulo y que es de tipo BOQuestion.
			if ((objQuestion == null) || (!(objQuestion instanceof BOQuestion)))
			{
				// Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_QUESTION_ID));
			}
			// Cast de objQuestion.
			BOQuestion question = (BOQuestion) objQuestion;
			// Vemos si la pregunta es del profesor. Sentencia HQL.
			if (question.getCategory().getUser().getId() == idTeacher)
			{
				// Comprobamos si la pregunta es de tipo BOQuestionMultipleChoices.
				if (objQuestion instanceof BOQuestionMultipleChoices)
				{
					// Tenemos que borrar todas las opciones (respuestas). Cast de objQuestion.
					BOQuestionMultipleChoices boMC = (BOQuestionMultipleChoices) objQuestion;
					// Recorremos toda la lista de respuestas.
					for (BOQuestionMultipleChoicesAnswer boAnswer : boMC.getAnswers())
					{
						// Borramos la respuesta.
						session.delete(boAnswer);
						// Consolidamos la operacin
						session.flush();
					}
				}
				// Comprobamos si la pregunta es de tipo BOQuestionMatch
				if (objQuestion instanceof BOQuestionMatch)
				{
					// Tenemos que borrar todas las opciones (parejas). Cast de objQuestion.
					BOQuestionMatch boMA = (BOQuestionMatch) objQuestion;
					// Recorremos la lista de opciones de respuesta.
					for (BOQuestionMatchAnswer boAnswer : boMA.getAnswers())
					{
						// Borramos la respuesta.
						session.delete(boAnswer);
						// Consolidamos la operacin.
						session.flush();
					}
				}
				// La pregunta es del Profesor as que la borramos. 
				session.delete((BOQuestion) objQuestion);
				// Consolidamos la operacin.
				session.flush();
			}
			else
			{
				// La pregunta no es del profesor as que lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_QUESTION_ID));
			}
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando obtener la informaci&oacute;n
	 * de una de sus Categor&iacute;a de Preguntas.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idCategory <code>int</code> identificador de la Categor&iacute;a de Preguntas.
	 * @return <code>TOQuestionCategory</code> con toda la informaci&oacute;n de la Categor&iacute;a.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de trabajar con la 
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador del Profesor no corresponde con el de un
	 * Usuario de Profesor registrado o si el identificador de la Categor&iacute;a no corresponde
	 * con el de una Categor&iacute;a de preguntas dentro del Banco de Preguntas del Profesor.
	 */
	@Override
	public TOQuestionCategory getCategory(int idTeacher, int idCategory) 
			throws DataStoreException, InvalidIdException
	{

		// Comprobamos que quien ha llamado al mtodo es un Profesor.
		if (!isTeacher(idTeacher))
		{
			// Al no se un Profesor lanzamos una excepcin.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
		}
		// Llegados a este punto sabemos que tenemos a un Profesor. Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Vamos a comprobar que la categora de preguntas que se quiere eliminar es
			// del usuario que ha llamado al mtodo. Sentencia HQL.
			Query query = session.createQuery("from BOQuestionCategory where id=? and user=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, idCategory, Hibernate.INTEGER);
			query.setParameter(1, idTeacher, Hibernate.INTEGER);
			// Ejecutamos la consulta y vemos el nmero de resultados devueltos.
			if ((query.list().size() != 1) || (!(query.list().get(0) instanceof BOQuestionCategory)))
			{
				// El identificador de la categora no corresponde con el de una categora
				// de preguntas definida por el Profesor. Lanzamos una excepcin.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_QUESTION_CATEGORY_ID));
			}
			else
			{
				// Obtenemos el objeto BOQuestionCategory que representa a la Categora
				BOQuestionCategory category = (BOQuestionCategory) query.list().get(0);
				// Retornamos el objeto TOQuestionCategory con toda la informacin.
				return category.getTOQuestionCategoryComplete();
			}
		}
		finally
		{ 
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo que permite recuperar la informaci&oacute;n de una Pregunta registrada en 
	 * el sistema y cuyo identificador se le pasa como par&aacute;metro.</p>
	 * 
	 * @param idQuestion <code>int</code> identificador de la Pregunta.
	 * @return <code>TOQuestion</code> con la informaci&oacute;n de la Pregunta.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con
	 * la capa de persistencia.
	 * @throws InvalidIdException si el identificador que se le pasa no se corresponde con el de
	 * ninguna de las Preguntas que est&aacute;n registradas en el sistema.
	 */
	@Override
	public TOQuestion getQuestion(int idQuestion) throws DataStoreException, InvalidIdException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos que el identificador que se le pasa corresponde con el de una de las 
			// Preguntas que estn registradas en el sistema.
			Object objQuestion = existFieldValue(session, Table.QUESTIONS, "id", idQuestion);
			// Comprobamos que no es nulo y es de tipo BOQuestion.
			if ((objQuestion == null) || (!(objQuestion instanceof BOQuestion)))
			{
				// Lanzamos una InvalidIdExcepcin ya que no existe la Preguntas.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_QUESTION_ID));
			}
			// Cast de objQuestion.
			BOQuestion question = (BOQuestion) objQuestion;
			// Referencia local a un objeto TOQuestin que retornaremos.
			TOQuestion to = null;
			// Vemos el tipo de pregunta de la que se trata.
			if (question instanceof BOQuestionTrueOrFalse)
			{
				// Asignamos to a un objeto TOQuestionTrueOrFalse.
				to = ((BOQuestionTrueOrFalse) question).getTOQuestionTrueOrFalse();
			}
			else if (question instanceof BOQuestionNumeric)
			{
				// Asignamos to a un objeto TOQuestionNumeric
				to = ((BOQuestionNumeric) question).getTOQuestionNumeric();
			}
			else if (question instanceof BOQuestionShortAnswer)
			{
				// Asignamos to a un objeto TOQuestionShortAnswer
				to = ((BOQuestionShortAnswer) question).getTOQuestionShortAnswer();
			}
			else if (question instanceof BOQuestionMultipleChoices)
			{
				// Asignamos to a un objeto TOQuestionMultipleChoices
				to = ((BOQuestionMultipleChoices) question).getTOQuestionMultipleChoices();
			}
			else if (question instanceof BOQuestionMatch)
			{
				// Asignamos to a un objeto TOQuestionMatch.
				to = ((BOQuestionMatch) question).getTOQuestionMatch();
			}
			// Retornamos el objeto TOQuestion.
			return to;
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera obtener la 
	 * informaci&oacute;n de una de sus Preguntas creadas en su Banco de Preguntas.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idQuestion <code>int</code> identificador de la Pregunta.
	 * @return <code>TOQuestion</code> con la informaci&oacute;n de la Pregunta.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de trabajar con la 
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador del Profesor no corresponde con el de un
	 * Usuario de Profesor registrado o si el identificador de la Pregunta no corresponde con
	 * el de una Pregunta que pertenece al Profesor:
	 */
	@Override
	public TOQuestion getQuestion(int idTeacher, int idQuestion)
			throws DataStoreException, InvalidIdException
	{
		// Comprobamos que quien ha llamado al mtodo es un Profesor.
		if (!isTeacher(idTeacher))
		{
			// Al no se un Profesor lanzamos una excepcin.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
		}
		// Llegados a este punto sabemos que tenemos a un Profesor. Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos si existe la Pregunta dentro del sistema.
			Object objQuestion = existFieldValue(session, Table.QUESTIONS, "id", idQuestion);
			// Comprobamos que el objeto no es nulo y que es de tipo BOQuestion.
			if ((objQuestion == null) || (!(objQuestion instanceof BOQuestion)))
			{
				// Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_QUESTION_ID));
			}
			// Cast de objQuestion.
			BOQuestion question = (BOQuestion) objQuestion;
			// Vemos si la pregunta es del profesor. 
			if (question.getCategory().getUser().getId() == idTeacher)
			{
				// Referencia local a un objeto TOQuestin que retornaremos.
				TOQuestion to = null;
				// Vemos el tipo de pregunta de la que se trata.
				if (question instanceof BOQuestionTrueOrFalse)
				{
					// Asignamos to a un objeto TOQuestionTrueOrFalse.
					to = ((BOQuestionTrueOrFalse) question).getTOQuestionTrueOrFalse();
				}
				else if (question instanceof BOQuestionNumeric)
				{
					// Asignamos to a un objeto TOQuestionNumeric
					to = ((BOQuestionNumeric) question).getTOQuestionNumeric();
				}
				else if (question instanceof BOQuestionShortAnswer)
				{
					// Asignamos to a un objeto TOQuestionShortAnswer
					to = ((BOQuestionShortAnswer) question).getTOQuestionShortAnswer();
				}
				else if (question instanceof BOQuestionMultipleChoices)
				{
					// Asignamos to a un objeto TOQuestionMultipleChoices
					to = ((BOQuestionMultipleChoices) question).getTOQuestionMultipleChoices();
				}
				else if (question instanceof BOQuestionMatch)
				{
					// Asignamos to a un objeto TOQuestionMatch.
					to = ((BOQuestionMatch) question).getTOQuestionMatch();
				}
				// Retornamos el objeto TOQuestion.
				return to;
			}
			else
			{
				// La pregunta no es del Profesor. Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_QUESTION_ID));
			}
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado para obtener la lista de Categor&iacute;as
	 * de Preguntas que el usuario couyo identificador se pasa en el par&aacute;metro <i>idTeacher</i>
	 * tiene definido dentro de su Bando de Preguntas.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del profesor.
	 * @return <code>List&lt;TOQuestionCategory&gt;</code> con todas las Categor&iacute;as de 
	 * Preguntas que el Usuario ha definido dentro de su Bando de Preguntas.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de trabajar con la 
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador de profesor proporcionado no corresponde 
	 * con el de ninguno de los Profesores que est&aacute;n registrados en el sistema.
	 */
	@Override
	public List<TOQuestionCategory> listCategories(int idTeacher) throws DataStoreException, InvalidIdException
	{
		// Comprobamos que quien ha llamdo al mtodo es un Profesor.
		if (isTeacher(idTeacher))
		{
			// Abrimos una sesin de Hibernate.
			Session session = this.sessionFactory.openSession();
			try
			{
				// Obtenemos la lista de todas las categoras de preguntas del profesor.
				List<Object> list_obj_categories = getAll(session, Table.QUESTION_CATEGORIES, "user", idTeacher);
				// Definimos la lista de a retornar.
				List<TOQuestionCategory> list_to_categories = new ArrayList<TOQuestionCategory>();
				// Recorremos la lista de objeto grabados en la capa de persistencia.
				for (Object objQuestionCategory : list_obj_categories)
				{
					// Comprobamos que objQuestionCategory sea de tipo BOQuestionCategory.
					if (objQuestionCategory instanceof BOQuestionCategory)
					{
						// Guardamos en list_to_categories el objeto equivalente a objQuestionCategory.
						list_to_categories.add(((BOQuestionCategory) objQuestionCategory).getTOQuestionCategory());
					}
				}
				// Ordenamos la lista con los resultados.
				Collections.sort(list_to_categories);
				// Retornamos la lista con las categoras de preguntas.
				return list_to_categories;
			}
			finally
			{
				// Cerramos la sesin de Hibernate.
				closeSession(session);
			}
		}
		// Si llegamos a este punto querr decir que quien ha llamado al mtodo no 
		// es un Profesor as que lanzamos una InvalidIdException.
		throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un profesor cuando quiera modificar una 
	 * la informaci&oacute;n de una de sus Categor&iacute;as de Preguntas.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idCategory <code>int</code> identificador de la Categor&iacute;a de Preguntas.
	 * @param description <code>String</code> la nueva descripci&oacutge;n de la Categor&iacute;a.
	 * @return <code>TOQuestionCategory</code> con toda la informaci&oacute;n de la Categor&iacute;a
	 * una vez se ha modificado.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de trabajar con la 
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador del Profesor no corresponde con el de un
	 * Usuario de Profesor registrado o si el identificador de la Categor&iacute;a no corresponde
	 * con el de una Categor&iacute;a de preguntas dentro del Banco de Preguntas del Profesor.
	 */
	@Override
	public TOQuestionCategory modifyCategory(int idTeacher, int idCategory, 
			String description)	throws DataStoreException, InvalidIdException
	{
		// Comprobamos que quien ha llamado al mtodo es un Profesor.
		if (!isTeacher(idTeacher))
		{
			// Al no se un Profesor lanzamos una excepcin.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
		}
		// Llegados a este punto sabemos que tenemos a un Profesor. Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Vamos a comprobar que la categora de preguntas que se quiere eliminar es
			// del usuario que ha llamado al mtodo. Sentencia HQL.
			Query query = session.createQuery("from BOQuestionCategory where id=? and user=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, idCategory, Hibernate.INTEGER);
			query.setParameter(1, idTeacher, Hibernate.INTEGER);
			// Ejecutamos la consulta y vemos el nmero de resultados devueltos.
			if ((query.list().size() != 1) || (!(query.list().get(0) instanceof BOQuestionCategory)))
			{
				// El identificador de la categora no corresponde con el de una categora
				// de preguntas definida por el Profesor. Lanzamos una excepcin.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_QUESTION_CATEGORY_ID));
			}
			else
			{
				// Obtenemos el objeto BOQuestionCategory que representa a la Categora
				BOQuestionCategory category = (BOQuestionCategory) query.list().get(0);
				// Establecemos el valor de la nueva descripcin.
				category.setDescription(WebUtils.stringToHTML(description));
				// Actulizamos la informacin dentro de la capa de persistencia.
				session.saveOrUpdate(category);
				// Consolidamos la operacin.
				session.flush();
				// Retornamos el objeto equivalente con la informacin de la categora.
				return category.getTOQuestionCategory();
			}
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado para modificar la informaci&oacute;n de una
	 * Pregunta del Profesor cuyo identificador se le pasa como par&aacute;metro.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idQuestion <code>int</code> identificador de la Pregunta.
	 * @param question <code>TOQuestion</code> con la informaci&oacute;n de la pregunta.
	 * @return <code>TOQuestion</code> con la informaci&aocute;n de la Pregunta moficada.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de trabajar con la 
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador del Profesor no corresponde con el de un
	 * Usuario de Profesor registrado o si el identificador de la Pregunta no corresponde
	 * con el de una de las Preguntas que el Profesor tiene creadas en el sistema.
	 * @throws BusinessRulesViolationException si no se han especificado al menos dos opciones
	 * de respuesta para una pregunta de tipo "Opci&oacute;n M&uacute;ltiple".
	 */
	@Override
	public TOQuestion modifyQuestion(int idTeacher, int idQuestion, TOQuestion question)
			throws DataStoreException, InvalidIdException, BusinessRulesViolationException
	{
		// Comprobamos que quien ha llamado al mtodo es un Profesor.
		if (!isTeacher(idTeacher))
		{
			// Al no se un Profesor lanzamos una excepcin.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
		}
		// Llegados a este punto sabemos que tenemos a un Profesor. Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos si existe la Pregunta dentro del sistema.
			Object objQuestion = existFieldValue(session, Table.QUESTIONS, "id", idQuestion);
			// Comprobamos que el objeto no es nulo y que es de tipo BOQuestion.
			if ((objQuestion == null) || (!(objQuestion instanceof BOQuestion)))
			{
				// Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_QUESTION_ID));
			}
			// Cast de objQuestion.
			BOQuestion boQuestion = (BOQuestion) objQuestion;
			// Vemos si la pregunta es del profesor. 
			if (boQuestion.getCategory().getUser().getId() == idTeacher)
			{
				// La pregunta es del profesor. Realizamos la modificacin en funcin
				// del tipo de pregunta. Vemos si es de tipo true/false 
				if ((question instanceof TOQuestionTrueOrFalse) && 
						(boQuestion instanceof BOQuestionTrueOrFalse))
				{
					// Cast de BOQuestion.
					BOQuestionTrueOrFalse boTF = (BOQuestionTrueOrFalse) boQuestion;
					// Cast de TOQuestion.
					TOQuestionTrueOrFalse toTF = (TOQuestionTrueOrFalse) question;
					// Establecemos los nuevos valores de los atributos.
					boTF.setStatement(WebUtils.stringToHTML(toTF.getStatement()));
					boTF.setValue(toTF.isValue());
					boTF.setTxtTrue(WebUtils.stringToHTML(toTF.getTxtTrue()));
					boTF.setTxtFalse(WebUtils.stringToHTML(toTF.getTxtFalse()));
					// Actualizamos la informacin.
					session.saveOrUpdate(boTF);
					// Consolidamos la operacin.
					session.flush();
				}
				else if ((question instanceof TOQuestionNumeric) && 
						(boQuestion instanceof BOQuestionNumeric))
				{
					// Cast de BOQuestion.
					BOQuestionNumeric boNU = (BOQuestionNumeric) boQuestion;
					// Cast de TOQuestion.
					TOQuestionNumeric toNU = (TOQuestionNumeric) question;
					// Establecemos los nuevos valores de los atributos.
					boNU.setStatement(WebUtils.stringToHTML(toNU.getStatement()));
					boNU.setValue(toNU.getValue());
					boNU.setError(toNU.getError());
					boNU.setText(WebUtils.stringToHTML(toNU.getText()));
					// Actualizamos la informacin.
					session.saveOrUpdate(boNU);
					// Consolidamos la operacin.
					session.flush();
				}
				else if ((question instanceof TOQuestionShortAnswer) && 
						(boQuestion instanceof BOQuestionShortAnswer))
				{
					// Cast de BOQuestion
					BOQuestionShortAnswer boSA = (BOQuestionShortAnswer) boQuestion;
					// cast de TOQuestion
					TOQuestionShortAnswer toSA = (TOQuestionShortAnswer) question;
					// Establecemos los valores de los nuevos atributos.
					boSA.setStatement(WebUtils.stringToHTML(toSA.getStatement()));
					boSA.setAnswer(WebUtils.stringToHTML(toSA.getAnswer()));
					boSA.setCaseSensitive(toSA.isCaseSensitive());
					boSA.setText(WebUtils.stringToHTML(toSA.getText()));
					// Actualizamos la informacin
					session.saveOrUpdate(boSA);
					// Consolidamos la operacin.
					session.flush();
				}
				else if ((question instanceof TOQuestionMultipleChoices) && 
						(boQuestion instanceof BOQuestionMultipleChoices))
				{
					// Cast de BOQuestion
					BOQuestionMultipleChoices boMC = (BOQuestionMultipleChoices) boQuestion;
					// Cast de TOQuestion
					TOQuestionMultipleChoices toMC = (TOQuestionMultipleChoices) question;
					// Comprobamos que hay al menos dos opciones de respuesta.
					if ((toMC.getAnswers() != null) && (toMC.getAnswers().size() >= 2))
					{
						// Establecemos los nuevos valores de los atributos.
						boMC.setStatement(WebUtils.stringToHTML(toMC.getStatement()));
						boMC.setCorrect(toMC.getAnswers().size());
						// Actulizamos la informacin de las pregunta.
						session.saveOrUpdate(boMC);
						// Consolidamos la operacin.
						session.flush();
						// Ahora vamos a borrar todas las respuestas asociadas a la pregunta.
						for (BOQuestionMultipleChoicesAnswer answer : boMC.getAnswers())
						{
							// Eliminamos la respuesta
							session.delete(answer);
							// Consolidamos la operacin.
							session.flush();
						}
						// Recorremos la lista de respuestas para actualizar la lista de respuestas.
						for (TOQuestionMultipleChoicesAnswer toAnswer : toMC.getAnswers())
						{
							// Creamos un objeto BOQuestionMultipleChoicesAnswer a partir de toAnswer.
							BOQuestionMultipleChoicesAnswer boAnswer = 
									new BOQuestionMultipleChoicesAnswer(boMC, toAnswer);
							// Guardamos la informacin de la opcin de respuesta.
							session.saveOrUpdate(boAnswer);
							// Consolidamos la operacin.
							session.flush();
						}
					}
					else
					{
						// No hay opciones de respuesta. Lanzamos una BusinessRulesViolationException;
						throw (new BusinessRulesViolationException(BusinessRulesViolationException.MSG_NO_ANSWERS_EXPECIFIED));
					}
				}
				else if ((question instanceof TOQuestionMatch) && 
						(boQuestion instanceof BOQuestionMatch))
				{
					// Cast de BOQuestion
					BOQuestionMatch boMA = (BOQuestionMatch) boQuestion;
					// Cast de TOQuestion
					TOQuestionMatch toMA = (TOQuestionMatch) question;
					// Comprobamos que hay al menos dos opciones de respuesta.
					if ((toMA.getAnswers() != null) && (toMA.getAnswers().size() >= 2))
					{
						// Establecemos los nuevos valores de los atributos.
						boMA.setStatement(WebUtils.stringToHTML(toMA.getStatement()));
						boMA.setOptions(toMA.getAnswers().size());
						// Actulizamos la informacin de la pregunta.
						session.saveOrUpdate(boMA);
						// Consolidamos la operacin.
						session.flush();
						// Ahora vamos a borrar todas las respuesta asociadas a la pregunta.
						for (BOQuestionMatchAnswer answer : boMA.getAnswers())
						{
							// Eliminamos la respuesta.
							session.delete(answer);
							// Consolidamos la operacin.
							session.flush();
						}
						// Recorremos la lista de "parejas" y actulizamos las respuestas a la pregunta.
						for (TOQuestionMatchAnswer toAnswer : toMA.getAnswers())
						{
							// Creamos el objeto BOQuestionMatchAnswer a partir de "toAnswer".
							BOQuestionMatchAnswer boAnswer = new BOQuestionMatchAnswer(boMA, toAnswer);
							// Guardamos la opcin de respuesta.
							session.saveOrUpdate(boAnswer);
							// Consolidamos la operacin.
							session.flush();
						}
					}
					else
					{
						// No hay opciones de respuesta. Lanzamos una BusinessRulesViolationException;
						throw (new BusinessRulesViolationException(BusinessRulesViolationException.MSG_NO_ANSWERS_EXPECIFIED));
					}
				}
				// Retornamos el objeto TOQuestion equivalente.
				return boQuestion.getTOQuestion();
			}
			else
			{
				// La pregunta no es del Profesor. Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_QUESTION_ID));
			}
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
}
