package edu.uoc.pfc.business.services;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.hibernate.Hibernate;
import org.hibernate.Query;
import org.hibernate.Session;

import edu.uoc.pfc.business.exceptions.DataStoreException;
import edu.uoc.pfc.business.exceptions.DuplicatedFieldException;
import edu.uoc.pfc.business.exceptions.InvalidIdException;
import edu.uoc.pfc.business.interfaces.IBSubjectManager;
import edu.uoc.pfc.business.objects.BOClassroom;
import edu.uoc.pfc.business.objects.BOCurriculum;
import edu.uoc.pfc.business.objects.BOSubject;
import edu.uoc.pfc.business.objects.BOSubjectCurriculum;
import edu.uoc.pfc.business.transfers.TOCurriculum;
import edu.uoc.pfc.business.transfers.TOSubject;
import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Clase que implementa la interfaz <code>IBSubjectManager</code> encargada
 * de ofrecer los m&eacute;todos que la capa de negocio ofrece a la capa de 
 * presentaci&oacute;n relacionados con la gesti&oacute;n de las asignaturas.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0 
 */
public class IBSubjectManagerImpl extends IBBaseImpl implements IBSubjectManager
{
	
	// -------- CONSTRUCTORES.
	/**
	 * <p><code>Construye un nuevo objeto <code>IBSubjectManagerImpl</code> e inicializa el 
	 * marco de persistencia <i>Hibernate</i>.</p>
	 * 
	 * @throws DataStoreException si se produce una excepci&oacute;n a la hora de iniciar
	 * el marco de persistencia.
	 */
	public IBSubjectManagerImpl() throws DataStoreException
	{
		// Constructor de la superclase.
		super();
	}
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un administrador del sistema cuando quiera asignar 
	 * una asignatura a un plan de estudios.</p>
	 * 
	 * @param idSubject <code>int</code> identificador de la Asignatura.
	 * @param idCurriculum <code>int</code> identificador del Plan de Estudios.
	 * @param subjectType <code>String</code> tipo de Asignatura dentro del Plan de Estudios. 
	 * @param code <code>String</code> c&oacute;digo de la Asignatura.
	 * @param credits <code>double</code> n&uacute;mero de los cr&eeacute;ditos de la Asignatura.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de excepci&oacute;n a
	 * la hora de trabajar con la capa de persistencia.
	 * @throws InvalidIdException si el identificador del Plan de Estudios o de la Asignatura que se 
	 * le ha pasado es inv&aacute;lido.
	 * @throws DuplicatedFieldException si la asignaci&oacute;n de la Asignatura al Plan de Estudios
	 * ya haba sido realizada anteriormente.
	 */
	@Override
	public void addCurriculum(int idSubject, int idCurriculum, String subjectType, String code, 
			double credits) throws DataStoreException, InvalidIdException, DuplicatedFieldException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos que existe el Plan de Estudios.
			Object objectCurriculum = existFieldValue(session, Table.CURRICULUM, "id", idCurriculum);
			// Vemos que no es nulo y de tipo BOCurriculum
			if ((objectCurriculum == null) || (!(objectCurriculum instanceof BOCurriculum)))
			{
				// Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_CURRICULUM_ID));
			}
			// Comprobamos la existencia de la Asignatura.
			Object objectSubject = existFieldValue(session, Table.SUBJECTS, "id", idSubject);
			// Vemos que no es nulo y adems es de tipo BOSubject
			if ((objectSubject == null) || (!(objectSubject instanceof BOSubject)))
			{
				// Lanzamos una InvalidIdException
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_SUBJECT_ID));
			}
			// Vemos si previamente se haba realizado la asignacin. Creamos la sentencia HQL.
			Query query = session.createQuery("from BOSubjectCurriculum where subject=? and curriculum=?");
			// Establecemos los parmetros.
			query.setParameter(0, idSubject, Hibernate.INTEGER);
			query.setParameter(1, idSubject, Hibernate.INTEGER);
			// Ejecutamos la sentencia y vemos el nmero de resultados.
			if (query.list().size() == 0)
			{
				// No existe el registro dentro de la capa de persistencia.
				// Cast del objeto BOCurriculum
				BOCurriculum boCurriculum = (BOCurriculum) objectCurriculum;
				// Cast del objeto BOSubject
				BOSubject boSubject = (BOSubject) objectSubject;
				// Creamos un objeto BOSubjectCurriculum
				BOSubjectCurriculum boSubjectCurriculum = new BOSubjectCurriculum(boSubject, 
						boCurriculum, code, credits, subjectType);
				// Guardamos el objeto en la capa de persistencia.
				session.saveOrUpdate(boSubjectCurriculum);
				// Consolidamos la operacin.
				session.flush();
			}
			else
			{
				// Ya exista un registro con esta informacin as que lanzamos una DuplicatedFieldException.
				throw (new DuplicatedFieldException(DuplicatedFieldException.MSG_SUBJECT_CURRICULUM_DUPLICATED));
			}
		}
		finally 
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
		
	/**
	 * <p>Crea una nueva asignatura dentro de la base de datos a partir de la informaci&oacute;n
	 * que se le pasa como pa&aacute;metro. En el caso de que ya exista una asignatura con el 
	 * mismo nombre (<i>name</i>) este m&eacute;todo lanzar&aacute; una <code>DuplicatedFieldException
	 * </code>. Si el proceso de creaci&oacute;n de la asignatura finaliza correctamente la 
	 * informaci&oacute; de la nueva asignatura ser&aacute; retornada dentro de un objeto de 
	 * tipo <code>TOSubject</code>.</p>
	 *  
	 * @param name <code>String</code> nombre de la asignatura.
	 * @param description <code>String</code> descripci&oacute;n de la asignatura.
	 * @return <code>TOSubject</code> con la informaci&oacute;n de la nueva asignatura.
	 * @throws DataStoreException si se produce al&uacute;n tipo de error dentro del
	 * marco de persistencia.
	 * @throws DuplicatedFieldException si ya existe una asignatura con el mismo nombre registrada
	 * dentro de la aplicaci&oacute;n.
	 */
	@Override
	public TOSubject createSubject(String name, String description)
			throws DataStoreException, DuplicatedFieldException
	{
		// Abrimos la sessin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos si ya existe una asignatura con el mismo nombre.
			Object oSubject = existFieldValue(session, Table.SUBJECTS, "name", name);
			// Comprobamos si existe el objecto y si es de tipo BOSubject
			if ((oSubject != null) && (oSubject instanceof BOSubject))
			{
				// Ya existe la asignatura. Lanzamos un DuplicatedFieldException.
				throw (new DuplicatedFieldException(DuplicatedFieldException.MSG_SUBJECT_NAME_DUPLICATED));
			}
			// Si llegamos a este punto habr que crear el objeto BOSubject a partir 
			// de la informacin recibida como parmetro.
			BOSubject subject = new BOSubject(name, description);
			// Guardamos la informacin en la capa de persistencia.
			session.saveOrUpdate(subject);
			// Consolidamos la operacin.
			session.flush();
			// Retornamos el objeto TOSubject equivalente.
			return subject.getTOSubject();
		}
		finally 
		{
			// Intentamos cerrar la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>Elimina del sistema la asignatura cuyo identificador se le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador de la asignatura.
	 * @throws DataStoreException si se produce al&uacute;n tipo de error dentro del
	 * marco de persistencia.
	 * @throws InvalidIdException si el identificador de la asignatura que se ha pasado no
	 * coincide con el de ninguna de las asignaturas que est&aacute;n registradas en el sistema.
	 */
	@Override
	public void deleteSubject(int id) throws DataStoreException, InvalidIdException
	{
		// Creamos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{	
			// Vemos si sexiste la asignatura.
			Object oSubject = existFieldValue(session, Table.SUBJECTS, "id", id);
			// Comprobamos si existe la asignatura y si es un objeto BOSubject
			if ((oSubject != null) && (oSubject instanceof BOSubject))
			{
				// Cast oSubject.
				BOSubject subject = (BOSubject) oSubject;				
				// Obtenemos la lista de Planes de Estudios en los que est asignada la Asignatura.
				List<Object> list_curriculum = getAll(session, Table.SUBJECTS_CURRICULUM, "subject", id);
				// Recorremos la lista retornada.
				for (Object obj : list_curriculum)
				{
					// Comprobamos que "obj" es de tipo BOSubjectCurriculum
					if (obj instanceof BOSubjectCurriculum)
					{
						// Cast del objeto y lo eliminamos de la sesin.
						session.delete(((BOSubjectCurriculum) obj));
						// Consolidamos la operacin
						session.flush();
					}
				}
				// Obtenemos la lista de Aulas donde se imparte la asignatura.
				List<Object> list_classroom = getAll(session, Table.CLASSROOM, "subject", id);
				// Recorremos la lista de Aula.
				for (Object obj : list_classroom)
				{
					// Comprobamos que el elemento es de tipo BOClassroom.
					if (obj instanceof BOClassroom)
					{
						// Eliminamos el objeto.
						session.delete(((BOClassroom) obj));
						// Consolidamos la operacin.
						session.flush();
					}
				}
				// Eliminamos la asignatura.
				session.delete(subject);
				// Consolidamos la operacin.
				session.flush();
			}
		}
		finally 
		{
			// Intentamos cerrar la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>Obtiene la informaci&oacute;n de la asignatura cuyo identificador se le pasa como
	 * par&aacute;metro y la devuelve encapsulada dentro del un objeto <code>TOSubject</code>.</p>
	 * 
	 * @param id <code>int</code> identificador de la asignatura.
	 * @return <code>TOSubject</code> con la informaci&oacute;n de la asignatura.
	 * @throws DataStoreException si se produce al&uacute;n tipo de error dentro del
	 * marco de persistencia.
	 * @throws InvalidIdException si el identificador que se ha pasado no corresponde con el
	 * de ninguna de las asignaturas registradas dentro de la aplicaci&oacute;n.
	 */
	@Override
	public TOSubject getSubject(int id) throws DataStoreException, InvalidIdException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos la existencia de la asignatura.
			Object oSubject = existFieldValue(session, Table.SUBJECTS, "id", id);
			// Vemos que no es nulo y que es de tipo BOSubject.
			if ((oSubject != null) && (oSubject instanceof BOSubject))
			{
				// Cast al objeto BOSubject.
				BOSubject subject = (BOSubject) oSubject;
				// Retornamos el objeto TOSubject equivalente.
				return subject.getTOSubjectComplete();
			}
			// Si llegamos a este punto querr decir que no existe una asignatura 
			// cuyo identificador coincide con el que se ha recibido como parmetro
			// as que lanzamos una InvalidIdException.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_SUBJECT_ID));
		}
		finally 
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>Obtiene una lista de objetos <code>TOCurriculum</code> donde cada uno de ellos recoge
	 * la informaci&oacute;n de un Plan de Estudios que no ha sido asignado a la Asignatura cuyo
	 * identificador se le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador de la Asignatura.
	 * @return <code>List&lt;TOCurriculum&gt;</code> donde cada elemento representa a un Plan
	 * de Estudios al que no ha sido asignado la asignatura.
	 * @throws DataStoreException si se produce al&uacute;n tipo de error dentro del
	 * marco de persistencia.
	 * @throws InvalidIdException si el identificador que se ha pasado no corresponde con el
	 * de ninguna de las asignaturas registradas dentro de la aplicaci&oacute;n.
	 */
	public List<TOCurriculum> getUnassignedCurriculum(int id)
			throws DataStoreException, InvalidIdException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos que si existe la asignatura.
			Object objectSubject = existFieldValue(session, Table.SUBJECTS, "id", id);
			// Vemos que no es nulo y de tipo BOSubject
			if ((objectSubject != null) && (objectSubject instanceof BOSubject))
			{
				// Cast del "object".
				BOSubject subject = (BOSubject) objectSubject;
				// Obtenemos la lista de todos los Planes de Estudio registrados.
				List<Object> list_curriculum = getAll(session, Table.CURRICULUM);
				// Creamos un Set donde almacenaremos todos los Planes de Estudio.
				Set<BOCurriculum> set_curriculum = new HashSet<BOCurriculum>();
				// Recorremos la "list_curriculum"
				for (Object obj : list_curriculum)
				{
					// Comprobamos que "obj" es de tipo BOCurriculum
					if (obj instanceof BOCurriculum)
					{
						// Cast de "obj" y lo aadimos al conjunto.
						set_curriculum.add((BOCurriculum) obj);
					}
				}
				// Recorremos el conjunto de Planes de Estudio que ya tienen a al Asignatura.
				for (BOSubjectCurriculum bo : subject.getCurriculum())
				{
					// Eliminamos el Plan de Estudios de "set_curriculum"
					set_curriculum.remove(bo.getCurriculum());
				}
				// Creamos la lista de Planes de Estudio a retornar.
				List<TOCurriculum> list_valid_curriculum = new ArrayList<TOCurriculum>();
				// Recorremos todos los elementos del conjunto.
				for (BOCurriculum bo : set_curriculum)
				{
					// Aadimos "bo" a la lista a retornar.
					list_valid_curriculum.add(bo.getTOCurriculum());
				}
				// Ordenamos la lista de resultados.
				Collections.sort(list_valid_curriculum);
				// Retornamos la lista.
				return list_valid_curriculum;
			}
			else
			{
				// Si llegamos a este punto querr decir que no existe la asignatura
				// con el "id" que se le ha pasado. Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_SUBJECT_ID));
			}
		}
		finally 
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>Obtiene una lista de objetos <code>TOSubject</code> cada uno de los cuales
	 * contiene la informaci&oacute;n de una de las asignaturas que est&aacute;n 
	 * registradas en el sistema.</p>
	 * 
	 * @return List&lt;TOSubject&gt; con todas las asignaturas registradas.
	 * @throws DataStoreException si se produce al&uacute;n tipo de error dentro del
	 * marco de persistencia.
	 */
	public List<TOSubject> listSubject() throws DataStoreException
	{
		// Abrimos la sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Creamos la lista de objetos TOSubject que contendrn las asignaturas.
			List<TOSubject> listSubjects = new ArrayList<TOSubject>();
			// Obtenemos la lista de todos los objetos que forman la tabla.
			List<Object> listObjects = getAll(session, Table.SUBJECTS);
			// Recorremos la lista de objetos retornados.
			for (Object oObject : listObjects)
			{
				// Comprobamos que el objeto es de tipo BOSubject.
				if (oObject instanceof BOSubject)
				{
					// Cast al objeto.
					BOSubject boSubject = (BOSubject) oObject;
					// Aadimos el objeto TOSubject equivalente a la lista.
					listSubjects.add(boSubject.getTOSubject());
				}
			}
			// Ordenamos la lista de resultados obtenidos.
			Collections.sort(listSubjects);
			// Retornamos la lista de asignaturas.
			return listSubjects;
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado por un administrador del sistema cada vez
	 * que quiera modificar la informaci&oacute;n de la Asignatura cuyo identificador se
	 * le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador de a Asignatura.
	 * @param description <code>String</code> nueva descripci&oacute; para la asignatura.
	 * @return <code>TOSubject</code> con la informaci&oacute;n de la Asignatura ya modificada.
	 * @throws DataStoreException  si se produce al&uacute;n tipo de error dentro del
	 * marco de persistencia.
	 * @throws InvalidIdException si el identificador que se ha pasado no corresponde con el
	 * de ninguna de las asignaturas registradas dentro de la aplicaci&oacute;n.
	 */
	public TOSubject modifySubject(int id, String description) 
			throws DataStoreException, InvalidIdException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Vamos a ver si existe la asignatura que se quiere modificar.
			Object object = existFieldValue(session, Table.SUBJECTS, "id", id);
			// Comprobamos que no es null y es de tipo BOSubject.
			if ((object != null) && (object instanceof BOSubject))
			{
				// Cast a "BOSubject"
				BOSubject subject = (BOSubject) object;
				// Establecemos la nueva descripcin.
				subject.setDescription(WebUtils.stringToHTML(description));
				// Actualizamos la informacin.
				session.saveOrUpdate(subject);
				// Consolidamos la operacin.
				session.flush();
				// Retornamos el objeto TOSubject equivalente.
				return subject.getTOSubject();
			}
			else
			{
				// Si llegamos a este punto querr decir que no existe la 
				// asignatura por lo que lanzaremos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_SUBJECT_ID));
			}
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
		
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado en el momento en el que se quiera desasignar
	 * una Asignatura (identificada por "idSubject") de un Plan de Estudios (identificado 
	 * por "idCurriculum").</p>
	 * 
	 * @param idSubject <code>int</code> Identificador de la Asignatura.
	 * @param idCurriculum <code>int</code> Identificador del Plan de Estudios.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de excepci&oacute;n a
	 * la hora de trabajar con la capa de persistencia.
	 * @throws InvalidIdException si no se ha realizado previamente la asignaci&oacute;n
	 * de la asignatura al Plan de Estudios previamente.
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void removeCurriculum(int idSubject, int idCurriculum) 
			throws DataStoreException, InvalidIdException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Preparamos la sentencia HQL para comprobar que se ha realiza previamente la Asignatura al
			// Plan de Estudios.
			Query query = session.createQuery("from BOSubjectCurriculum where curriculum=? and subject=?");
			// Asignamos los parmetros.
			query.setParameter(0, idCurriculum, Hibernate.INTEGER);
			query.setParameter(1, idSubject, Hibernate.INTEGER);
			// Ejecutamos la sentencia.
			List<BOSubjectCurriculum> listBOSubjectCurriculum = query.list();
			// Comprobamos el nmero de registros retornados.
			if (listBOSubjectCurriculum.size() == 1)
			{
				// Borramos el registro.
				session.delete(listBOSubjectCurriculum.get(0));
				// Consolidamos la operacin.
				session.flush();
			}
			else
			{
				// No se haba realizado la asignacin as que lanzamos una excepcin.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_SUBJECT_ASSIGN_CURRICULUM));
			}
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
}
