package edu.uoc.pfc.business.services;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.hibernate.Hibernate;
import org.hibernate.Query;
import org.hibernate.Session;

import edu.uoc.pfc.business.exceptions.DataStoreException;
import edu.uoc.pfc.business.exceptions.DuplicatedFieldException;
import edu.uoc.pfc.business.exceptions.InvalidIdException;
import edu.uoc.pfc.business.interfaces.IBTestManager;
import edu.uoc.pfc.business.objects.BOClassroomUser;
import edu.uoc.pfc.business.objects.BOQuestion;
import edu.uoc.pfc.business.objects.BOTest;
import edu.uoc.pfc.business.objects.BOTestAnswer;
import edu.uoc.pfc.business.objects.BOTestClassroom;
import edu.uoc.pfc.business.objects.BOTestQuestion;
import edu.uoc.pfc.business.objects.BOUser;
import edu.uoc.pfc.business.transfers.TOTest;
import edu.uoc.pfc.business.transfers.TOTestAnswer;
import edu.uoc.pfc.business.transfers.TOTestAnswerStats;
import edu.uoc.pfc.business.transfers.TOTestStats;
import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Implementaci&oacute;n de la interfaz <code>IBTestManager</code>. </p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0 
 */
public class IBTestManagerImpl extends IBBaseImpl implements IBTestManager 
{
	
	// -------- CONSTRUCTORES.
	/**
	 * <p><code>Construye un nuevo objeto <code>IBTestManagerImpl</code> e inicializa el 
	 * marco de persistencia <i>Hibernate</i>.</p>
	 * 
	 * @throws DataStoreException si se produce una excepci&oacute;n a la hora de iniciar
	 * el marco de persistencia.
	 */
	public IBTestManagerImpl() throws DataStoreException
	{
		// Constructor de la superclase.
		super();
	}
	
	// -------- MTOODOS PRIVADOS.
	/**
	 * <p>Comprueba si el identificador de usuario que recibe corresponde con el de 
	 * un usuario definido como un Profesor dentro de la aplicaci&oacute;n.</p>
	 * 
	 * @param idTeacher <code>int</code> el identificador del Usuario a comprobar.
	 * @return <code>true</code> si el identificador del usaurio corresponde con el de
	 * un Profesor o <code>false</code> en caso contrario.
	 * @thows DataStoreException si se produce alg&uacute;n error a la hora de trabajar
	 * con la capa de persistencia.
	 * @throws InvalidIdException siempre que el identificador recibido no corresponde
	 * con el de ninguno de los Usuarios que est&aacute;n registrados dentro de la 
	 * aplicaci&oacute;n.
	 */
	private boolean isTeacher(int idUser) throws DataStoreException, InvalidIdException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos si existe un Usuario con el identificador "idUser".
			Object objUser = existFieldValue(session, Table.USERS, "id", idUser);
			// Comprobamos que objUser no sea null y sea de tipo BOUser.
			if ((objUser != null) && (objUser instanceof BOUser))
			{
				// Retornamos valor del atributo "teacher" del objeto.
				return ((BOUser) objUser).isTeacher();
			}
			// Si llegamos a este punto querr decir que no existe el usuario por
			// lo que lanzamos una InvalidIdException.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_USER_ID));
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera asignar una de las 
	 * Preguntas de su Banco de Preguntas a uno de sus Cuestionarios. En el caso de que la Pregunta
	 * ya haya sido asignada previamente al Cuestionario este m&eacute;todo actualizar&aacute; la 
	 * informaci&oacute;n.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idTest <code>int</code> identificador del Cuestionario.
	 * @param idQuestion <code>int</code> identificador de la Pregunta.
	 * @param value <code>double</code> valor de la Pregunta en el Cuestionario.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con
	 * la capa de persistencia.
	 * @throws InvalidIdException si el identificador del Profesor no se corresponde con el de un Profesor
	 * registrado en el sistema; si el identificador del Cuestionario no se corresponde con  el de un 
	 * Cuestionario del Profesor; o si el identificador de la Pregunta no se corresponde con el de una
	 * de las Preguntas dentro del Banco de Preguntas del Profesor.
	 */
	@Override
	public void addQuestion(int idTeacher, int idTest, int idQuestion, double value)
			throws DataStoreException, InvalidIdException
	{
		// Comprobamos que quien ha llamado al mtodo es un Profesor.
		if (!isTeacher(idTeacher))
		{
			// Al no se un Profesor lanzamos una excepcin.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
		}
		// El identificador corresponde al de un Profesor. Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Vemos si el Test es del usuario. Sentencia HQL
			Query query = session.createQuery("from BOTest where id=? and teacher=?");
			// Asignamos los valores de los parmetros.
			query.setParameter(0, idTest, Hibernate.INTEGER);
			query.setParameter(1, idTeacher, Hibernate.INTEGER);
			// Ejecutamos la consulta.
			if ((query.list().size() != 1) || (!(query.list().get(0) instanceof BOTest)))
			{
				// El Test no pertenece al usuario. Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEST_ID));
			}
			// Realizamos un cast del resultado obtenido.
			BOTest boTest = (BOTest) query.list().get(0);
			// Ahora tenemos que ver que la Pregunta es del Profesor. Sentencia HQL.
			query = session.createQuery("from BOQuestion where id=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, idQuestion, Hibernate.INTEGER);
			// Ejecutamos la consulta.
			if ((query.list().size() != 1) || (!(query.list().get(0) instanceof BOQuestion)))
			{
				// No existe la pregunta. Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_QUESTION_ID));
			}
			// Realizamos un cast del resultado obtenido.
			BOQuestion boQuestion = (BOQuestion) query.list().get(0);
			// Comprobamos que la categora de preguntas a la que pertenece la pregunta es del Profesor.
			query = session.createQuery("from BOQuestionCategory where id=? and user=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, boQuestion.getCategory().getId(), Hibernate.INTEGER);
			query.setParameter(1, idTeacher, Hibernate.INTEGER);
			// Ejecutamos la consulta.
			if (query.list().size() != 1)
			{
				// La pregunta pertenece a una categora que no es del usuario. Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_QUESTION_ID));
			}
			// Vamos a comprobar si tenemos que actualizar la informacin o crear una asignacin nueva.
			query = session.createQuery("from BOTestQuestion where test=? and question=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, boTest.getId(), Hibernate.INTEGER);
			query.setParameter(1, boQuestion.getId(), Hibernate.INTEGER);
			// Creamos una referencia de tipo BOTestQuestion.
			BOTestQuestion testQuestion = null;
			// Comprobamos si se trata de una actualizacin o de una nueva asignacin
			if ((query.list().size() == 0) || (!(query.list().get(0) instanceof BOTestQuestion)))
			{
				// Se trata de una nueva asignacin.
				testQuestion = new BOTestQuestion(boTest, boQuestion, value);
			}
			else 
			{
				// Cast del resultado obtenido.
				testQuestion = (BOTestQuestion) query.list().get(0);
				// Asignamos el valor de "value".
				testQuestion.setValue(value);
			}
			// Actulizamos o modificamos los datos.
			session.saveOrUpdate(testQuestion);
			// Consolidamos la operacin.
			session.flush();
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor del sistema cada vez que quiera crear
	 * un nuevo Cuestionario.</p>
	 *
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param title <code>String</code> t&iacute;tulo del Cuestionario.
	 * @param introduction <code>String</code> texto de introducci&oacute;n.
	 * @param shuffleQuestions <code>boolean</code> indica si se barajar&aacute;n las Preguntas.
	 * @param shuffleAnswers <code>boolean</code> indica si se barajar&aacute;n las Respuestas.
	 * @param showSolution <code>boolean</code> indica si se muestra la soluci&oacute;n.
	 * @param showReview <code>indica si se muestra la revisi&oacute;n</code>
	 * @return <code>TOTest</code> con toda la informaci&oacute;n de Cuestionario que se acaba
	 * de crear en el sistema.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con
	 * la capa de persistencia.
	 * @throws DuplicatedFieldException si el Profesor ya tiene registrado en el sistema un Cuestionario
	 * cuyo t&iacute;tulo coincide con el que se le pasa como par&aacute;metro.
	 * @throws InvalidIdException si el identificador de usuario que se le pasa no se corresponde con
	 * el de ninguno de los profesores registrados en el sistema.
	 */
	public TOTest createTest(int idTeacher, String title, String introduction, 
			boolean shuffleQuestions, boolean shuffleAnswers, boolean showSolution, 
			boolean showReview) throws DataStoreException, DuplicatedFieldException,
			InvalidIdException
	{
		// Comprobamos que quien ha llamado al mtodo es un Profesor.
		if (!isTeacher(idTeacher))
		{
			// Al no se un Profesor lanzamos una excepcin.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
		}
		// El identificador corresponde al de un Profesor. Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos que no se haya creado previamente el Cuestionario. Sentencia HQL.
			Query query = session.createQuery("from BOTest where teacher=? and title=?");
			// Establecemos los parmetros.
			query.setParameter(0, idTeacher, Hibernate.INTEGER);
			query.setParameter(1, WebUtils.stringToHTML(title), Hibernate.STRING);
			// Ejecutamos la consulta y vemos el nmero de resultados obtenidos.
			if (query.list().size() != 0)
			{
				// Lanzamos una DuplicatedFieldException.
				throw (new DuplicatedFieldException(DuplicatedFieldException.MSG_TEST_DUPLICATED));
			}
			// Si llegamos a este punto tendremos que crear guardar la informacin del Cuestionario.
			// Creamos una objeto BOTest a partir de los parmetros. En primer lugar obtenemos el 
			// objeto BOUser que representa al Profesor.
			Object objTeacher = existFieldValue(session, Table.USERS, "id", idTeacher);
			// Comprobamos que no es nulo y es de tipo BOUser.
			if ((objTeacher != null) && (objTeacher instanceof BOUser))
			{
				// Cast del usuario.
				BOUser teacher = (BOUser) objTeacher;	
				// Creamos el objeto BOTest.
				BOTest test = new BOTest(teacher, WebUtils.stringToHTML(title), WebUtils.stringToHTML(introduction), 
						shuffleQuestions, shuffleAnswers, showSolution, showReview);
				// Guardamos el objeto en la capa de persistencia.
				session.saveOrUpdate(test);
				// Consolidamos la operacin.
				session.flush();
				// Retornamos el objeto TOTest equivalente.
				return test.getTOTest();
			}
			// Si llegamos a este punto querr decir que "objTeacher" es nulo o que no es una
			// instancia de BOUser por lo que lanzamos una InvalidIdException.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
		}	
		finally 
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>Elimina del sistema el Cuestionario cuyo indentificador es "idTest" siempre y cuando este
	 * Cuestionario pertenezca al Usuario cuyo identificador es "idTeacher".</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idTest <code>int</code> identificador del Cuestionario.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con
	 * la capa de persistencia.
	 * @throws InvalidIdException si "idTeacher" no es el de un usuario Profesor o si "idTest" no 
	 * corresponde con el identificador de ninguno de los Cuestionarios del sistema que pertenecen 
	 * al Profesor.
	 */
	public void deleteTest(int idTeacher, int idTest) throws DataStoreException, InvalidIdException
	{
		// Comprobamos que quien ha llamado al mtodo es un Profesor.
		if (!isTeacher(idTeacher))
		{
			// Al no se un Profesor lanzamos una excepcin.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
		}
		// Obtenemos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos si "idTest" corresponde a un Cuestionario del Profesor. Sentencia HQL.
			Query query = session.createQuery("from BOTest where id=? and teacher=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, idTest, Hibernate.INTEGER);
			query.setParameter(1, idTeacher, Hibernate.INTEGER);
			// Ejecutamos la sentencia HQL y comprobamos el resultado.
			if ((query.list().size() == 1) && (query.list().get(0) instanceof BOTest))
			{
				// Cast del objeto Cuestionario.
				BOTest test = (BOTest) query.list().get(0);
				// Recorremos el conjunto de asociaciones de preguntas al cuestionario.
				for (BOTestQuestion question : test.getQuestions())
				{
					// Eliminamos la asociacin.
					session.delete(question);
					// Consolidamos la operacin.
					session.flush();
				}
				// Borramos el Cuestionario.
				session.delete(test);
				// Consolidamos la operacin.
				session.flush();
			}
			else
			{
				// Si se llega a este punto querr decir que el "idTest" no corresponde con el
				// de ninguno de los Cuestionarios del Profesor as que lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEST_ID));
			}
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>Obtiene los datos del Cuestionario cuyo identificador se le pasa como par&aacute;metro.</p>
	 * 
	 * @param idTest <code>int</code> identificador del Cuestionario.
	 * @param idClassroom <code>int</code> identificador del Aula.
	 * @return <code>TOTest</code> con la informaci&oacute;n del Cuestionario.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con
	 * la capa de persistencia.
	 * @throws InvalidIdException si el identificador que se le pasa no se corresponde con el
	 * de ninguno de los Cuestionarios que est&aacute;n registrados en el sistema o si el identificador
	 * del Aula no corresponde a ninguna de las Aulas registradas en el sistema o si el test no ha
	 * sido asignado al aula.
	 */
	@Override
	public TOTest getTestClassroom(int idTest, int idClassroom) throws DataStoreException, InvalidIdException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos si existe o no el Test en el sistema.
			Object objTest = existFieldValue(session, Table.TEST, "id", idTest);
			// Vemos si es nulo o no es de tipo BOTest.
			if ((objTest == null) || (!(objTest instanceof BOTest)))
			{
				// Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEST_ID));
			}
			// Comprobamos si el Test ha sido asignado al Aula. Sentencia HQL.
			Query query = session.createQuery("from BOTestClassroom where test=? and classroom=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, idTest, Hibernate.INTEGER);
			query.setParameter(1, idClassroom, Hibernate.INTEGER);
			// Ejecutamos la consulta, vemos el nmero de elementos retornados y el tipo.
			if ((query.list().size() != 1) || (!(query.list().get(0) instanceof BOTestClassroom)))
			{
				// No existe el Test. Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEST_ID));
			}
			// Hacemos un cast del resultado.
			BOTestClassroom boTestClassroom = (BOTestClassroom) query.list().get(0);
			// Cast de "objTest".
			BOTest boTest = (BOTest) objTest;
			// Creamos una referencia a TOTest equivalente a boTest.
			TOTest toTest = boTest.getTOTestComplete();
			// Le asignamos el identificador del cuestionario dentro del aula.
			toTest.setIdTestClassroom(boTestClassroom.getId());
			// Retornamos toTest.
			return toTest;
		}
		finally
		{
			// Cerramso la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo que obtiene las Estad&iacute;sticas detalladas de un Cuestionario de 
	 * un Profesor.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idTest <code>int</code> identificador del Cuestionario.
	 * @return <code>TOTestStats</code> con la informaci&oacute;n de las Estad&iacute;sticas.
	 * @throws DataStoreException  si se produce alg&uacute;n tipo de error a la hora de trabajar con
	 * la capa de persistencia.
	 * @throws InvalidIdException si "idTeacher" no es el de un usuario Profesor o si "idTest" no 
	 * corresponde con el identificador de ninguno de los Cuestionarios del sistema que pertenecen 
	 * al Profesor.
	 */
	@Override
	public TOTestStats getTestStats(int idTeacher, int idTest)
			throws DataStoreException, InvalidIdException
	{
		// Comprobamos que el identificador pasado es el de un Profesor.
		if (!isTeacher(idTeacher))
		{
			// No se trata de un Profesor as que lanzamos una InvalidIdException.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
		}
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos que "idTest" es del Profesor. Sentencia HQL.
			Query query = session.createQuery("from BOTest where id=? and teacher=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, idTest, Hibernate.INTEGER);
			query.setParameter(1, idTeacher, Hibernate.INTEGER);
			// Ejecutamos la consulta, vemos el nmero de resultados y el tipo.
			if ((query.list().size() != 1) || (!(query.list().get(0) instanceof BOTest)))
			{
				// El identificador del Test ser invlido. Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEST_ID));
			}
			// Cast a BOTEst.
			BOTest boTest = (BOTest) query.list().get(0);
			// Creamos una lista de objetos TOTestAnswerStats.
			List<TOTestAnswerStats> list_stats = new ArrayList<TOTestAnswerStats>();
			// Referencia al objeto TOTestStats que retornaremos.
			TOTestStats stats = null;
			// Comprobamos si el Test tiene definidas Preguntas o no.
			if ((boTest.getQuestions() == null) || (boTest.getQuestions().size() == 0))
			{
				// El test no tiene preguntas asignadas. Creamos el objeto TOTestStats.
				stats = new TOTestStats(boTest.getId(), boTest.getTitle(), 0, 0);
			}
			else
			{
				// Obtemeos la informacin de todos los identificadores del Test una 
				// vez asignados a las Aulas. Sentencia HQL.
				query = session.createQuery("from BOTestClassroom where test=?");
				// Establecemos los parmetros.
				query.setParameter(0, idTest, Hibernate.INTEGER);
				// Recorremos los resultados devueltos por la consulta.
				for (Object objTestClassroom : query.list())
				{
					// Comprobamos la clase de objTestClassroom.
					if (objTestClassroom instanceof BOTestClassroom)
					{
						// Cast de objTestClassroom
						BOTestClassroom boTestClassroom = (BOTestClassroom) objTestClassroom;
						// Vamos a obtener todas las respuestas a este Test. Sentencia HQL.
						Query query2 = session.createQuery("from BOTestAnswer where test_classroom=?");
						// Establecemos los valores de los parmetros.
						query2.setParameter(0, boTestClassroom.getId(), Hibernate.INTEGER);
						// Ejecutamos la consulta y recorremos los resultados.
						for (Object objTestAnswer : query2.list())
						{
							// Comprobamos el tipo de objTestAnswer.
							if (objTestAnswer instanceof BOTestAnswer)
							{
								// Cast de objTestAnswer.
								BOTestAnswer boTestAnswer = (BOTestAnswer) objTestAnswer;
								// Aadimos a "list_stats" el objeto TOTestAnswerStats equivalente.
								list_stats.add(boTestAnswer.getTOTestAnswerStats());
							}
						}
					}
				}
				// Creamos el objeto TOTestStats.
				stats = new TOTestStats(boTest.getId(), boTest.getTitle(), 
						boTest.getQuestions().size(), list_stats.size());
			}
			// Asignamos la lista de estadsticas a TOTestStats.
			stats.setDetails(list_stats);
			// Retornamos el objeto TOTestStats
			return stats;
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>Obtiene los datos del Cuestionario cuyo identificador se pasa en "idTest" siempre y cuando
	 * ese Cuestionario pertenezca al Profesor cuyo identificador se pasa en el par&aacute;metro "idTeacher".</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idTest <code>int</code> identificador del Cuestionario.
	 * @return <code>TOTest</code> con la informaci&oacute;n del Cuestionario.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con
	 * la capa de persistencia.
	 * @throws InvalidIdException si "idTeacher" no es el de un usuario Profesor o si "idTest" no 
	 * corresponde con el identificador de ninguno de los Cuestionarios del sistema que pertenecen 
	 * al Profesor.
	 */
	@Override
	public TOTest getTest(int idTeacher, int idTest) throws DataStoreException, InvalidIdException
	{
		// Comprobamos que quien ha llamado al mtodo es un Profesor.
		if (!isTeacher(idTeacher))
		{
			// Al no se un Profesor lanzamos una excepcin.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
		}
		// El identificador corresponde al de un Profesor. Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos si "idTest" corresponde a un Cuestionario del Profesor. Sentencia HQL.
			Query query = session.createQuery("from BOTest where id=? and teacher=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, idTest, Hibernate.INTEGER);
			query.setParameter(1, idTeacher, Hibernate.INTEGER);
			// Ejecutamos la sentencia HQL y comprobamos el resultado.
			if ((query.list().size() == 1) && (query.list().get(0) instanceof BOTest))
			{
				// Retornamos el objeto TOTest que es equivalente a BOTest.
				return ((BOTest) query.list().get(0)).getTOTestComplete();
			}
			else
			{
				// Si se llega a este punto querr decir que el "idTest" no corresponde con el
				// de ninguno de los Cuestionarios del Profesor as que lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEST_ID));
			}
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado para obtener una lista de objetos <code>TOTest</code>
	 * que representar&aacute;n a los diferentes Cuestionarios que el Profesor cuyo identidicador
	 * se le pasa como par&aacute;metro, tiene registrados dentro del sistema.</p> 
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @return <code>List&lt;TOTest&gt;</code> donde cada elemento representa a un Cuestionario del 
	 * Profesor registrado en el sistema.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con
	 * la capa de persistencia.
	 * @throws InvalidIdException si el identificador de usuario que se le pasa no se corresponde con
	 * el de ninguno de los profesores registrados en el sistema.
	 */
	@Override
	public List<TOTest> listTest(int idTeacher) throws DataStoreException, InvalidIdException
	{
		// Comprobamos que quien ha llamado al mtodo es un Profesor.
		if (!isTeacher(idTeacher))
		{
			// Al no se un Profesor lanzamos una excepcin.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
		}
		// El identificador corresponde al de un Profesor. Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Creamos una lista de objetos TOTest que ser la que retornaremos.
			List<TOTest> list_test = new ArrayList<TOTest>();
			// Obtenemos la lista de Test creados por el Profesor.
			List<Object> list_obj_test = getAll(session, Table.TEST, "teacher", idTeacher);
			// Recorremos la lista de resultados obtenidos.
			for (Object obj_test : list_obj_test)
			{
				// Comprobamos que "obj_test" es de tipo BOTest.
				if (obj_test instanceof BOTest)
				{
					// Realizamos un cast de BOTest y aadimos a la lista a retornar
					// el objeto TOTest equivalente.
					list_test.add(((BOTest) obj_test).getTOTest());
				}
			}
			// Ordenamos la lista de elementos a retornar.
			Collections.sort(list_test);
			// Retornamos la lista de Cuestionarios.
			return list_test;
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado para obtener una lista de objetos <code>TOTestStat</code>
	 * donde cada uno de ellos representar&aacute; las estad&iacute;sticas de resoluci&oacute;n de cada
	 * uno de los Cuestionarios que el Profesor cuyo identificador se recibe como par&aacute;metro tiene
	 * definido en el sistema.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @return <code>List&lt;TOTestStats&gt;</code> donde cada elemento representa a las estad&iacute;sticas
	 * de un Cuestionario del Profesor.
	 * @throws DataStoreException si se produce alg&uacute;n error a la hora de trabajar con la capa de 
	 * persistenica.
	 * @throws InvalidIdException si el identificador de usuario que se le pasa no se corresponde con
	 * el de ninguno de los profesores registrados en el sistema.
	 */
	@Override
	public List<TOTestStats> listTestStats(int idTeacher) throws DataStoreException, InvalidIdException
	{
		// Comprobamos que el identificador recibido es de un Profesor.
		if (!isTeacher(idTeacher))
		{
			// No es un Profesor. Lanzamos una InvalidIdException.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
		}
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Creamos la lista que contendr los objeto TOTestStats
			List<TOTestStats> list_stats = new ArrayList<TOTestStats>();
			// Obtenemos la lista de todos los Cuestionarios del Profesor. Sentencia HQL.
			Query query = session.createQuery("from BOTest where teacher=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, idTeacher, Hibernate.INTEGER);
			// Recorremos la lista de resultados obtenidos.
			for (Object objTest : query.list())
			{
				// Comprobamos que objTest es de tipo BOTest.
				if (objTest instanceof BOTest)
				{
					// Cast de objTest
					BOTest boTest = (BOTest) objTest;
					// Comprobamos si el Test tiene preguntas asignadas o no.
					if ((boTest.getQuestions() == null) || (boTest.getQuestions().size() == 0))
					{
						// El Test no tiene preguntas asignada. Creamos un objeto TOTestStats
						// y lo aadimos a la lista de Estadsticas.
						list_stats.add(new TOTestStats(boTest.getId(), boTest.getTitle(), 0, 0));
					}
					else
					{
						// Variable local para contar el nmero de respuesta al Test.
						int totalAnswers = 0;
						// Tenemos que obtener la informacin de las veces que el Test
						// ha sido asignado a un Aula. Sentencia HQL.
						Query query2 = session.createQuery("from BOTestClassroom where test=?");
						// Establecemos el valor del parmetro.
						query2.setParameter(0, boTest.getId(), Hibernate.INTEGER);
						// Ejecutamos la sentencia y recorremos los resultados.
						for (Object objTestClassroom : query2.list())
						{
							// Comprobamos que objTestClassroom es de tipo BOTestClasroom
							if (objTestClassroom instanceof BOTestClassroom)
							{
								// Cast de objTestClassroom.
								BOTestClassroom boTestClassroom = (BOTestClassroom) objTestClassroom;
								// Ahora tenemos que contar el nmero de respuestas de ese test en
								// ese aula. Sentencia HQL.
								Query query3 = session.createQuery("from BOTestAnswer where test_classroom=?");
								// Establecemos el valor del parmetro.
								query3.setParameter(0, boTestClassroom.getId(), Hibernate.INTEGER);
								// Ejecutamos la consulta y vemos si hay resultados.
								if (query3.list() != null)
								{
									// Sumamos el nmero de registros devuelto al total de respuestas.
									totalAnswers += query3.list().size();
								}
							}
						}
						// Creamos un objeto TOTestStats y lo aadimos a la lista.
						list_stats.add(new TOTestStats(boTest.getId(), boTest.getTitle(), 
								boTest.getQuestions().size(), totalAnswers));
					}
				}
			}
			// Ordenamos la lista de Estadsticas.
			Collections.sort(list_stats);
			// Retornamos la lista de las Estadsticas.
			return list_stats;
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>Modifica los datos del Cuestionario identificado por el atributo "idTest" (Cuestionario que
	 * ha de pertenecer al Profesor identificador por "idTeacher") con la informaci&oacute;n suministrada
	 * en el resto de par&aacute;metros del m&eacute;todo.</p>
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idTest <code>int</code> identificador del Cuestionario.
	 * @param introduction <code>String</code> texto de introducci&oacute;n.
	 * @param shuffleQuestions <code>boolean</code> indica si se deben barajar las preguntas.
	 * @param shuffleAnswers <code>boolean</code> indica si se deben barajar las respuestas.
	 * @param showResult <code>boolean</code> indica si se mostrar&aacute; el resultado.
	 * @param showReview <code>boolean</code> indic asi se mostrar&aacute; la revisi&oacute;n.
	 * @return <code>TOTest</code> con la informaci&oacute;n del Cuestionario modificada.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con
	 * la capa de persistencia.
	 * @throws InvalidIdException si "idTeacher" no es el de un usuario Profesor o si "idTest" no 
	 * corresponde con el identificador de ninguno de los Cuestionarios del sistema que pertenecen 
	 * al Profesor.
	 */
	@Override
	public TOTest modifyTests(int idTeacher, int idTest, String introduction, 
			boolean shuffleQuestions, boolean shuffleAnswers, boolean showResult, boolean showReview)
			throws DataStoreException, InvalidIdException
	{
		// Comprobamos que quien ha llamado al mtodo es un Profesor.
		if (!isTeacher(idTeacher))
		{
			// Al no se un Profesor lanzamos una excepcin.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
		}
		// El identificador corresponde al de un Profesor. Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos si "idTest" corresponde a un Cuestionario del Profesor. Sentencia HQL.
			Query query = session.createQuery("from BOTest where id=? and teacher=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, idTest, Hibernate.INTEGER);
			query.setParameter(1, idTeacher, Hibernate.INTEGER);
			// Ejecutamos la sentencia HQL y comprobamos el resultado.
			if ((query.list().size() == 1) && (query.list().get(0) instanceof BOTest))
			{
				// Cast del objeto BOTest.
				BOTest test = (BOTest) query.list().get(0);
				// Establecemos los nuevos valores de los atributos.
				test.setIntroduction(WebUtils.stringToHTML(introduction));
				test.setShuffleQuestions(shuffleQuestions);
				test.setShuffleAnswers(shuffleAnswers);
				test.setShowResult(showResult);
				test.setShowReview(showReview);
				// Modificamos la informacin en la capa de persistencia.
				session.saveOrUpdate(test);
				// Consolidamos la operacin.
				session.flush();
				// Retornamos el objeto TOTest equivalente al que acabamos de actulizar.
				return test.getTOTest();
			}
			else
			{
				// Si se llega a este punto querr decir que el "idTest" no corresponde con el
				// de ninguno de los Cuestionarios del Profesor as que lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEST_ID));
			}
		}
		finally
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo que se encargar&aacute; de desasignar una Pregunta de un Cuestionario de un
	 * un Profesor. En el caso de que se llame a este m&eacute;todo y la pregunta no haya sido asignada
	 * previamente el Cuestionario, el m&eacuete;todo no realizar&aacute; ninguna acci&oacute;n.</p> 
	 * 
	 * @param idTeacher <code>int</code> identificador del Profesor.
	 * @param idTest <code>int</code> identificador del Cuestionario.
	 * @param idQuestion <code>int</code> identificador de la Pregunta.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con
	 * la capa de persistencia.
	 * @throws InvalidIdException si el identificador de Profesor no corresponde con el de un Profesor
	 * registrado en el sistema; si el identificador de Cuestionario no corresponde con el de un Cuestionario
	 * del Profesor; si el identificador de la Pregunta no corresponde con el de una Pregunta de Profesor.
	 */
	@Override
	public void removeQuestion(int idTeacher, int idTest, int idQuestion)
			throws DataStoreException, InvalidIdException
	{
		// Comprobamos que quien ha llamado al mtodo es un Profesor.
		if (!isTeacher(idTeacher))
		{
			// Al no se un Profesor lanzamos una excepcin.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEACHER_ID));
		}
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Comprobamos que el Cuestionario pertenece al profesor. Sentencia HQL.
			Query query = session.createQuery("from BOTest where id=? and teacher=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, idTest, Hibernate.INTEGER);
			query.setParameter(1, idTeacher, Hibernate.INTEGER);
			// Ejecutamos la consulta y vemos el nmero de resultados y el tipo.
			if ((query.list().size() != 1) || (!(query.list().get(0) instanceof BOTest)))
			{
				// No se trata de un Cuestionario del Profesor. Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEST_ID));
			}
			// Comprobamos que la pregunta sea del profesor. 
			Object objQuestion = existFieldValue(session, Table.QUESTIONS, "id", idQuestion);
			// Comprobamos que no sea nulo y que sea de tipo BOQuestion.
			if ((objQuestion == null) || (!(objQuestion instanceof BOQuestion)))
			{
				// El identificador de la pregunta es errneo. Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_QUESTION_ID));
			}
			// Cast de objQuestion.
			BOQuestion boQuestion = (BOQuestion) objQuestion;
			// Comprobamos que la categora de preguntas a la que pertenece la pregunta es del Profesor.
			query = session.createQuery("from BOQuestionCategory where id=? and user=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, boQuestion.getCategory().getId(), Hibernate.INTEGER);
			query.setParameter(1, idTeacher, Hibernate.INTEGER);
			// Ejecutamos la consulta.
			if (query.list().size() != 1)
			{
				// La pregunta pertenece a una categora que no es del usuario. Lanzamos una InvalidIdException.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_QUESTION_ID));
			}
			// Finalizadas todas las comprobaciones ya podemos desasignar la pregunta. Sentencia HQL.
			query = session.createQuery("from BOTestQuestion where test=? and question=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, idTest, Hibernate.INTEGER);
			query.setParameter(1, idQuestion, Hibernate.INTEGER);
			// Ejecutamos la consulta y comprobamos el nmero de resultados y el tipo.
			if ((query.list().size() == 1) && (query.list().get(0) instanceof BOTestQuestion))
			{
				// nicamente en este caso tendremos que realizar la desasignacin. (si la pregunta no
				// haba sido asignada previamente este mtodo no har nada). Cast del objeto.
				BOTestQuestion bo = (BOTestQuestion) query.list().get(0);
				// Eliminamos el objeto de la capa de persistencia.
				session.delete(bo);
				// Consolidamos la operacin.
				session.flush();
			}
		}
		finally 
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; el encargado de recoger dentro del sistema la informaci&oacute;n
	 * relativa a una respuesta de un Cuestionario por parte de un Alumno.</p>
	 * 
	 * @param idStudent <code>int</code> el identificador del Alumno.
	 * @param idTestClassroom <code>int</code> el identificador del Test una vez asociado a un Aula.
	 * @param score <code>double</code> la puntuaci&oacute;n obtenida.
	 * @param maxScore <code>double</code> la puntuaci&oacute;n m&aacute;xima que se pod&iacute;a obtener.
	 * @param totalQuestions <code>int</code> n&uacute;mero de Preguntas que forman el Cuestionario.
	 * @param correctQuestions <code>int</code> n$uacute;mero de Preguntas contestadas correctamente.
	 * @return <code>TOTestAnswer</code> con toda la informaci&oacute;n de la respuesta una vez que ha 
	 * sido grabada dentro del sistema.
	 * @throws DataStoreException si se produce alg&uacute;n tipo de error a la hora de trabajar con la
	 * capa de persistencia.
	 * @throws InvalidIdException si el identificador que se le pasa no corresponde con el de un usuario
	 * registrado como Alumno en el sistema o si el identificador del Cuestionario no corresponde con el
	 * identificador de ninguna de los Cuestionarios una vez han sido asignados a un Aula.
	 */
	@Override
	public TOTestAnswer saveAnswer(int idStudent, int idTestClassroom, double score,
			double maxScore, int totalQuestions, int correctQuestions) throws DataStoreException, InvalidIdException
	{
		// Abrimos una sesin de Hibernate.
		Session session = this.sessionFactory.openSession();
		try
		{
			// Vamos a comprobar que idTestClassroom es correcto. Sentencia HQL.
			Query query = session.createQuery("from BOTestClassroom where id=?");
			// Definimos el parmetro de la consulta.
			query.setParameter(0, idTestClassroom, Hibernate.INTEGER);
			// Ejecutamos la consulta comprobando el nmero de resultados y el tipo.
			if ((query.list().size() != 1) || (!(query.list().get(0) instanceof BOTestClassroom)))
			{
				// El identificador de test es incorrecto. Lanzamos una excepcin.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_TEST_ID));
			}
			// Realizamos un Cast del resultado.
			BOTestClassroom boTestClassroom = (BOTestClassroom) query.list().get(0);
			// Preparemos la consulta HQL para comprobar si idStudent es de un estudiante del Aula.
			query = session.createQuery("from BOClassroomUser where classroom=? and user=? and role=?");
			// Establecemos los valores de los parmetros.
			query.setParameter(0, boTestClassroom.getClassroom().getId(), Hibernate.INTEGER);
			query.setParameter(1, idStudent, Hibernate.INTEGER);
			query.setParameter(2, BOClassroomUser.ROLE_STUDENT, Hibernate.STRING);
			// Ejecutamos la consulta, vemos el nmero de resultado y el tipo.
			if ((query.list().size() != 1) || (!(query.list().get(0) instanceof BOClassroomUser)))
			{
				// El identificador de estudiante no ser vlido. Lanzamos una excepcin.
				throw (new InvalidIdException(InvalidIdException.MSG_INVALID_STUDENT_ID));
			}
			// Cast de BOClassroomUser.
			BOClassroomUser boClassroomUser = (BOClassroomUser) query.list().get(0);
			// Vamos a ver si ya se haba contestado previamente al Test o no. Sentencia HQL.
			query = session.createQuery("from BOTestAnswer where student=? and test_classroom=?");
			// Establecemos los valores de los atributos.
			query.setParameter(0, idStudent, Hibernate.INTEGER);
			query.setParameter(1, idTestClassroom, Hibernate.INTEGER);
			// Ejecutamos la consulta viendo el nmero de resultados y el tipo.
			if ((query.list().size() == 1) && (query.list().get(0) instanceof BOTestAnswer))
			{
				// El alumno ya haba contestado previamente al Test. En este caso retornamos
				// la informacin de la respuesta que ya est registrada. Cast de la respuesta.
				BOTestAnswer boTestAnswer = (BOTestAnswer) query.list().get(0);
				// Retornamos el objeto TOTestAnswer equivalente.
				return (boTestAnswer.getTOTestAnswer());
			}
			else
			{
				// Creamos un objeto BOTestAnswer a partir de la informacin que tenemos.
				BOTestAnswer boTestAnswer = new BOTestAnswer(boClassroomUser.getUser(), 
						boTestClassroom, score, maxScore, totalQuestions, correctQuestions);
				// Grabamos la informacin de la respuesta.
				session.saveOrUpdate(boTestAnswer);
				// Consolidamos la operacin.
				session.flush();
				// Retornamos el objeto TOTestAnswer equivalente al que hemos guardado.
				return (boTestAnswer.getTOTestAnswer());
			}
		}
		finally 
		{
			// Cerramos la sesin de Hibernate.
			closeSession(session);
		}
	}	
		
}
