package edu.uoc.pfc.business.transfers;

import java.io.Serializable;
import java.util.List;

/**
 * <p>Clase que recoge las estad&iacute;sticas de un determinado Cuestionario dentro del sistema.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0 
 */
public class TOTestStats implements Serializable, Comparable<TOTestStats>
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Identificador del Cuestionario.</p> */
	private int idTest;
	/** <p>T&iacute;tulo del Cuestionario.</p> */
	private String title;
	/** <p>N&uacute;mero de Preguntas.</p> */
	private int totalQuestions;
	/** <p>N&uacute;mero de Respuestas registradas.</p> */
	private int totalAnswers;
	
	/** <p>Lista de objetos <code>TOTestAnswerStats</code> con las Estad&iacute;sticas de las Respuestas.</p> */
	private List<TOTestAnswerStats> details;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objeto <code>TOTestStats</code> a partir de la informaci&oacute;n
	 * que se le pasa como par&aacute;metro.</p>
	 * 
	 * @param idTest <code>int</code> identificador del Cuestionario.
	 * @param title <code>String</code> t&iacute;tulo del Cuestionario.
	 * @param totalQuestions <code>int</code> n&uacute;mero total de Preguntas del Cuestionario.
	 * @param totalAnswers <code>int</code> n&uacute;mero de Respuestas recibidas del Cuestionario.
	 */
	public TOTestStats(int idTest, String title, int totalQuestions,
			int totalAnswers) {
		// Llamamos al constructor de la superclase.
		super();
		// Asignamos los valores a los atributos.
		this.idTest = idTest;
		this.title = title;
		this.totalQuestions = totalQuestions;
		this.totalAnswers = totalAnswers;
	}
	
	// -------- MTODOS PBLICOS
	@Override
	public int compareTo(TOTestStats o)
	{
		// Comprobamos que el parmetro no sea nulo.
		if (o == null)
		{
			// Si el parmetro es nulo retornamos 1 indicando que "this" es mayor.
			return 1;
		}
		// Retornamos la comparacin de los ttulos de los objetos en minsculas.
		return (this.title.toLowerCase().compareTo(o.getTitle().toLowerCase()));
	}
	
	/**
	 * <p>Retorna el n&uacute;mero de aprobados del Cuestionario.</p>
	 * 
	 * @return <code>int</code> n&uacute;mero de aprobados del Cuestionario.
	 */
	public int getTotalPass()
	{
		// Comprobamos que tenemos la informacin de las respuestas.
		if (this.details == null)
		{
			// Si no tenemos informacin de las respuestas retornamos 0.
			return 0;
		}
		// Declaramos una variable local que contar el nmero de respuestas correctas.
		int count = 0;
		// Recorremos todas las respuestas.
		for (TOTestAnswerStats stats : this.details)
		{
			// Comprobamos si est aprobado y si es as incrementamos el nmero de 
			// respuestas correctas.
			if (stats.isPass()) count++;
		}
		// Retornamos el nmero de respuestas correctas.
		return count;
	}
	
	/**
	 * <p>Retorna el n&uacute;mero de suspensos del Cuestionario.</p>
	 * 
	 * @return <code>int</code> n&uacute;mero de suspensos del Cuestionario.
	 */
	public int getTotalNoPass()
	{
		// Comprobamos que tenemos la informacin de las respuestas.
		if (this.details == null)
		{
			// Si no tenemos informacin de las respuestas retornamos 0.
			return 0;
		}
		// Declaramos una variable local que contar el nmero de respuestas errneas.
		int count = 0;
		// Recorremos todas las respuestas.
		for (TOTestAnswerStats stats : this.details)
		{
			// Comprobamos si est suspenso y si es as incrementamos el nmero de 
			// respuestas errneas.
			if (!stats.isPass()) count++;
		}
		// Retornamos el nmero de respuestas correctas.
		return count;
	}
	
	/**
	 * <p>Retorna el n&uacute;mero de respuestas que han obtenido una Matr&iacute;la de Honor.</p> 
	 * 
	 * @return <code>int</code> con el n&uacute;mero de respuestas con dicha calificaci&oacute;n.
	 */
	public int getTotalMH()
	{
		// Retornamos el nmero de respuestas que cumplen la condicin.
		return getTotalFinalScore(TOTestAnswer.FINAL_SCORE_MATRICULA_HONOR);
	}
	

	/**
	 * <p>Retorna el n&uacute;mero de respuestas que han obtenido un Sobresaliente.</p> 
	 * 
	 * @return <code>int</code> con el n&uacute;mero de respuestas con dicha calificaci&oacute;n.
	 */
	public int getTotalEX()
	{
		// Retornamos el nmero de respuestas que cumplen la condicin.
		return getTotalFinalScore(TOTestAnswer.FINAL_SCORE_SOBRESALIENTE);
	}
	

	/**
	 * <p>Retorna el n&uacute;mero de respuestas que han obtenido un Notable.</p> 
	 * 
	 * @return <code>int</code> con el n&uacute;mero de respuestas con dicha calificaci&oacute;n.
	 */
	public int getTotalNO()
	{
		// Retornamos el nmero de respuestas que cumplen la condicin.
		return getTotalFinalScore(TOTestAnswer.FINAL_SCORE_NOTABLE);
	}

	/**
	 * <p>Retorna el n&uacute;mero de respuestas que han obtenido un Aprobado.</p> 
	 * 
	 * @return <code>int</code> con el n&uacute;mero de respuestas con dicha calificaci&oacute;n.
	 */
	public int getTotalA()
	{
		// Retornamos el nmero de respuestas que cumplen la condicin.
		return getTotalFinalScore(TOTestAnswer.FINAL_SCORE_APROBADO);
	}
	
	/**
	 * <p>Retorna el n&uacute;mero de respuestas que han obtenido un Suspenso.</p> 
	 * 
	 * @return <code>int</code> con el n&uacute;mero de respuestas con dicha calificaci&oacute;n.
	 */
	public int getTotalSU()
	{
		// Retornamos el nmero de respuestas que cumplen la condicin.
		return getTotalFinalScore(TOTestAnswer.FINAL_SCORE_SUSPENSO);
	}
	
	// -------- MTODOS PRIVADOS
	/**
	 * <p>Retorna el n&uacute;mero total de Respuestas del Cuestionario que han obtenido la calificaci&oacute;n
	 * que se le pasa como par&aacute;metro.</p>
	 * 
	 * @param value <code>String</code> calificaci&oacute;n a buscar.
	 * @return <code>int</code> con el n&uacute;mero de Respuestas que han alcanzado la nota especificada.
	 */
	private int getTotalFinalScore(String value)
	{
		// Comprobamos que tanto this.details como value no son nulos.
		if ((this.details == null) || (value == null))
		{
			// En este caso retornaremos 0
			return 0;
		}
		// Declaramos una variable local para llevar la cuenta de respuestas que coinciden.
		int totalAnswers = 0;
		// Recorremos la lista de respuestas.
		for (TOTestAnswerStats toTestAnswerStats : this.details)
		{
			// Vemos si la calificacin es la misma y si as es sumamos uno a totalAnswers.
			if (value.equals(toTestAnswerStats.getFinalScore()))
				totalAnswers++;
		}
		// Retornamos el nmero de respuestas encontradas.
		return totalAnswers;
	}
	
	
	// -------- GETTERS & SETTERS.
	public int getIdTest()
	{
		return idTest;
	}
	public void setIdTest(int idTest)
	{
		this.idTest = idTest;
	}
	public String getTitle()
	{
		return title;
	}
	public void setTitle(String title)
	{
		this.title = title;
	}
	public int getTotalQuestions()
	{
		return totalQuestions;
	}
	public void setTotalQuestions(int totalQuestions)
	{
		this.totalQuestions = totalQuestions;
	}
	public int getTotalAnswers()
	{
		return totalAnswers;
	}
	public void setTotalAnswers(int totalAnswers)
	{
		this.totalAnswers = totalAnswers;
	}
	public List<TOTestAnswerStats> getDetails() 
	{
		return details;
	}
	public void setDetails(List<TOTestAnswerStats> details) 
	{
		this.details = details;
	}

}
