package edu.uoc.pfc.business.transfers;

import java.io.Serializable;
import java.util.Date;

/**
 * <p>Clase que ser&aacute; utilizada para realizar el intercambio de informaci&oacute;n
 * correspondientes a las cuentas de los usuarios de la aplicaci&oacute;n entre las
 * diferentes capas que componen la arquitectura del sistema.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0 
 */
public class TOUser implements Serializable, Comparable<TOUser> 
{
	
	// ---------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	/** <p>Ordenar los objetos de este tipo por el apellido.</p> */
	public static final int ORDER_BY_SURNAME = 1;
	/** <p>Ordenar los objetos de este tipo por el login.</p> */
	public static final int ORDER_BY_LOGIN = 2;
	
	
	// ---------- ATRIBUTOS.
	/** <p>Identificador del usuario. </p> */
	private int id;
	/** <p>Nombre de usuario (login).</p> */
	private String login;
	/** <p>Contrase&ntilde;a de acceso (password).</p> */
	private String password;
	/** <p>Indica si la cuenta est&aacute; activa o no.</p> */
	private boolean active;
	/** <p>Fecha en la que se ha realizado el &uacute;ltimo acceso a la cuenta.</p> */
	private Date lastLogin;
	/** <p>Fecha de creaci&oacute; de la cuenta.</p> */
	private Date created;
	/** <p>Indica si el usuario es un <i>Administrador</i>.</p> */
	private boolean admin;
	/** <p>Indica si el usuario es un <i>Profesor</i>.</p> */
	private boolean teacher;
	/** <p>Indica si el usuario es un <i>Estudiante</i>.</p> */
	private boolean student;
	/** <p>Nombre real del usuario.</p> */
	private String name;
	/** <p>Apellidos del usuario.</p> */
	private String surname;
	/** <p>Direcci&oacute;n de correo electr&oacute;nico.</p> */
	private String email;
	/** <p>Indica por qu atributo hay que ordenar los objetos.</p> */
	private int orderBy;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objeto <code>TOUser</code> inicializando todos sus 
	 * par&aacute;metros a sus valores por defecto.</p>
	 */
	public TOUser()
	{
		// Constructor de la superclase.
		super();
	}
	
	/**
	 * <p>Construye un objeto <code>TOUser</code> inicializando sus atributos a los
	 * hom&oacute;nimos que recibe como par&aacute;metro.</p>
	 * 
	 * @param id <code>id</code> Identificador del usuario.
	 * @param login <code>String</code> login de acceso.
	 */
	public TOUser(int id, String login)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Asignamos los atributos.
		this.id = id;
		this.login = login;
		// Por defecto hacemos que se ordene por el login.
		this.orderBy = ORDER_BY_LOGIN;
	}
	
	/**
	 * <p>Construye un objeto <code>TOUser</code> inicializando sus atributos a 
	 * los hom&oacute;nimos que recibe como par&acute;metros.</p>
	 * 
	 * @param login <code>String</code> nombre de usuario (login).
	 * @param password <code>String</code> contrase&ntilde;a.
	 * @param email <code>String</code> direcci&oacute;n de correo electr&oacute;nico.
	 * @param isAdmin <code>boolean</code> es un administrador?
	 * @param isTeacher <code>boolean</code> es un profesor?
	 * @param isStudent <code>boolean</code> es un estudiante?
	 * @param isActive <code>boolean</code> cuenta activa?
	 * @param name <code>String</code> nombre real del usuario.
	 * @param surname <code>String</code> apellidos del usuario.
	 */
	public TOUser(String login, String password, String email, boolean isAdmin, 
			boolean isTeacher, boolean isStudent, boolean isActive, String name,
			String surname)
	{
		// Constructor de la superclase.
		super();
		// Establecemos los valores de los atributos.
		this.login = login;
		this.password = password;
		this.email = email;
		this.admin = isAdmin;
		this.teacher = isTeacher;
		this.student = isStudent;
		this.active = isActive;
		this.name = name;
		this.surname = surname;
		// Por defecto hacemos que se ordene por el login.
		this.orderBy = ORDER_BY_LOGIN;
	}
	
	/**
	 * <p>Construye un objeto <code>TOUser</code> a partir de la informaci&oacute;n 
	 * que se le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador del usuario.
	 * @param name <code>String</code> nombre del usuario.
	 * @param surname <code>String</code> apellidos del usuario.
	 */
	public TOUser(int id, String name, String surname)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Establecemos los valores de los atributos.
		this.id = id;
		this.name = name;
		this.surname = surname;
	}
	
	// -------- MTODOS PBLICOS.
	@Override
	public int compareTo(TOUser o) 
	{
		// Comprobamos que el parmetro no es nulo.
		if (o == null)
		{
			// Si el parmetro es null vamos a suponer que "this" es mayor siempre.
			return 1;
		}
		// Vemos por qu atributo tenemos que ordenar.
		switch (this.orderBy)
		{
			case ORDER_BY_SURNAME :
				// Comprobamos si el atributo "surname" tiene el valor "null"
				if (this.surname == null)
				{
					// En el caso de que el apellido sea null el mtodo retornar -1 (es menor)
					return -1;
				}
				// Comparamos los apellidos de los usuarios.
				if (this.surname.toLowerCase().equals(o.getSurname().toLowerCase()))
				{
					// Los apellidos son iguales por lo que pasamos a comparar los nombres.
					return this.name.toLowerCase().compareTo(o.getName().toLowerCase());
				}
				else
				{
					// Retornamos el resultao de comparar los apellidos.
					return this.surname.toLowerCase().compareTo(o.getSurname().toLowerCase());
				}
			default:
				// Tenemos que ordenar por al "login".
				return this.login.toLowerCase().compareTo(o.getLogin().toLowerCase());
		}
	}

	// ---------- GETTERS & SETTERS.
	public String getLogin() 
	{
		return login;
	}
	public void setLogin(String login) 
	{
		this.login = login;
	}
	public String getPassword() 
	{
		return password;
	}
	public void setPassword(String password) 
	{
		this.password = password;
	}
	public boolean isActive()
	{
		return active;
	}
	public void setActive(boolean active)
	{
		this.active = active;
	}
	public Date getLastLogin()
	{
		return lastLogin;
	}
	public void setLastLogin(Date lastLogin)
	{
		this.lastLogin = lastLogin;
	}
	public boolean isAdmin()
	{
		return admin;
	}
	public void setAdmin(boolean admin)
	{
		this.admin = admin;
	}
	public boolean isTeacher()
	{
		return teacher;
	}
	public void setTeacher(boolean teacher)
	{
		this.teacher = teacher;
	}
	public boolean isStudent()
	{
		return student;
	}
	public void setStudent(boolean student)
	{
		this.student = student;
	}
	public String getName()
	{
		return name;
	}
	public void setName(String name)
	{
		this.name = name;
	}
	public String getSurname()
	{
		return surname;
	}
	public void setSurname(String surname)
	{
		this.surname = surname;
	}
	public String getEmail()
	{
		return email;
	}
	public void setEmail(String email)
	{
		this.email = email;
	}
	public int getId()
	{
		return id;
	}
	public void setId(int id)
	{
		this.id = id;
	}
	public Date getCreated()
	{
		return created;
	}
	public void setCreated(Date created)
	{
		this.created = created;
	}
	public int getOrderBy()
	{
		return orderBy;
	}
	public void setOrderBy(int orderBy)
	{
		this.orderBy = orderBy;
	}
	
}
