package edu.uoc.pfc.lib.utils;

import java.util.Properties;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

/**
 * <p>Ofrece una serie de m&eacute;todos que sern utilizados para realizar el 
 * env&iacute;o de correo electr&oacute;nicos.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0 
 */
public class MailUtils
{
	
	// -------- CONSTANTES.
	/** <p>Clave dentro del objeto <code>Properties</code> donde se define el servidor SMTP.</p> */
	private static final String SMTP_SERVER = "mail.smtp.host";
	/** <p>Clave dentro del objeto <code>Properties</code> donde se define el puerto de env&iacute;o SMTP.</p> */
	private static final String SMTP_PORT = "mail.smtp.port";
	/** <p>Clave dentro del objeto <code>Properties</code> para definir si TLS est&aacute; disponible o no.</p> */
	private static final String SMTP_TLS = "mail.smtp.starttls.enable";
	/** <p>Clave dentro del objeto <code>Properties</code> para definir si hay que autentificarse o no.</p> */
	private static final String SMTP_AUTH = "mail.smtp.auth";
	
	// -------- MTODOS PBLICOS Y ESTTICOS.
	/**
	 * <p>Enva un correo electr&oacute;nico a partir de los par&aacute;metros de configuraci&oacute;n que se le pasan.</p>
	 * 
	 * @param smtpServer <code>String</code> servidor SMTP.
	 * @param port <code>int</code> puerto SMTP en el servidor.
	 * @param tls <code>boolean</code> indica si el servidor utiliza TLS (Transport Layer Security).
	 * @param authenticate <code>boolean</code> indica si es necesario autentificarse en el servidor SMTP.
	 * @param smtpUser <code>String</code> usuario dentro del servidor SMTP. (Es <code>final</code> porque en el c&oacute;digo
	 * ser&aacute; utilizada dentro de una clase an&oacute;nima)
	 * @param smtpPassword <code>String</code> contrase&ntilde;a del usuario del servidor SMTP. Es <code>final</code> porque 
	 * en el c&oacute;digo ser&aacute; utilizada dentro de una clase an&oacute;nima)
	 * @param from <code>String</code> direcci&opacute;n de correo electr&oacute;nico de la persona que env&iacute;a el email.
	 * @param to <code>String</code> direcci&oacute;n de correo electr&oacute;nico de la persona que va a recibir el email.
	 * @param subject <code>String</code> asunto del correo.
	 * @param text <code>String</code> texto (cuerpo) del correo.
	 * @throws Exception si se produce un error a la hora de enviar el correo electr&oacute;nico.
	 */
	public static void sendSMTPEmail(String smtpServer, int port, boolean tls, boolean authenticate, final String smtpUser, 
			final String smtpPassword, String from, String to, String subject, String text) throws Exception
	{
		// No verificamos ninguno de los parmetros ya que si alguno de ellos no fuese correcto (por ejemplo
		// que no se haya pasado el servidor SMTP) se lanzar una Exception la cual propagaremos.
		try
		{
			// Creamos un objeto Properties.
			Properties properties = new Properties();
			// Establecemos el servidor SMTP.
			properties.setProperty(SMTP_SERVER, smtpServer);
			// Establecemos el puerto del servidor SMTP.
			properties.setProperty(SMTP_PORT, Integer.toString(port));
			// Establecemos si se utiliza TLS.
			properties.setProperty(SMTP_TLS, Boolean.toString(tls));
			// Establecemos si se es necesaria la autentificacin SMTP.
			properties.setProperty(SMTP_AUTH, Boolean.toString(authenticate));
			// Creamos una sesin de JavaMail en el servidor
			Session session = Session.getDefaultInstance(properties, 
					// Creamos una Authenticar con los datos del usuario y la contrasea.
					new javax.mail.Authenticator() 
					    {
							// Sobreescribimos el mtodo.
				   			protected PasswordAuthentication getPasswordAuthentication() 
				   			{
				   				return new PasswordAuthentication(smtpUser, smtpPassword);
				   			}
					    });
			// Creamos un objeto para contener el mensaje.
			MimeMessage message = new MimeMessage(session);
			// Establecemos el asunto.
			message.setSubject(subject, "ISO-8859-1");
			// Establecemos el texto.
			message.setText(text, "ISO-8859-1", "plain");
			// Aadimos los destinatarios del mensaje.
			message.addRecipient(Message.RecipientType.TO, new InternetAddress(to));
			// Aadimos la informacin de quin enva el mensaje.
			message.setFrom(new InternetAddress(from));
			// Establecemos Message como una variable final para poder ser accedida dentro
			// de un hilo que vamos a crear ya que el envo de correo suele llevar bastante 
			// tiempo y as ahorramos recursos.
			final Message msg = message;
			// Creamos un hilo, sobreescribimos el mtodo run() y lanzamos su ejecucin.
			new Thread() 
			{
				// Sobreescribimos el mtodo run().
				public void run()
				{
					try
					{
						// Enviamos el mensaje.		
						Transport.send(msg);
					} 
					catch (MessagingException e)
					{
						// Si se produce una excepcin escribimos la traza de la pila
						e.printStackTrace();
					}
				}
			// Inicializamos el hilo.
			}.start();
		}
		catch (Exception ex)
		{
			// Cualquier excepcin que se produzca la propagamos.				
			throw ex;
		}
	}
	
}
