package edu.uoc.pfc.lib.utils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * <p>Clase de utilidad cuya API est&aacute; formada por una serie de m&eacute;todos 
 * p&uacute;blicos y est&aacute;ticos que permtir&aacute;n realizar validaciones sobre 
 * los par&aacute;metros que reciben.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez.
 * @version 1.0
 */
public class ValidationUtils
{
	
	// ---------- METODOS PUBLICOS Y ESTATICOS
	/**
	 * <p>Comprueba si el <code>String</code> que recibe como par&aacute;metro est&aacute;
	 * vac&iacute;o o no. Se considera que un <code>String</code> est&aacute; vac&iacute;o
	 * si es <code>null</code> o su longitud (n&uacute;mero de caracteres que lo forman),
	 * tras ignorar los espacios en blanco que pueda contener al principio o final, es
	 * igual a cero.</p>
	 * 
	 * @param param <code>String</code> que se quiere verificar.
	 * 
	 * @return <code>true</code> si se cumplen las condiciones en el <code>String</code> para que
	 * sea considerado como vac&iacute;o o <code>false</code> en caso contrario.
	 */
	public static boolean isEmpty(String param)
	{
		// Si el parametro que se recibe es null se retorna true.
		if (param == null) return true;
		// Comprobamos que la longitud de la cadena no sea cero.
		if (param.trim().length() == 0) return true;
		// Si llegamos a este punto quiere decir que la cadena no estar vaca.
		return false;
	}
	
	/**
	 * <p>Comprueba si el <code>String</code> que se le pasa como par&aacute;metro 
	 * est&aacute; formado por un  n&uacute;mero de caracteres comprendidos entre el 
	 * valor <i>min</i> y <i>max</i> sin tener en cuenta los espacios en blanco que 
	 * el <code>String</code> pueda poseer al principio o final.</p>
	 * 
	 * @param param <code>String</code> que se quiere verificar.
	 * @param min <code>int</code> n&uacute;mero m&iacute;nimo de caracteres.
	 * @param max <code>int</code> n&uacute;mero m&aacute;ximo de caracteres.
	 * @return <code>true</code> en el caso de que el <code>String</code> tenga un 
	 * n&uacute;mero de caracteres dentro del rango que se ha pasado como par&aacute;metro 
	 * o <code>false</code> en caso contrario.
	 */
	public static boolean isInRange(String param, int min, int max)
	{
		// Comprobamos que min y max son valores mayores que cero. Si no es as retornamos false.
		if ((min < 0) || (max < 0)) return false;
		// Comprobamos que min <= max. Si no es as retornamos false.
		if (min > max) return false;
		// Si el String que se ha pasado es vaco retornaremos false.
		if (isEmpty(param)) return false;
		// Comprobamos si el nmero de caracteres de la cadena cae en el rango especificado.
		if ((param.trim().length() >= min) && (param.trim().length() <= max))
			// Cae en el rango as que retornamos true.
			return true;
		// Aqu solamente llegaremos si el nmero de caracteres no cae en el rango.
		return false;
	}
	
	/**
	 * <p>Comprueba si el <code>String</code> que se le pasa como par&aacute;metro contiene 
	 * una direccin de correo electrnico vlida. El m&eacute;todo no comprueba la existencia
	 * de dicha direcci&oacute;n de correo, solamente su formato.</p>
	 * 
	 * @param param <code>String</code> que se quiere verificar.
	 * @return <code>true</code> si <i>param</i> contiene una direcci&oacute;n de correo
	 * electr&oacute;nico v&aacute;lida o <code>false</code> en caso contrario.
	 */
	public static boolean isEmail(String param)
	{
		// Si "param" es null retornamos false.
		if (param == null) return false;
		// Utilizamos una expresin regular para verificar el email.
		Pattern patter = Pattern.compile("[a-zA-Z0-9]+[.[a-zA-Z0-9_-]+]*@[a-z0-9][\\w\\.-]*[a-z0-9]\\.[a-z][a-z\\.]*[a-z]$");
		Matcher matcher = patter.matcher(param.trim());
		// Vemos si se ha producido una coincidencia o no.
		if (matcher.find()) return true;
		// Llegados a este punto no se habr producido una coincidencia.
		return false;
	}
	
	/**
	 * <p>Comprueba que <i>day</i> (d&iacute;a), <i>month</i> (mes) y <i>year</i> (a&ntilde;o)
	 * se pueden interpretar como una fecha v&aacute;lida.</p>
	 * 
	 * @param day <code>String</code> d&acute;ia.
	 * @param month <code>String</code> mes.
	 * @param year <code>String</code> a&ntilde;o.
	 * @return <code>true</code> si los par&aacute;metros recibidos pueden ser interpretrados 
	 * como una fecha v&aacute;lida o <code>false</code> en caso contrario.
	 */
	public static boolean isValidDate(String day, String month, String year)
	{
		// Si alguno de los parmetros es null retornamos false.
		if ((day == null) || (month == null) || (year == null)) return false;
		// Utilizamos un objeto SimpleDateFormat.
		try 
		{
			SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy", Locale.getDefault());
			// Establecemos que value las fechas en sentido estricto (bisiestos).
			sdf.setLenient(false);
			// Interpretamos la fecha pasada.
			sdf.parse(day + "/" + month + "/" + year);
			// La fecha es correcta.
			return true;
		}
		catch (ParseException pe)
		{
			// Los parmetros no corresponden con una fecha.
			return false;
		}
	}
	
	/**
	 * <p>Comprueba que los valores pasados en los par&aaacute;metros se corresponden con los de una
	 * fecha v&aacute;lida.</p>
	 * 
	 * @param day <code>String</code> d&iacute;a del mes.
	 * @param month <code>String</code> mes.
	 * @param year <code>String</code> a&ntilde;o
	 * @param hour <code>String</code> hora del d&iacute;a
	 * @param minutes <code>String</code> minutos dentro de la hora.
	 * @return <code>true</code> si se trata de una fecha v&aacute;lida o <code>false</code> en caso 
	 * contrario.
	 */
	public static boolean isValidDate(String day, String month, String year, String hour, String minutes)
	{
		// Comprobamos que los parmetros no sean nulos.
		if (isEmpty(day)) return false;
		if (isEmpty(month)) return false;
		if (isEmpty(year)) return false;
		if (isEmpty(hour)) return false;
		if (isEmpty(minutes)) return false;
		// Preparamos la cadena que formar la fecha.
		String date = day + "-" + month + "-" + year + " " + hour + ":" + minutes;
		// Creamos un SimpleDateFormat con el patrn que vamos a seguir.
		SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyy HH:mm");
		// Establecemos que interprete las fechas estrictamente.
		sdf.setLenient(false);
		try
		{
			// Intentamos crear un objeto Date a partir de la informacin que tenemos.
			sdf.parse(date);
			// Si llegamos a este punto la fecha ser correcta.
			return true;
		}
		catch (ParseException pe)
		{
			// Se ha producido una excepcin debido a que la fecha no es correcta.
			return false;
		}
	}
	
	/**
	 * <p>Comprueba si el par&aacute;metro que recibe puede ser interpretado como un n&uacute;mero entero
	 * o no.</p>
	 * 
	 * @param number <code>String</code> a evaluar.
	 * @return <code>true</code> si el par&aacute;metro puede ser evaluado como un entero o <code>false</code>
	 * en caso contrario.
	 */
	public static boolean isInteger(String number)
	{
		// Comprobamos si es nulo.
		if (isEmpty(number)) return false;
		try
		{
			// Intentamos interpretar el string como un entero.
			Integer.parseInt(number);
			// Si llegamos a este punto querr decir que s es un entero.
			return true;
		}
		catch (NumberFormatException nfe)
		{
			// Se lanza una excepcin porque el parmetro no puede ser interpretado como un nmero.
			return false;
		}
	}
	
	/**
	 * <p>Comprueba si el parmetro que se recibe puede ser interpretado como un n&uacute;mero
	 * entero mayor o igual que cero.</p>
	 * 
	 * @param number <code>String</code> a evaluar.
	 * @return <code>true</code> si el par&aacute;metro puede ser como un entero mayor o igual
	 * a certo o <code>false</code> en caso contrario.
	 */
	public static boolean isIntegerPositive(String number)
	{
		// Comprobamos si el parmetro es un entero.
		if (isInteger(number))
		{
			// Retornamos el resultado de compararlo con 0.
			return ((Integer.parseInt(number)) >= 0);
		}
		else
		{
			// No se puede interpretar como un entero.
			return false;
		}
	}
	
	/**
	 * <p>Comprueba si el par&aacute;metro que recibe puede ser interpretado como un n&uacute;mero double
	 * o no.</p>
	 * 
	 * @param number <code>String</code> a evaluar.
	 * @return <code>true</code> si el par&aacute;metro puede ser evaluado como un entero o <code>false</code>
	 * en caso contrario.
	 */
	public static boolean isDouble(String number)
	{
		// Comprobamos si es nulo.
		if (isEmpty(number)) return false;
		try
		{
			// Intentamos interpretar el string como un entero.
			Double.parseDouble(number);
			// Si llegamos a este punto querr decir que s es un entero.
			return true;
		}
		catch (NumberFormatException nfe)
		{
			// Se lanza una excepcin porque el parmetro no puede ser interpretado como un nmero.
			return false;
		}
	}
}
