package edu.uoc.pfc.struts.forms;

import org.apache.struts.action.ActionErrors;

import edu.uoc.pfc.lib.utils.BeanUtils;
import edu.uoc.pfc.lib.utils.ValidationUtils;

/**
 * <code>ActionForm</code> de Struts que ser&aacute; utilizado en todas las transacciones
 * que est&aacute;n relacionadas con la gesti&oacute;n de los planes de estudio dentro de
 * la aplicaci&oacute;n.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class AFCurriculum extends AFBase 
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie que se asignar&aacute; a los objetos de estas clase.</p> */
	private static final long serialVersionUID = 1L;
	/** <p>Tipos de estudios v&aacute;lidos.</p> */
	private static final String[] VALID_CURRICULUM_TYPES = { "DIP", "GRA", "ING", "LIC", "MAS" };
	/** <p>Tipos de asignaturas v&aacute;lidos.</p> */
	private static final String[] VALID_SUBJECT_TYPES = { "OBL", "OPT", "TRO" };
	
	// -------- ATRIBUTOS.
	/** <p>Nombre del Plan de Estudios.</p> */
	private String name;
	/** <p>Tipo de Plan de Estudios.</p> */
	private String type;
	/** <p>Descripci&oacute;n del Plan de Estudios.</p> */
	private String description;
	
	/** <p>Identificador de la asignatura para asignarla al Plan de Estudios.</p> */
	private String subjectId;
	/** <p>Tipo de la Asignatura para asignar al Plan de Estudios.</p> */
	private String subjectType;
	/** <p>C&oacute;digo de la Asignatura.</p> */
	private String code;
	/** <p>N&uacute;mero de cr&eacute;ditos.</p> */
	private String credits;
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>Retorna un objeto <code>ActionErrors</code> con los errores de validaci&oacute;n simples
	 * que hay en los datos del formulario. En el caso de que no haya errores este m&eeacute;todo
	 * retornar&aacute; un objeto <code>ActionErrors</code> vac&iacute;o.</p> 
	 * 
	 * @return <code>ActionErrors</code> con los errores de validaci&oacute;n simples en los datos
	 * del formulario.
	 */
	public ActionErrors validateCreateCurriculum()
	{
		// Borramos los errores que pudiramos tener.
		clearErrors();
		// Validamos el campo "name"
		validateName();
		// Validamos el campo "type"
		validateCurriculumType();
		// Comprobamos si ha habido errores.
		if (this.errors.size() == 0)
		{
			// Al no haber errores eliminamos los espacios en blanco al principio y final de las propiedades.
			BeanUtils.trimProperties(this);
		}
		// Retornamos el objeto ActionErrors.
		return this.errors;
	}
	
	/**
	 * <p>Retorna un objecto <code>ActionErrors</code> con los errores de validaci&oacute;n simples
	 * que hay en los datos del formulario cuando la transacci&oacute;n que se est&aacute; realizando
	 * es la a&ntilde;adir un Asignatura a un Plan de Estudios. En el caso de que no haya errores 
	 * este m&eacute;todo retornar&aacute; un objeto <code>ActionErrors</code> vac&iacute;o.</p>
	 * 
	 * @return <code>ActionErrors</code> con los errores de validaci&oacute;n simples en los datos
	 * del formulario.
	 */
	public ActionErrors validateAddSubject()
	{
		// Borramos los errores que pudiramos tener.
		clearErrors();
		// Validamos la asignatura.
		validateSubjectId();
		// Validamos el tipo de Asignatura.
		validateSubjectType();
		// Validamos el campo "code"
		validateCode();
		// Vadlidamos el campo "credits"
		validateCredits();
		// Comprobamos si ha habido errores.
		if (this.errors.size() == 0)
		{
			// Al no haber errores eliminamos los espacios en blanco al principio y final de las propiedades.
			BeanUtils.trimProperties(this);
		}
		// Retornamos el objeto ActionErrors.
		return this.errors;
	}
	
	// -------- MTODOS PRIVADOS.
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>name</i>. En el caso
	 * de que exista al&uacute;a error ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateName()
	{
		// Validamos que el campo no est vaco.
		if (ValidationUtils.isEmpty(this.name))
		{
			// El campo "name" est vaco. Grabamos un error.
			saveEmptyError("Name", "Nombre");
		}
	}
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>type</i>. En el caso
	 * de que exista alg&uacute;n error este ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateCurriculumType()
	{
		// Validamos que el campo no est vaco.
		if (ValidationUtils.isEmpty(this.type))
		{
			// El campo "type" est vaco. Grabamos un errors.
			saveEmptyError("Type", "Tipo");
		}
		else
		{
			// Tenemos que comprobar que el valor pasado es uno vlido.
			// Definimos un variable local booleana.
			boolean isOk = false;
			// Recorremos los valores de VALID_CURRICULUM_TYPES
			for (String valid : VALID_CURRICULUM_TYPES)
			{
				// Comprobamos si el valor de "type" es igual a "valid"
				if (valid.equals(this.type.trim()))
				{
					// Se ha seleccionado un valor vlido. Actualizamos la variable booleana
					isOk = true;
					// Salimos del bucle.
					break;
				}
			}
			// Comprobamos si se ha encontrado el valor o no.
			if (!isOk)
			{
				// Grabamos el error "debe elegir uno"
				saveSelectOne("Type", "Tipo");
			}
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>code</i>. En el caso de que exista
	 * alg&uacute;n error este ser&aacute; depositado dentro del objeto <code>ActionErrors</code> 
	 * asociado al atributo <i>errors</i>.</p>
	 */
	private void validateCode()
	{
		// Validamos que el campo no est vaco.
		if (ValidationUtils.isEmpty(this.code))
		{
			// El campo "code" est vaco. Grabamos un error.
			saveEmptyError("Code", "Codigo");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>credits</i>. En el caso de que exista
	 * alg&uacute;n error este ser&aacute; depositado dentro del objeto <code>ActionErrors</code> 
	 * asociado al atributo <i>errors</i>.</p>
	 */
	private void validateCredits()
	{
		// Validamos que el campo no est vaco.
		if (ValidationUtils.isEmpty(this.credits))
		{
			// El campo "code" est vaco. Grabamos un error.
			saveEmptyError("Credits", "Creditos");
		}
		else if (!ValidationUtils.isDouble(this.credits))
		{
			// El campo "code" no contiene un nmero "double".
			saveNoNumber("Credits", "Creditos");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>subjectId</i>. En el caso de que exista
	 * alg&uacute;n error este ser&aacute; depositado dentro del objeto <code>ActionErrors</code> 
	 * asociado al atributo <i>errors</i>.</p>
	 */
	private void validateSubjectId()
	{
		// Validamos que el campo no est vaco.
		if (ValidationUtils.isEmpty(this.subjectId))
		{
			// El campo "subjectId" est vaco. Grabamos un error.
			saveEmptyError("SubjectId", "Asignatura");
		}
		else if (!ValidationUtils.isInteger(this.subjectId))
		{
			// El "id" que se ha pasado no es numrico. Indicamos que seleccione uno de la lista.
			saveSelectOne("SubjectId", "Asignatura");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>subjectType</i>. En el caso de que 
	 * exista alg&uacute;n error este ser&aacute; depositado dentro del objeto <code>ActionErrors</code>
	 * asociado al atributo <i>errors</i>.</p>
	 */
	private void validateSubjectType()
	{
		// Validamos que el campo no est vaco.
		if (ValidationUtils.isEmpty(this.subjectType))
		{
			// El campo "subjectType" est vaco. Grabamos un error.
			saveEmptyError("SubjectType", "Tipo de Asignatura");
		}
		else
		{
			// Tenemos que comprobar que el valor pasado es uno vlido.
			// Definimos una variable local booleana.
			boolean isOk = false;
			// Recorremos los valores VALID_SUBJECT_TYPES
			for (String valid : VALID_SUBJECT_TYPES)
			{
				// Comprobamos si el valor de "subjetType" es igual a "valid"
				if (valid.equals(this.subjectType.trim()))
				{
					// Se ha seleccionado un valor vlido. Actualizamos la variable booleana.
					isOk = true;
					// Salimos del bucle.
					break;
				}
			}
			// Comprobamos si se ha producido un error o no.
			if (!isOk)
			{
				// Grabamos el error "debe elegir uno"
				saveSelectOne("SubjectType", "Tipo de Asignatura");
			}
		}
	}
	
	// -------- GETTERS & SETTERS
	public String getName()
	{
		return name;
	}
	public void setName(String name)
	{
		this.name = name;
	}
	public String getType()
	{
		return type;
	}
	public void setType(String type)
	{
		this.type = type;
	}
	public String getDescription()
	{
		return description;
	}
	public void setDescription(String description)
	{
		this.description = description;
	}

	public String getSubjectId()
	{
		return subjectId;
	}

	public void setSubjectId(String subjectId)
	{
		this.subjectId = subjectId;
	}
	public String getSubjectType()
	{
		return subjectType;
	}
	public void setSubjectType(String subjectType)
	{
		this.subjectType = subjectType;
	}
	public String getCode()
	{
		return code;
	}
	public void setCode(String code)
	{
		this.code = code;
	}
	public String getCredits()
	{
		return credits;
	}
	public void setCredits(String credits)
	{
		this.credits = credits;
	}
	
	
}
