package edu.uoc.pfc.struts.forms;

import org.apache.struts.action.ActionErrors;

import edu.uoc.pfc.lib.utils.BeanUtils;
import edu.uoc.pfc.lib.utils.ValidationUtils;

/**
 * <code>ActionForm</code> de Struts que ser&aacute; utilizado en todas las transacciones
 * que est&aacute;n relacionadas con la gesti&oacute;n de los asignaturas dentro de
 * la aplicaci&oacute;n.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class AFSubject extends AFBase 
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie que se asignar&aacute; a los objetos de estas clase.</p> */
	private static final long serialVersionUID = 1L;
	/** <p>Tipos de asignaturas v&aacute;lidos.</p> */
	private static final String[] VALID_SUBJECT_TYPES = { "OBL", "OPT", "TRO" };
	
	// -------- ATRIBUTOS.
	/** <p>Nombre de la trasacci&oacute;n que se va a ejecturar.</p> */
	private String cmd;
	/** <p>Nombre de la asignatura.</p> */
	private String name;
	/** <p>Descripci&oacute;n de la asignatura.</p> */
	private String description;
	
	/** <p>C&oacute;digo de la Asignatura.</p> */
	private String code;
	/** <p>N&uacute;mero de cr&eacute;ditos.</p> */
	private String credits;
	/** <p>Tipo de la Asignatura para asignar al Plan de Estudios.</p> */
	private String subjectType;
	/** <p>Identificador del Plan de Estudios al que se va a asignar la Asignatura.</p> */
	private String curriculumId;
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>M&eeacute;todo que valida la informaci&oacute;n proporcionada por un administrador 
	 * en el formulario de creaci&oacute;n de una asignatura.</p>
	 * 
	 * @return <code>ActionErrors</code> con todos los errores de validaci&oacute;n 
	 * simples sobre los datos del formulario. En el caso de no haber errores retorna
	 * un objeto <code>ActionErrors</code> que est&aacute; vac&iacute;o.
	 */
	public ActionErrors validateCreateSubject()
	{
		// Borramos los errores de validacin simples que pudiese haber.
		clearErrors();
		// Realizamos la validacin simple sobre el campo "name".
		validateName();
		// Comprobamos si ha habido errores.
		if (this.errors.size() == 0)
		{
			// No ha habido errores. Eliminamos espacios en blanco de los atributos.
			BeanUtils.trimProperties(this);
		}
		// Retornamos el objeto ActionErrors
		return this.errors;
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado para realizar la validaci&oacute;n simple de los datos
	 * del formulario cuando la transacci&oacute;n que se est&aacute; realizando es la de asignar una
	 * Asignatura a un Plan de Estudios.</p>
	 * 
	 * @return <code>ActionErrors</code> con todos los errores de validaci&oacute;n 
	 * simples sobre los datos del formulario. En el caso de no haber errores retorna
	 * un objeto <code>ActionErrors</code> que est&aacute; vac&iacute;o.
	 */
	public ActionErrors validateAddCurriculum()
	{
		// Borramos los errores de validacin simples que pudiera haber.
		clearErrors();
		// Validamos el identificador del Plan de Estudios.
		validateCurriculumId();
		// Validamos el tipo de asignatura.
		validateSubjectType();
		// Validamos el cdigo.
		validateCode();
		// Validamos los crditos.
		validateCredits();
		// Comprobamos si ha habido errores.
		if (this.errors.size() == 0)
		{
			// Eliminamos los espacios en blanco al principio y final de los atributos.
			BeanUtils.trimProperties(this);
		}
		// Retornamos el objeto ActionErrors.
		return this.errors;
	}
	
	// -------- MTODOS PRIVADOS.
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>name</i>. En el caso
	 * de que exista al&uacute;a error ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateName()
	{
		// Validamos que el campo no est vaco.
		if (ValidationUtils.isEmpty(this.name))
		{
			// El campo "name" est vaco. Grabamos un error.
			saveEmptyError("Name", "Nombre");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>curriculumId</i>. En el caso de que exista
	 * alg&uacute;n error este ser&aacute; depositado dentro del objeto <code>ActionErrors</code> 
	 * asociado al atributo <i>errors</i>.</p>
	 */
	private void validateCurriculumId()
	{
		// Validamos que el campo no est vaco.
		if (ValidationUtils.isEmpty(this.curriculumId))
		{
			// El campo "subjectId" est vaco. Grabamos un error.
			saveEmptyError("CurriculumId", "Plan de Estudios");
		}
		else if (!ValidationUtils.isInteger(this.curriculumId))
		{
			// El "id" que se ha pasado no es numrico. Indicamos que seleccione uno de la lista.
			saveSelectOne("CurriculumId", "Plan de Estudios");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>subjectType</i>. En el caso de que 
	 * exista alg&uacute;n error este ser&aacute; depositado dentro del objeto <code>ActionErrors</code>
	 * asociado al atributo <i>errors</i>.</p>
	 */
	private void validateSubjectType()
	{
		// Validamos que el campo no est vaco.
		if (ValidationUtils.isEmpty(this.subjectType))
		{
			// El campo "subjectType" est vaco. Grabamos un error.
			saveEmptyError("SubjectType", "Tipo de Asignatura");
		}
		else
		{
			// Tenemos que comprobar que el valor pasado es uno vlido.
			// Definimos una variable local booleana.
			boolean isOk = false;
			// Recorremos los valores VALID_SUBJECT_TYPES
			for (String valid : VALID_SUBJECT_TYPES)
			{
				// Comprobamos si el valor de "subjetType" es igual a "valid"
				if (valid.equals(this.subjectType.trim()))
				{
					// Se ha seleccionado un valor vlido. Actualizamos la variable booleana.
					isOk = true;
					// Salimos del bucle.
					break;
				}
			}
			// Comprobamos si se ha producido un error o no.
			if (!isOk)
			{
				// Grabamos el error "debe elegir uno"
				saveSelectOne("SubjectType", "Tipo de Asignatura");
			}
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>code</i>. En el caso de que exista
	 * alg&uacute;n error este ser&aacute; depositado dentro del objeto <code>ActionErrors</code> 
	 * asociado al atributo <i>errors</i>.</p>
	 */
	private void validateCode()
	{
		// Validamos que el campo no est vaco.
		if (ValidationUtils.isEmpty(this.code))
		{
			// El campo "code" est vaco. Grabamos un error.
			saveEmptyError("Code", "Codigo");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>credits</i>. En el caso de que exista
	 * alg&uacute;n error este ser&aacute; depositado dentro del objeto <code>ActionErrors</code> 
	 * asociado al atributo <i>errors</i>.</p>
	 */
	private void validateCredits()
	{
		// Validamos que el campo no est vaco.
		if (ValidationUtils.isEmpty(this.credits))
		{
			// El campo "code" est vaco. Grabamos un error.
			saveEmptyError("Credits", "Creditos");
		}
		else if (!ValidationUtils.isDouble(this.credits))
		{
			// El campo "code" no contiene un nmero "double".
			saveNoNumber("Credits", "Creditos");
		}
	}
	
	
	// -------- GETTERS & SETTERS
	public String getCmd()
	{
		return cmd;
	}
	public void setCmd(String cmd)
	{
		this.cmd = cmd;
	}
	public String getName()
	{
		return name;
	}
	public void setName(String name)
	{
		this.name = name;
	}
	public String getDescription()
	{
		return description;
	}
	public void setDescription(String description)
	{
		this.description = description;
	}

	public String getCode()
	{
		return code;
	}

	public void setCode(String code)
	{
		this.code = code;
	}
	public String getCredits()
	{
		return credits;
	}
	public void setCredits(String credits)
	{
		this.credits = credits;
	}
	public String getSubjectType()
	{
		return subjectType;
	}
	public void setSubjectType(String subjectType)
	{
		this.subjectType = subjectType;
	}
	public String getCurriculumId()
	{
		return curriculumId;
	}
	public void setCurriculumId(String curriculumId)
	{
		this.curriculumId = curriculumId;
	}

}
