package edu.uoc.pfc.struts.plugins;

import java.io.File;
import java.io.FileInputStream;
import java.util.Properties;
import java.util.StringTokenizer;

import javax.servlet.ServletException;

import org.apache.struts.action.ActionServlet;
import org.apache.struts.action.PlugIn;
import org.apache.struts.config.ModuleConfig;

import edu.uoc.pfc.business.interfaces.IBConfigManager;
import edu.uoc.pfc.lib.constants.CTPresentation;

/**
 * <p>Clase que se utiliza para cargar en el &aacute;mbito de la aplicaci&oacute;n 
 * la informaci&oacute;n de las clases que implementan a las interfaces que la
 * capa de negocio ofrece a la capa de presentaci&oacute;n.</p> 
 * 
 * <p>Esta clase est&aacute; pensada para trabajar con <i>Struts</i>. De hecho 
 * implementa la interfaz <code>PlugIn</code> de tal manera que se cargar&aacute; 
 * est&aacute; informaci&oacute;n al iniciar la aplicaci&oacute;n web para que 
 * est&eacute; disponible para todas las <code>Action</code> de la aplicaci&oacute;n.</p>
 * 
 * <p>La ruta completa de las clases que implementan las interfaces (paquete +
 * clase) se encuentran especificadas dentro del fichero de propiedades (por 
 * defecto este fichero se llama <code>business-factories.properties</code>)
 * del cual podr&aacute; especificarse su ubicaci&oacute; (con respecto a la 
 * ra&iacute;z del contexto) como el par&aacute;metro del <code>PlugIn</code> 
 * <i>properties-file</i>.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 *
 */
public class PFCPlugIn implements PlugIn {
	
	// -------- CONSTANTES.
	/** <p>Ruta por defecto para <code>business-factories.properties</code>.</p> */
	public static final String DEFAULT_PATH = "/WEB-INF/business-factories.properties";
	
	// -------- ATRIBUTOS.
	/** <p>Atributo para el par&aacute;metro del PlugIn <i>properties-file</i>.</p> */
	protected String factories;
	
	// -------- GETTERS & SETTERS.
	public String getFactories() 
	{
		return factories;
	}
	public void setFactories(String factories) 
	{
		this.factories = factories;
	}
	
	// -------- MTODOS PBLICOS.
	@Override
	public void destroy() 
	{
	}

	@Override
	public void init(ActionServlet servlet, ModuleConfig config)
			throws ServletException 
	{
		// Guardamos en el mbito de aplicacin la informacin de las factoras.
		loadFactories(servlet);
	}
	
	// -------- MTODOS PROTEGIDOS
	/**
	 * <p>M&eacute;todo que se encargar&aacute; de guardar la informaci&oacute;n de las diferentes
	 * clases que implementan las factoras que implementan a las interfaces de la capa de negocio
	 * dentro del &aacute;mbito de aplicaci&oacute;n.</p>
	 * 
	 * @param servlet <code>ActionServlet</code> de Struts que ha llamado al PlugIn.
	 * @throws ServletException si se produce un error a la hora de leer la informaci&oacute;n de 
	 * las factorias de la capa de negocio.
	 */
	protected void loadFactories(ActionServlet servlet) throws ServletException
	{
		// Comprobamos si se ha especificado el parmetro 'properties-file'.
		if (this.factories == null)
		{
			// No se ha especificado. Asignamos el valor por defecto.
			this.factories = DEFAULT_PATH;
		}
		// Path real en el servidor para la aplicacin.
		File file = new File(servlet.getServletContext().getRealPath("/") + this.factories);
		// Comprobamos la existencia del fichero y que se puede leer.
		if (file.exists() && file.canRead())
		{
			try {
				// El fichero existe. Lo cargamos dentro de un objeto Properties.
				Properties properties = new Properties();
				properties.load(new FileInputStream(file));
				// Leemos la lista de interfaces.
				if (properties.containsKey("interfaces"))
				{
					// Leemos la lista de todas las interfaces que se deben implementar.
					StringTokenizer st = new StringTokenizer(properties.getProperty("interfaces"), ",");
					// Recorremos la lista de todas las interfaces.
					while (st.hasMoreTokens())
					{
						// Interfaz a buscar.
						String interfaceName = st.nextToken().trim();
						// Siguiente interfaz a comprobar.
						String property = "factory." + interfaceName;
						// Comprobamos la existencia de la clave en el fichero de propiedades.
						if (properties.containsKey(property))
						{
							// Existe la clave. Obtenemos la clase que la implementa.
							String interfaceClass = properties.getProperty(property);
							try
							{
								// Ahora comprobamos la existencia de la clase.
								Class.forName(interfaceClass);
								// Comprobamos si se ha definido una clave para almacenar
								// la clase en el mbito de aplicacin.
								if (CTPresentation.IB_ACCOUNT_MANAGER_KEY.equals(interfaceName))
								{
									// Guardamos la informacin de la clase en el mbito
									// de aplicacin para que pueda ser utilizada por todas
									// las Actions.
									servlet.getServletContext().setAttribute(CTPresentation.IB_ACCOUNT_MANAGER_KEY, 
											interfaceClass);
								}
								else if (CTPresentation.IB_CONFIG_MANAGER_KEY.equals(interfaceName))
								{
									// Guardamos la informacin de la clase que implementa la 
									// interfaz IBConfigManager en el mbito de aplicacin.
									servlet.getServletContext().setAttribute(CTPresentation.IB_CONFIG_MANAGER_KEY, 
											interfaceClass);			
								}
								else if (CTPresentation.IB_CURRICULUM_MANAGER_KEY.equals(interfaceName))
								{
									// Guardamos la informacin de la clase que implementa la 
									// interfaz IBCurriculumManager en el mbito de aplicacin.
									servlet.getServletContext().setAttribute(CTPresentation.IB_CURRICULUM_MANAGER_KEY, 
											interfaceClass);
								}
								else if (CTPresentation.IB_SUBJECT_MANAGER_KEY.equals(interfaceName))
								{
									// Guardamos la informacin de la clase que implementa la
									// interfaz IBSubjectManager en el mbito de aplicacin.
									servlet.getServletContext().setAttribute(CTPresentation.IB_SUBJECT_MANAGER_KEY, 
											interfaceClass);
								}
								else if (CTPresentation.IB_QUESTION_BANK_MANAGER_KEY.equals(interfaceName))
								{
									// Guardamos la informacin de la clase que implementa la
									// interfaz IBQuestionManager en el mbito de aplicacin.
									servlet.getServletContext().setAttribute(CTPresentation.IB_QUESTION_BANK_MANAGER_KEY, 
											interfaceClass);
								}
								else if (CTPresentation.IB_TEST_MANAGER_KEY.equals(interfaceName))
								{
									// Guardamos la informacin de la clase que implementa 
									// la interfaz IBTestManager en el mbito de aplicacin.
									servlet.getServletContext().setAttribute(CTPresentation.IB_TEST_MANAGER_KEY, 
											interfaceClass);
								}
								else if (CTPresentation.IB_CLASSROOM_MANAGER_KEY.equals(interfaceName))
								{
									// Guardamos la informacin de la clase que implementa
									// la interfaz IBClassroomManager en el mbito de aplicacin.
									servlet.getServletContext().setAttribute(CTPresentation.IB_CLASSROOM_MANAGER_KEY, 
											interfaceClass);
								}
								else 
								{
									// No se ha definido una clave para guardar la interfaz. 
									throw new ServletException("No se ha definido una clave en [" + 
											CTPresentation.class + "] para la interfaz '" +
											interfaceName + "'");
								}
							}
							catch (ClassNotFoundException cnfe)
							{
								// No se ha encontrado la clase que implementa a la
								// interfaz por lo que lanzamos una excepcin.
								throw new ServletException("No se puede encontrar la clase asociada a " +
										"la propiedad '" + property + "' [" + interfaceClass + "]");
							}
						}
						
						else 
						{
							// Falta la propiedad que se usa para definir la clase que
							// implementa una interfaz de negocio. Lanzamos una excepcin.
							throw new ServletException("No existe la clave '" + 
									property + "' dentro del fichero " + this.factories);
						}
					}
					// Una vez hemos obtenido todas las interfaces solamente nos queda almacenar la informacin
					// de la configuracin activa en el mbito de la aplicacin.
					// Obtenemos la configuracin activa de la aplicacin y la guardamos en la sesin.
					// No se ha obtenido la configuracin activa. Obtenemos la clase que implementa
					// a la interfaz IBConfigManager.
					String className = (String) servlet.getServletContext().getAttribute(CTPresentation.IB_CONFIG_MANAGER_KEY);
					// Creamos una nueva instancia de esa clase.
					IBConfigManager configManager = (IBConfigManager) Class.forName(className).newInstance();
					// Obtenemos la informacin de la configuracin activa.
					servlet.getServletContext().setAttribute(CTPresentation.ACTIVE_CONFIGURATION_KEY, configManager.getConfiguration());
				}
				else 
				{
					// No existe la clave "interfaces" la cual es obligatoria
					// ya que en ella se definen todas las interfaces de negocio
					// que debern ser implementadas.
					throw new ServletException("No existe la clave 'interfaces' dentro del fichero " 
							+ this.factories);
				}
			} catch (Exception ex) {
				// Cualquier excepcin que se produzca en el cdigo la
				// encapsulamos dentro de una ServletException y la lanzamos.
				throw new ServletException(ex.getMessage());
			}
		}
		else 
		{
			// No existe el fichero o no se puede leer. Finalizamos la
			// ejecucin de la aplicacin lanzando una ServletException.
			throw new ServletException("No existe o no se puede leer el fichero " + this.factories);
		}
	}

}
